/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CApplication.h

		Interface for the CApplication Class
	
 ******************************************************************************/

#ifndef _CAPPLICATION_H_
#define _CAPPLICATION_H_

#include "CObject.h"

#include <fstream>

class CStringReader;
class CEngineCaller;
class CCommandRecord;
class CReportRecord;
class CReporter;
class CWriter;
class CReader;
class CProgress;

#include "CApplicationHelper.h"
		
class CApplication : public CObject {
	protected:
		char					mPrompt[256];
		CEngineCaller*			mEngineCaller;
		
		CReporter				*mResultsReporter;
		CReporter				*mLogReporter;
		CReporter				*mErrorReporter;
		
		CProgress				*mProgress;
		
		CStringReader			*mReader;
		
		CApplicationHelper		*mHelper;

	public:
								CApplication();
		virtual 				~CApplication();
		
		virtual void			writeTitle();

		virtual void			setApplicationHelper(CApplicationHelper *inHelper);

		virtual void			setResultsReporter(CReporter* inReporter);
		virtual void			setLogReporter(CReporter* inReporter);
		virtual void			setErrorReporter(CReporter* inReporter);
		
		virtual void			setProgress(CProgress *inProgress);

		virtual void			run();

		virtual CReportRecord*	processFile(CReader *inReader, bool inProcessCommands) = 0;
		virtual bool			processCommand(char *inCommandString, bool inHandleErrors);
		
	protected:
		virtual CCommandRecord*	parseCommand(char *inCommandString);

	public:
		virtual CReporter*		createLogReporter(CWriter *inWriter, bool inUseHTML);
		virtual void			closeLogReporter();

		virtual CWriter*		createFileWriter(CFileRequest *ioFileRequest);
		virtual CReader*		createFileReader(CFileRequest *ioFileRequest);

};

#endif /* _CAPPLICATION_H_ */
