/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CAxis.h
 
		CAxis Class
	 
 ******************************************************************************/

#ifndef _CAXIS_H_
#define _CAXIS_H_

enum AxisFlag {
	kAtZero,
	kAtValue,
	kAtMajorTick,
	kAtMajorTickPlus,
	kAtMinorTick,
	kAtMinorTickPlus
};

enum DivisionFlag {
	kUnit,
	kHalfs,
	kQuarters,
	kFifths
};
		
class CDoubleArray;

class CAxis {

	//====<<  FIELDS  >>=============================================================

protected:												
	bool				isCalibrated;
	
	// The minimum and maximum values of the data
	double minData, maxData;

	// A small correction to cope with rounding error and precision problems
	double epsilon;

	// The number of major ticks and minor ticks within them
	int numMajorTicks, numMinorTicks; // calculated automatically
	
	// The prefered minimum number of ticks
	int prefNumMajorTicks, prefNumMinorTicks; // set manually
	
	// Flags using the above constants
	AxisFlag minAxisFlag, maxAxisFlag;
	
	// The distance between major ticks and minor Ticks
	double majorTick, minorTick; // calculated automatically
	
	// The value of the first and last major tick
	double minTick, maxTick; // calculated automatically
	
	// The value of the beginning and end of the axis
	double minAxis, maxAxis;
	
	// Flags to specify descrete divisions
	bool isDiscrete;
	
	// Flags to specify that the first tick and last tick should have labels.
	// It is up to the AxisPanel to do something about this.
	bool labelFirst, labelLast;
	
	// Used internally
	DivisionFlag fraction;
	
public:
	//====<<  METHODS  >>============================================================
		
	//----<<  INITIALIZATION & DISPOSAL  >>------------------------------------------

 						CAxis( );
 						CAxis(AxisFlag inMinAxisFlag, AxisFlag inMaxAxisFlag);
 						CAxis(AxisFlag inMinAxisFlag, AxisFlag inMaxAxisFlag, 
 								bool inIsDiscrete);

protected:
	void				InitAxis(AxisFlag inMinAxisFlag, AxisFlag inMaxAxisFlag, 
									bool inIsDiscrete);

	//----<<  ACCESSING  >>----------------------------------------------------------
		
public:
	virtual	bool		GetIsDiscrete() { return isDiscrete; }
	virtual	bool		GetUseScientific() { return false; }
	
	virtual	void		SetRange(CDoubleArray *inData);
	virtual	void		AddRange(CDoubleArray *inData);
	virtual	void		SetRange(double inMinValue, double inMaxValue);
	virtual	void		AddRange(double inMinValue, double inMaxValue);
	
	virtual	void		SetLabelFirst(bool inLabelFirst) { labelFirst=inLabelFirst; };
	virtual	void		SetLabelLast(bool inLabelLast) { labelLast=inLabelLast; };
	
	virtual double 		Transform(double inValue);
	virtual double 		ScaleValue(double inValue);
	
	virtual	bool		GetLabelFirst();
	virtual	bool		GetLabelLast();
	
	virtual int 		GetNumMajorTicks();
	virtual int 		GetNumMinorTicks(int majorTickNo);
	
	virtual double 		GetMajorTick(int majorTickNo);
	virtual double 		GetMinorTick(int minorTickNo, int majorTickNo);

	virtual double 		GetMinAxis();
	virtual double 		GetMaxAxis();

	virtual double 		GetMinData() { return minData; }
	virtual double 		GetMaxData() { return maxData; }

	//----<<  CALIBRATING  >>--------------------------------------------------------
	
	virtual	void		Invalidate() { isCalibrated=false; };
	
protected:
	virtual	void		Calibrate();
	
	virtual	void		SetMinTick();
	virtual	void		SetMaxTick();
	virtual	void		SetMajorTick();
	virtual	void		SetMinorTick();
	virtual	void		HandleAxisFlags();
	
};

#endif /* _CAXIS_H_ */
