/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CCommandRecord.h

		Interface for the CCommandRecord Class
	
 ******************************************************************************/

#ifndef _CCOMMAND_RECORD_H_
#define _CCOMMAND_RECORD_H_

#include "CObject.h"

#include "CExceptions.h"

//: A base class that provides exceptions for the file in/out
//  These are throw by the file using classes (MEStream, MEReader and MEWriter) 
//  and their descendants.
class CFailedConversion : public CException { 
	public:
		CFailedConversion() : CException() { };
		CFailedConversion(const char *inMsg) : CException(inMsg) { };
		CFailedConversion(const char *inMsg1, const char *inMsg2) : CException(inMsg1, inMsg2) { };
};


class SubCommandRecord : public CObject {
	protected:
		char 				*mLabel;
		char 				*mValue;
		
	public:
							SubCommandRecord(char *inLabel, char *inValue);
							~SubCommandRecord();
		
		char 				*getLabel() { return mLabel; }
		char 				*getValue() { return mValue; }
		bool 				getHasValue() { return (mValue != NULL? true : false); }
		
		bool 				compareLabel(char *inString, int inMinLen, bool inCaseSensitive = true);
		bool 				compareValue(char *inString, int inMinLen, bool inCaseSensitive = true);

	protected:
		bool 				compareTokens(char *inToken, char *inString, int inMinLen, bool inCaseSensitive);

	public:
		double 				getValueAsDouble();
		int	 				getValueAsInteger();
		bool 				getValueAsBoolean();
		
		int 				getValueAsOption(char **inList, int *inMinLength, int inNumItems, bool inCaseSensitive);

		int 				getNumValuesInList();
		int 				getValueAsDoubleList(double *outList, int inMaxItems);
		int 				getValueAsIntegerList(int *outList, int inMaxItems);
		int 				getValueAsStringList(char **outList, int inMaxItems);

		void 				getValueAsRange(int &outFirst, int &outLast);
};

class CCommandRecord : public SubCommandRecord {
	public:
		static const int 	kMaxSubCommands = 256;
		
	protected:
		int 				numSubCommands;
		SubCommandRecord 	*subCommands[kMaxSubCommands];

	public:
							CCommandRecord(char *inLabel, char *inValue);
		
							~CCommandRecord();
		
		void				addSubCommand(char *inLabel, char *inValue);
		
		int					getNumSubCommands() { return numSubCommands; }
		
		SubCommandRecord	*getSubCommand(int index);
		
		char				*getSubCommandLabel(int index);
		char				*getSubCommandValue(int index);
};

#endif /* _CCOMMAND_RECORD_H_ */
