/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CDataSeries.cpp
 
		CDataSeries Class
	 
 ******************************************************************************/

#include <stdio.h>

#include "CDataSeries.h"

#include "CDoubleArray.h"

#pragma mark === Construction & Destruction ===

// ---------------------------------------------------------------------------------
//		 CDataSeries()
// ---------------------------------------------------------------------------------

CDataSeries::CDataSeries( )
{
	mXData = mYData = NULL;
	mIsDataOwner = false;

	mYHigh = mYLow = NULL;
	mIsLimitsOwner = false;
	
	mHasLimits = false;
}


// ---------------------------------------------------------------------------------
//		 ~CDataSeries()
// ---------------------------------------------------------------------------------

CDataSeries::~CDataSeries( )
{
	if (mIsDataOwner) {
		if (mXData!=NULL)
			delete mXData;
		if (mYData!=NULL)
			delete mYData;
	}
	if (mIsLimitsOwner) {
		if (mYHigh!=NULL)
			delete mYHigh;
		if (mYLow!=NULL)
			delete mYLow;
	}
}

#pragma mark === Accessing ===

//-------------------------------------------------------------------------------------
// 		CDataSeries::SetData
//-------------------------------------------------------------------------------------

void CDataSeries::SetData(CDoubleArray *inXData, CDoubleArray *inYData, bool isOwner)
{
	mXData = inXData;
	mYData = inYData;
	mIsDataOwner = isOwner;
}

//-------------------------------------------------------------------------------------
// 		CDataSeries::SetLimits
//-------------------------------------------------------------------------------------

void CDataSeries::SetLimits(CDoubleArray *inYHigh, CDoubleArray *inYLow, bool isOwner)
{
	mYHigh = inYHigh;
	mYLow = inYLow;
	mIsLimitsOwner = isOwner;
	mHasLimits = true;
}


//-------------------------------------------------------------------------------------
// 		CDataSeries::GetXData
//-------------------------------------------------------------------------------------

CDoubleArray *CDataSeries::GetXData()
{
	if (mXData==NULL) {
		mXData = new CDoubleArray();
		mIsDataOwner = true;
	}
		
	return mXData;
}


//-------------------------------------------------------------------------------------
// 		CDataSeries::GetYData
//-------------------------------------------------------------------------------------

CDoubleArray *CDataSeries::GetYData()
{
	if (mYData==NULL) {
		mYData = new CDoubleArray();
		mIsDataOwner = true;
	}
		
	return mYData;
}

//-------------------------------------------------------------------------------------
// 		CDataSeries::GetYHigh
//-------------------------------------------------------------------------------------

CDoubleArray *CDataSeries::GetYHigh()
{
	if (mYHigh==NULL) {
		mYHigh = new CDoubleArray();
		mIsLimitsOwner = true;
		SetupLimits();
	}
		
	return mYHigh;
}


//-------------------------------------------------------------------------------------
// 		CDataSeries::GetYLow
//-------------------------------------------------------------------------------------

CDoubleArray *CDataSeries::GetYLow()
{
	if (mYLow==NULL) {
		mYLow = new CDoubleArray();
		mIsLimitsOwner = true;
		SetupLimits();
	}
		
	return mYLow;
}


//-------------------------------------------------------------------------------------
// 		CDataSeries::Sort
//-------------------------------------------------------------------------------------

void CDataSeries::Sort(SortOrder inSortOrder, SortData inSortData)
{
	CDoubleArray *array1, *array2;
	
	if (inSortData = kSortXData) {
		array1 = mXData;
		array2 = mYData;
	} else {
		array1 = mXData;
		array2 = mYData;
	}

	if (array1->getNumValues() > 1) {
	
			// Shellsort Algorithm
		
		int	hh = 1;						// Find starting "h" value
		int	stopH = array1->getNumValues() / 9;
		while (hh < stopH) {
			hh = 3 * hh + 1;
		}
		
		double tmp1, tmp2;
		
		int direction = (inSortOrder == kSortAscending ? 1 : -1);
		
		for ( ; hh > 0; hh /= 3) {
				
			for (int step = hh + 1; step <= array1->getNumValues() ; step++) {
				tmp1 = array1->getValue(step - 1);
				tmp2 = array2->getValue(step - 1);
				
				long i;
				for (i = step - hh; i > 0; i -= hh) {
					if ((tmp1 > array1->getValue(i-1) ? 1 : -1) * direction > 0) {
						break;
					}
					array1->setValue(i + hh - 1, array1->getValue(i-1));
					array2->setValue(i + hh - 1, array2->getValue(i-1));
				}
					
				array1->setValue(i + hh - 1, tmp1);
				array2->setValue(i + hh - 1, tmp2);
			}
		}
	}
}


//-------------------------------------------------------------------------------------
// 		CDataSeries::SetupLimits
//-------------------------------------------------------------------------------------

void CDataSeries::SetupLimits()
{
	mHasLimits = true;
}



