/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CDataSeries.h
 
		CDataSeries Class
	 
 ******************************************************************************/

#ifndef _CDATASERIES_H_
#define _CDATASERIES_H_

class CDoubleArray;

class CDataSeries {
public:
	enum SortOrder {
		kSortAscending,
		kSortDescending
	};

	enum SortData {
		kSortXData,
		kSortYData
	};

	//====<<  FIELDS  >>=============================================================

protected:												
	CDoubleArray		*mXData, *mYData;
	CDoubleArray		*mYHigh, *mYLow;
	bool				mIsDataOwner, mIsLimitsOwner, mHasLimits;

public:
	//====<<  METHODS  >>============================================================
		
	//----<<  INITIALIZATION & DISPOSAL  >>------------------------------------------

 						CDataSeries( );
 	virtual				~CDataSeries( );

	//----<<  ACCESSING  >>----------------------------------------------------------
		
	virtual	bool		GetHasLimits() { return mHasLimits; }
	
	virtual	void		SetData(CDoubleArray *inXData, CDoubleArray *inYData, bool isOwner=true);
	virtual	void		SetLimits(CDoubleArray *inYHigh, CDoubleArray *inYLow, bool isOwner=true);

	virtual	CDoubleArray *GetXData();
	virtual	CDoubleArray *GetYData();

	virtual	CDoubleArray *GetYHigh();
	virtual	CDoubleArray *GetYLow();

	virtual void		Sort(SortOrder inSortOrder, SortData inSortData);
		
protected:												
	virtual	void		SetupLimits();
};

#endif /* _CDATASERIES_H_ */
