/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CDataTextReader.cpp

				CDataTextReader Class
				
 ******************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "CExceptions.h"
#include "CDataTextReader.h"


#pragma mark === CDataTextReader ===

// ---------------------------------------------------------------------------------
//		 CDataTextReader
// ---------------------------------------------------------------------------------

CDataTextReader::CDataTextReader() : CReader()
{
	setBuffer(NULL, 0);
}

CDataTextReader::CDataTextReader(char *inBuffer, int inLength) : CReader()
{
	setBuffer(inBuffer, inLength);
}

// ---------------------------------------------------------------------------------
//		 ~CDataTextReader
// ---------------------------------------------------------------------------------

CDataTextReader::~CDataTextReader()
{
}

// ---------------------------------------------------------------------------------
//		 CDataTextReader::setBuffer
// ---------------------------------------------------------------------------------

void CDataTextReader::setBuffer(char *inBuffer, int inLength)
{
	buffer=inBuffer;
	bufLen=inLength;
	bufPos=0;
	markPos=0;
}

#pragma mark === Low Level Reading ===

// ---------------------------------------------------------------------------------
//		 CDataTextReader::markPosition
// ---------------------------------------------------------------------------------

// Store the current position in the file. 

void CDataTextReader::markPosition()
{
	markPos=bufPos;
}

// ---------------------------------------------------------------------------------
//		 CDataTextReader::restorePosition
// ---------------------------------------------------------------------------------

// Reset the file to the stored position. 

void CDataTextReader::restorePosition()
{
	bufPos=markPos;
}

// ---------------------------------------------------------------------------------
//		 CDataTextReader::readChar
// ---------------------------------------------------------------------------------

// Returns the next character from the file. 

char CDataTextReader::readChar()
{
	if (bufPos>=bufLen)
		throw CEndOfFile();

	return buffer[bufPos++];
};


// ---------------------------------------------------------------------------------
//		 CDataTextReader::nextChar
// ---------------------------------------------------------------------------------

// Returns the next character from the file without moving the file pointer on. 

char CDataTextReader::nextChar()
{	
	return buffer[bufPos];
};


// ---------------------------------------------------------------------------------
//		 CDataTextReader::unreadChar
// ---------------------------------------------------------------------------------

// Puts a character back in to the buffer. Ignores the value of ch.

void CDataTextReader::unreadChar(char ch) 
{
	bufPos--;
}




