/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CDataTextWriter.cpp

				CDataTextWriter Class
				
 ******************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "CExceptions.h"
#include "CDataTextWriter.h"


#pragma mark === CDataTextWriter ===

// ---------------------------------------------------------------------------------
//		 CDataTextWriter
// ---------------------------------------------------------------------------------

CDataTextWriter::CDataTextWriter() : CWriter()
{
	buffer=NULL;
	setIsCounting(true);
}

CDataTextWriter::CDataTextWriter(char *inBuffer, int inLength) : CWriter()
{
	buffer=NULL;
	setBuffer(inBuffer, inLength);
}

// ---------------------------------------------------------------------------------
//		 ~CDataTextWriter
// ---------------------------------------------------------------------------------

CDataTextWriter::~CDataTextWriter()
{
}

// ---------------------------------------------------------------------------------
//		 CDataTextWriter::setBuffer
// ---------------------------------------------------------------------------------

void CDataTextWriter::setBuffer(char *inBuffer, int inLength)
{
	buffer=inBuffer;
	bufLen=inLength;
	bufPos=0;
	setIsCounting(false);
}

// ---------------------------------------------------------------------------------
//		 CDataTextWriter::setIsCounting
// ---------------------------------------------------------------------------------

void CDataTextWriter::setIsCounting(bool inCounting)
{
	isCounting=inCounting;
	if (isCounting)
		writeCount=0;
}

// ---------------------------------------------------------------------------------
//		 CDataTextWriter::nullTerminate
// ---------------------------------------------------------------------------------

void CDataTextWriter::nullTerminate()
{
	if (bufPos>=bufLen)
		throw CWriteError("End of buffer reached");

	buffer[bufPos] = '\0';
}

#pragma mark === Low Level Writing ===

// ---------------------------------------------------------------------------------
//		 CDataTextWriter::writeChar
// ---------------------------------------------------------------------------------

void CDataTextWriter::writeChar(char ch)
{
	if (isCounting) {
		writeCount++;
		return;
	}
	
	if (bufPos>=bufLen)
		throw CWriteError("End of buffer reached");
		
	buffer[bufPos] = ch;
	bufPos++;
}


// ---------------------------------------------------------------------------------
//		 CDataTextWriter::writeText
// ---------------------------------------------------------------------------------

void CDataTextWriter::writeText(const char *st, int len)
{
	for (int i=0; i<len; i++)
		writeChar(st[i]);
}


// ---------------------------------------------------------------------------------
//		 CDataTextWriter::writeInteger
// ---------------------------------------------------------------------------------

void CDataTextWriter::writeInteger(int d)
{
	char str[64];
	sprintf(str, "%ld", d);
	writeText(str, strlen(str));
}


// ---------------------------------------------------------------------------------
//		 CDataTextWriter::writeInteger
// ---------------------------------------------------------------------------------

void CDataTextWriter::writeInteger(int d, int width)
{
	char pre[16];
	char str[64];
	sprintf(pre, "%%%ldld", width);
	sprintf(str, pre, d);
	writeText(str, strlen(str));
}


// ---------------------------------------------------------------------------------
//		 CDataTextWriter::writeDouble
// ---------------------------------------------------------------------------------


void CDataTextWriter::writeDouble(double f)
{
	char str[64];
	sprintf(str, "%lf", f);
	writeText(str, strlen(str));
}


// ---------------------------------------------------------------------------------
//		 CDataTextWriter::writeDouble
// ---------------------------------------------------------------------------------

void CDataTextWriter::writeDouble(double f, int precision, bool scientific)
{
	char fmt[16];
	char str[64];
	
	if (scientific) {
		if (precision < 0)
			sprintf(fmt, "%%lE");
		else
			sprintf(fmt, "%%.%ldlE", precision);
	} else {
		if (precision < 0)
			sprintf(fmt, "%%lf");
		else
			sprintf(fmt, "%%.%ldlf", precision);
	}
	sprintf(str, fmt, f);
	writeText(str, strlen(str));
}


