/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CDoubleArray.h

		Interface for the CDoubleArray Class
					
 ******************************************************************************/
//! author="Andrew Rambaut"
//! group=Core
 
 
#ifndef _CDOUBLE_ARRAY_H_
#define _CDOUBLE_ARRAY_H_

#include "CObject.h"

/******************************************************************************
	class CDoubleArray
 ******************************************************************************/
#pragma mark CDoubleArray class

//:	Implements a dynamically resizing list of doubles

class CDoubleArray : public CObject {
	protected:
		int 				numValues; 	//: The number of doubles in the list
		int					capacity;	//: The capacity of the list
		int					increment;	//: The amount by which capacity increments
		int					iterator;	//: Used to iterate through the list

		double		 		*values;	//: An array of doubles

	public:
							CDoubleArray(int inInitial = 50, 
										int inIncrement = 20);	//: The constructor
	 	virtual				~CDoubleArray();	//: The destructor		
	 	
		virtual void		setInitial(int inInitial, int inIncrement = 20);
		//: Set the initial capacity and (optionally) the increment

		int					getNumValues() { return numValues; };
		//: Returns the number of doubles in the list
		
		virtual double*		getValues() { return values; };
		//: Returns the list of double pointers
		
		virtual void		setCapacity(int inCapacity);
		//: Set the capacity
		
		virtual int			getCapacity() { return capacity; };
		//: Returns the capacity

		virtual int			getIncrement() { return increment; };
		//: Sets the amount that capacity increments by
		virtual void		setIncrement(int inIncrement);
		//: Returns the increment
		
		virtual double		getValue(int index);
		//: Returns the double that is at an index.
		
		virtual double		getSum();
		//: Returns the sum of all the doubles

		virtual double		getMaxValue();
		//: Returns the maximum double in the list
		
		virtual double		getMinValue();
		//: Returns the minimum double in the list
		
		virtual void		setValue(int index, double inValue);
		//: Sets the double at an index

		virtual int			add(double inValue);
		//: Adds an double to the end of the list
		
		virtual void		insert(int index, double inValue);
		//: Inserts an double into the list at a position
		
		virtual void		swap(int index1, int index2);
		//: Swaps an object at index1 with that at index2
		
		virtual double		remove(int index);
		//: Removes the double at index and moves others up 
		
		virtual void		removeAll();
		//: Removes all the doubles in the list

		virtual void		sort(bool inAscending=true);
		//: Sorts the objects in the list 
		
		virtual double		operator[](int index) { return getValue(index); }

	protected:
		virtual void		checkCapacity(int requiredCapacity);
		//: Checks that the integer list has enough space and resizes it as necessary
};

#endif /* _CDOUBLE_ARRAY_H_ */
