/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CEngineCaller.cpp

				CEngineCaller Class
	
 ******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "CExceptions.h"

#include "CApplication.h"
#include "CEngineCaller.h"
#include "CCommandRecord.h"
#include "CReportRecord.h"

// ---------------------------------------------------------------------------------
//		 CEngineCaller::CEngineCaller
// ---------------------------------------------------------------------------------

CEngineCaller::CEngineCaller(CApplication *inApplication)
{
	ThrowIfNull(inApplication);
	
	mApplication = inApplication;
	mApplication->retain();
}

// ---------------------------------------------------------------------------------
//		 CEngineCaller::~CEngineCaller
// ---------------------------------------------------------------------------------

CEngineCaller::~CEngineCaller()
{
	mApplication->release();
}

// ---------------------------------------------------------------------------------
//		 CEngineCaller::execute
// ---------------------------------------------------------------------------------

CReportRecord* CEngineCaller::execute(CCommandRecord *inCommand, bool &outQuit)
{
	CReportRecord *report = NULL;
	
	outQuit = false;
	
	if (inCommand->compareLabel("RUN", 3) ) {
		report = doRunFile(inCommand);
	} else if (inCommand->compareLabel("LOAD", 4) ) {
		report = doLoadFile(inCommand);
	} else if (inCommand->compareLabel("HELP", 4)) {
		report = doHelp(inCommand);
	} else if (inCommand->compareLabel("LOG", 3)) {
		report = doLog(inCommand);
	} else if (inCommand->compareLabel("QUIT", 4)) {
		if (doQuit(inCommand))
			outQuit = true;
	} else
		report = executeCommand(inCommand);
		
	return report;
}

// ---------------------------------------------------------------------------------
//		 CEngineCaller::doRunFile
// ---------------------------------------------------------------------------------

CReportRecord *CEngineCaller::doRunFile(CCommandRecord* inCommand) 
{
	CFileRequest ioFileRequest;
	
	if (inCommand->getHasValue())
		strcpy(ioFileRequest.mFileName, inCommand->getValue());
	else
		strcpy(ioFileRequest.mFileName, "");
		
	CReader *reader = mApplication->createFileReader(&ioFileRequest);

	if (strlen(ioFileRequest.mFileName) == 0)
		throw CMissingField("Filename missing");

	CReportRecord *report = NULL;
	
	if (reader != NULL)
		report = mApplication->processFile(reader, true);
	
	return report;	
}

// ---------------------------------------------------------------------------------
//		 CEngineCaller::doLoadFile
// ---------------------------------------------------------------------------------

CReportRecord *CEngineCaller::doLoadFile(CCommandRecord* inCommand) 
{
	CFileRequest ioFileRequest;
	
	if (inCommand->getHasValue())
		strcpy(ioFileRequest.mFileName, inCommand->getValue());
	else
		strcpy(ioFileRequest.mFileName, "");
		
	CReader *reader = mApplication->createFileReader(&ioFileRequest);

	CReportRecord *report = NULL;
	
	if (reader != NULL)
		report = mApplication->processFile(reader, false);
	
	return report;	
}

// ---------------------------------------------------------------------------------
//		 CEngineCaller::doLog
// ---------------------------------------------------------------------------------

CReportRecord *CEngineCaller::doLog(CCommandRecord* inCommand) 
{
	CFileRequest ioFileRequest;
	CReportRecord* report = new CReportRecord();
	
	if (inCommand->getValue() == NULL) { // Closing log file
		mApplication->closeLogReporter();

		CParagraphElement *para = new CParagraphElement();
		para->add("Closed log file.");
		report->add(para);
	} else {
		SubCommandRecord *sub;
		
		ioFileRequest.mFormat = 0;
		for (int i = 0; i < inCommand->getNumSubCommands(); i++) {
		
			sub = inCommand->getSubCommand(i);

			if (sub->compareLabel("FORMAT", 1)) {
		
				if (sub->compareValue("TEXT", 1, false))
					ioFileRequest.mFormat = 0;
				else if (sub->compareValue("HTML", 1, false))
					ioFileRequest.mFormat = 1;
				else {
					throw CMissingField("Unknown log file format");
				}
			}
		}
	
		strcpy(ioFileRequest.mFileName, inCommand->getValue());
		ioFileRequest.mAskAppend = true;
		ioFileRequest.mAskFormat = true;

		ioFileRequest.mFormatNames[0] = "Text";
		ioFileRequest.mFormatNames[1] = "HTML";
		ioFileRequest.mNumFormats = 2;
		
		CWriter *writer = mApplication->createFileWriter(&ioFileRequest);
		CReporter *reporter = mApplication->createLogReporter(writer, ioFileRequest.mFormat == 1);
		mApplication->setLogReporter(reporter);
	}
	
	return report;	
}

// ---------------------------------------------------------------------------------
//		 CEngineCaller::doHelp
// ---------------------------------------------------------------------------------

CReportRecord* CEngineCaller::doHelp(CCommandRecord *inCommand)
{
	CReportRecord *report = new CReportRecord("Application Help");
	CParagraphElement *para = new CParagraphElement();
	para->add("No help is available.");
	report->add(para);
	
	return report;
}

// ---------------------------------------------------------------------------------
//		 CEngineCaller::doQuit
// ---------------------------------------------------------------------------------

bool CEngineCaller::doQuit(CCommandRecord *inCommand)
{
	return true;
}		
