/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CGraph.h
 
		CGraph Class
	 
 ******************************************************************************/

#ifndef _CGRAPH_H_
#define _CGRAPH_H_

class CAxis;
class CPlot;
class CDataSeries;

class CComplexGraphicElement;

class CGraph {
public:
	static const int	mMaxPlots = 16;
	
protected:												
	bool				isCalibrated;

	bool				xRangeManual, xLogarithmic;
	bool				yRangeManual, yLogarithmic;
	
	double 				mMajorTickSize, mMinorTickSize;
	
	double				xMinValue, xMaxValue, yMinValue, yMaxValue;
			
	CAxis				*mXAxis, *mYAxis;
	
	CPlot*				mPlots[mMaxPlots];
	int					mNumPlots;
	
	double 				mPlotFrameTop, mPlotFrameLeft, mPlotFrameBottom, mPlotFrameRight;
	double				mHeight, mWidth; 
	double				xScale, yScale, xMin, yMin;
	
	int 				xTickLabelHeight, yTickLabelHeight, yTickLabelOfs, yTickLabelWidth;
	
	char*				mTitleText;
	char*				mXAxisLabelText;
	char*				mYAxisLabelText;
	
	char*				mTitleFontFace;
	char*				mAxisLabelFontFace;
	char*				mTickLabelFontFace;
	
	int					mTitleFontSize;
	int					mAxisLabelFontSize;
	int					mTickLabelFontSize;
	
public:
	//====<<  METHODS  >>============================================================
		
	//----<<  INITIALIZATION & DISPOSAL  >>------------------------------------------

 						CGraph( );
 						
private:
 	void				Initialise();
 	
public:
 	virtual				~CGraph();
 	
	//----<<  ACCESSING  >>----------------------------------------------------------
		
	virtual	void		SetAxes(CAxis *inXAxis, CAxis *inYAxis);	
	virtual	CAxis		*GetXAxis() { return mXAxis; }
	virtual	CAxis		*GetYAxis() { return mYAxis; }
	
	virtual	void		SetTitle(char *inText);
	virtual	void		SetXAxisLabel(char *inText);
	virtual	void		SetYAxisLabel(char *inText);
	
	virtual	void		SetTitleFont(int inFontSize, char *inFontFace);
	virtual	void		SetAxisLabelFont(int inFontSize, char *inFontFace);
	virtual	void		SetTickLabelFont(int inFontSize, char *inFontFace);
	
	virtual	int			AddPlot(CPlot *inPlot);
	virtual	CPlot		*GetPlot(int index);
	virtual	void		SetPlotData(int index, CDataSeries *inData);

	virtual	void		SetXAxisLogarithmic(bool inIsLogarithmic);
	virtual	void		SetYAxisLogarithmic(bool inIsLogarithmic);
	
	virtual	void		SetXRangeManual(double minValue, double maxValue);
	virtual	void		SetXRangeAutomatic();
	
	virtual	void		SetYRangeManual(double minValue, double maxValue);
	virtual	void		SetYRangeAutomatic();

	virtual	void		Invalidate() { isCalibrated=false; };
	
	//----<<  CALIBRATING  >>--------------------------------------------------------
	
protected:
	virtual	void		Calibrate(CComplexGraphicElement *ioGraphic);
	virtual	void		CalibrateXAxis();
	virtual	void		CalibrateYAxis();
	
public:			
	//----<<  DRAWING  >>------------------------------------------------------------

	double 				ConvertX(double x);
	double 				ConvertY(double y);
	
protected:
	virtual	int			GetYTickLabelWidth(double y);

	virtual	void		DrawMajorTickY(CComplexGraphicElement *ioGraphic, double y);
	virtual	void		DrawMinorTickY(CComplexGraphicElement *ioGraphic, double y);
	virtual	void		DrawYAxis(CComplexGraphicElement *ioGraphic);
	
	virtual	int			GetXTickLabelWidth(double x);

	virtual	void		DrawMajorTickX(CComplexGraphicElement *ioGraphic, double x);
	virtual	void		DrawMinorTickX(CComplexGraphicElement *ioGraphic, double x);
	virtual	void		DrawXAxis(CComplexGraphicElement *ioGraphic);
	
public:
	virtual	void		DrawGraph(CComplexGraphicElement *ioGraphic, 
									double inXTolerance = 0.0, 
									double inYTolerance = 0.0 );
};

#endif /* _CGRAPH_H_ */
