/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CGraphicsRecord.cpp

				CGraphicsRecord Class

 ******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "CExceptions.h"

#include "CGraphicsRecord.h"

#pragma mark === CGraphicElement ===
// ---------------------------------------------------------------------------------
//		 CGraphicElement::CGraphicElement
// ---------------------------------------------------------------------------------

CGraphicElement::CGraphicElement() : CReportElement()
{
	mX = 0.0;
	mY = 0.0;
	mWidth = 0.0;
	mHeight = 0.0;

	mHasStrokeWidth = false;
	mStrokeWidth = -1.0;
	
	mHasLineCap = false;
	mLineCap = kDefaultLineCap;
	
	mHasLineJoin = false;
	mLineJoin = kDefaultLineJoin;
	
	mHasStrokeColour = false;
	mStrokeColour.mType = kNoColour;
	
	mHasFillColour = false;
	mFillColour.mType = kNoColour;

	mHasRotation = false; 
	mRotation = 0.0;
	mRotationCentreX = 0.0;
	mRotationCentreY = 0.0;

	mHasScale = false; 
	mScaleX = 0.0;
	mScaleY = 0.0;
	mScaleCentreX = 0.0;
	mScaleCentreY = 0.0;
	
	mHasTranslation = false;
	mTransX = 0.0;
	mTransY = 0.0;
}

// ---------------------------------------------------------------------------------
//		 CGraphicElement::CGraphicElement
// ---------------------------------------------------------------------------------

CGraphicElement::CGraphicElement(const CGraphicElement &inSource)
{
	mX = inSource.mX;
	mY = inSource.mY;
	mWidth = inSource.mWidth;
	mHeight = inSource.mHeight;
	
	setStyle(inSource);
	
	mHasRotation = inSource.mHasRotation; 
	mRotation = inSource.mRotation;
	mRotationCentreX = inSource.mRotationCentreX;
	mRotationCentreY = inSource.mRotationCentreY;

	mHasScale = inSource.mHasScale; 
	mScaleX = inSource.mScaleX;
	mScaleY = inSource.mScaleY;
	mScaleCentreX = inSource.mScaleCentreX;
	mScaleCentreY = inSource.mScaleCentreY;
	
	mHasTranslation = inSource.mHasTranslation;
	mTransX = inSource.mTransX;
	mTransY = inSource.mTransY;
}

// ---------------------------------------------------------------------------------
//		 CGraphicElement::~CGraphicElement
// ---------------------------------------------------------------------------------

CGraphicElement::~CGraphicElement() 
{		
}

// ---------------------------------------------------------------------------------
//		 CGraphicElement::setStyle
// ---------------------------------------------------------------------------------

void CGraphicElement::setStrokeStyle(double inStrokeWidth, 
									StrokeLineCapType inLineCap, 
									StrokeLineJoinType inLineJoin)
{
	setStrokeWidth(inStrokeWidth);
	setStrokeLineCap(inLineCap);
	setStrokeLineJoin(inLineJoin);
}

// ---------------------------------------------------------------------------------
//		 CGraphicElement::setStyle
// ---------------------------------------------------------------------------------

void CGraphicElement::setStyle(const CGraphicElement &inSource)
{
	clearStyle();
	if (inSource.mHasStrokeWidth)
		setStrokeWidth(inSource.mStrokeWidth);
	if (inSource.mHasStrokeColour)
		setStrokeColour(inSource.mStrokeColour);
	if (inSource.mHasLineCap)
		setStrokeLineCap(inSource.mLineCap);
	if (inSource.mHasLineJoin)
		setStrokeLineJoin(inSource.mLineJoin);
	if (inSource.mHasFillColour)
		setFillColour(inSource.mFillColour);
}

// ---------------------------------------------------------------------------------
//		 CGraphicElement::clearStyle
// ---------------------------------------------------------------------------------

void CGraphicElement::clearStyle()
{
	mHasStrokeWidth = false;
	mStrokeWidth = 1.0;
	
	mHasLineCap = false;
	mLineCap = kDefaultLineCap;
	
	mHasLineJoin = false;
	mLineJoin = kDefaultLineJoin;
	
	mHasStrokeColour = false;
	mStrokeColour.mType = kNoColour;
	
	mHasFillColour = false;
	mFillColour.mType = kNoColour;

}

// ---------------------------------------------------------------------------------
//		 CGraphicElement::setStrokeWidth
// ---------------------------------------------------------------------------------

void CGraphicElement::setStrokeWidth(double inStrokeWidth)
{
	mHasStrokeWidth = true;
	mStrokeWidth = inStrokeWidth;
}

// ---------------------------------------------------------------------------------
//		 CGraphicElement::setStrokeLineCap
// ---------------------------------------------------------------------------------

void CGraphicElement::setStrokeLineCap(StrokeLineCapType inLineCap)
{
	mHasLineCap = true;
	mLineCap = inLineCap;
}

// ---------------------------------------------------------------------------------
//		 CGraphicElement::setStrokeLineJoin
// ---------------------------------------------------------------------------------

void CGraphicElement::setStrokeLineJoin(StrokeLineJoinType inLineJoin)
{
	mHasLineJoin = true;
	mLineJoin = inLineJoin;
}

// ---------------------------------------------------------------------------------
//		 CGraphicElement::setStrokeColour
// ---------------------------------------------------------------------------------

void CGraphicElement::setStrokeColour(const CColourRec &inColour)
{
	mHasStrokeColour = true;
	mStrokeColour = inColour;
}

// ---------------------------------------------------------------------------------
//		 CGraphicElement::setStrokeColour
// ---------------------------------------------------------------------------------

void CGraphicElement::setStrokeColour(ColourType inColour, double inAlpha)
{
	mHasStrokeColour = true;
	mStrokeColour.mType = inColour;
	mStrokeColour.mAlpha = inAlpha;
}

// ---------------------------------------------------------------------------------
//		 CGraphicElement::setStrokeColour
// ---------------------------------------------------------------------------------

void CGraphicElement::setStrokeColour(double inRed, double inGreen, double inBlue, double inAlpha)
{
	mHasStrokeColour = true;
	mStrokeColour.mType = kRGBColour;
	mStrokeColour.mRed = inRed;
	mStrokeColour.mGreen = inGreen;
	mStrokeColour.mBlue = inBlue;
	mStrokeColour.mAlpha = inAlpha;
}

// ---------------------------------------------------------------------------------
//		 CGraphicElement::setFillColour
// ---------------------------------------------------------------------------------

void CGraphicElement::setFillColour(const CColourRec &inColour)
{
	mHasFillColour = true;
	mFillColour = inColour;
}

// ---------------------------------------------------------------------------------
//		 CGraphicElement::setFillColour
// ---------------------------------------------------------------------------------

void CGraphicElement::setFillColour(ColourType inColour, double inAlpha)
{
	mHasFillColour = true;
	mFillColour.mType = inColour;
	mFillColour.mAlpha = inAlpha;
}

// ---------------------------------------------------------------------------------
//		 CGraphicElement::setFillColour
// ---------------------------------------------------------------------------------

void CGraphicElement::setFillColour(double inRed, double inGreen, double inBlue, double inAlpha)
{
	mHasFillColour = true;
	mFillColour.mType = kRGBColour;
	mFillColour.mRed = inRed;
	mFillColour.mGreen = inGreen;
	mFillColour.mBlue = inBlue;
	mFillColour.mAlpha = inAlpha;
}

#pragma mark === CLineElement ===
// ---------------------------------------------------------------------------------
//		 CLineElement::CLineElement
// ---------------------------------------------------------------------------------

CLineElement::CLineElement(double inX1, double inY1, 
							double inX2, double inY2) :
			CGraphicElement()
{
	double minX, maxX, minY, maxY;
	if (inX1 > inX2) {
		minX = inX2;
		maxX = inX1;
	} else {
		minX = inX1;
		maxX = inX2;
	}
	
	if (inY1 > inY2) {
		minY = inY2;
		maxY = inY1;
	} else {
		minY = inY1;
		maxY = inY2;
	}
	setSize(maxX - minX, maxY - minY);
	
	mX = inX1;
	mY = inY1;
	mX2 = inX2;
	mY2 = inY2;
}

// ---------------------------------------------------------------------------------
//		 CLineElement::CLineElement
// ---------------------------------------------------------------------------------

CLineElement::CLineElement(const CLineElement &inSource) :
			CGraphicElement(inSource)
{
	mX2 = inSource.mX2;
	mY2 = inSource.mY2;
}

// ---------------------------------------------------------------------------------
//		 CLineElement::setTopLeftPosition
// ---------------------------------------------------------------------------------

void CLineElement::setTopLeftPosition(double inX, double inY)
{
	double minX, minY;
	if (mX > mX2)
		minX = mX2;
	else
		minX = mX;
	
	if (mY > mY2)
		minY = mY2;	
	else
		minY = mY;
	
	double dx = inX - mX;
	double dy = inY - mY;
	
	mX += dx;	mX2 += dx;
	mY += dy;	mY2 += dy;
}

// ---------------------------------------------------------------------------------
//		 CLineElement::setCentrePosition
// ---------------------------------------------------------------------------------

void CLineElement::setCentrePosition(double inX, double inY)
{
	double minX, maxX, minY, maxY;
	if (mX > mX2) {
		minX = mX2;
		maxX = mX;
	} else {
		minX = mX;
		maxX = mX2;
	}
	
	if (mY > mY2) {
		minY = mY2;
		maxY = mY;
	} else {
		minY = mY;
		maxY = mY2;
	}

	double cx = (minX + maxX) / 2.0;
	double cy = (maxY + maxY) / 2.0;
	double dx = inX - cx;
	double dy = inY - cy;
	
	mX += dx;	mX2 += dx;
	mY += dy;	mY2 += dy;
}

// ---------------------------------------------------------------------------------
//		 CLineElement::setSize
// ---------------------------------------------------------------------------------

void CLineElement::setSize(double inWidth, double inHeight)
{	
	mWidth = inWidth;
	mHeight = inHeight;
	
	mX2 = mX + mWidth;
	mY2 = mY + inHeight;
}

// ---------------------------------------------------------------------------------
//		 CLineElement::getLeft
// ---------------------------------------------------------------------------------

double CLineElement::getLeft()
{	
	return (mX < mX2 ? mX : mX2);
}

// ---------------------------------------------------------------------------------
//		 CLineElement::getTop
// ---------------------------------------------------------------------------------

double CLineElement::getTop()
{	
	return (mY < mY2 ? mY : mY2);
}

// ---------------------------------------------------------------------------------
//		 CLineElement::getRight
// ---------------------------------------------------------------------------------

double CLineElement::getRight()
{	
	return (mX > mX2 ? mX : mX2);
}

// ---------------------------------------------------------------------------------
//		 CLineElement::getBottom
// ---------------------------------------------------------------------------------

double CLineElement::getBottom()
{	
	return (mY > mY2 ? mY : mY2);
}

#pragma mark === CRectElement ===
// ---------------------------------------------------------------------------------
//		 CRectElement::CRectElement
// ---------------------------------------------------------------------------------

CRectElement::CRectElement(double inX, double inY, 
							double inWidth, double inHeight) :
			CGraphicElement()
{
	mX = inX;
	mY = inY;
	mWidth = inWidth;
	mHeight = inHeight;
}

// ---------------------------------------------------------------------------------
//		 CRectElement::CRectElement
// ---------------------------------------------------------------------------------

CRectElement::CRectElement(const CRectElement &inSource) :
			CGraphicElement(inSource)
{
}

// ---------------------------------------------------------------------------------
//		 CRectElement::setTopLeftPosition
// ---------------------------------------------------------------------------------

void CRectElement::setTopLeftPosition(double inX, double inY)
{
	mX = inX;
	mY = inY;
}

// ---------------------------------------------------------------------------------
//		 CRectElement::setCentrePosition
// ---------------------------------------------------------------------------------

void CRectElement::setCentrePosition(double inX, double inY)
{
	double cx = mWidth / 2.0;
	double cy = mHeight / 2.0;
	double dx = inX - cx;
	double dy = inY - cy;
	
	mX += dx;
	mY += dy;
}

#pragma mark === CRoundedRectElement ===
// ---------------------------------------------------------------------------------
//		 CRoundedRectElement::CRoundedRectElement
// ---------------------------------------------------------------------------------

CRoundedRectElement::CRoundedRectElement(double inX, double inY, 
										double inWidth, double inHeight,
										double inRadiusX, double inRadiusY) :
			CRectElement(inX, inY, inWidth, inHeight)
{
	mRadiusX = inRadiusX;
	mRadiusY = inRadiusY;
}

// ---------------------------------------------------------------------------------
//		 CRoundedRectElement::CRoundedRectElement
// ---------------------------------------------------------------------------------

CRoundedRectElement::CRoundedRectElement(const CRoundedRectElement &inSource) :
			CRectElement(inSource)
{
	mRadiusX = inSource.mRadiusX;
	mRadiusY = inSource.mRadiusY;
}

#pragma mark === CCircleElement ===
// ---------------------------------------------------------------------------------
//		 CCircleElement::CCircleElement
// ---------------------------------------------------------------------------------

CCircleElement::CCircleElement(double inX, double inY, double inRadius) :
			CGraphicElement()
{
	mX = inX;
	mY = inY;
	mWidth = inRadius * 2;
	mHeight = mWidth;
	mRadius = inRadius;
}

// ---------------------------------------------------------------------------------
//		 CCircleElementent::CCircleElement
// ---------------------------------------------------------------------------------

CCircleElement::CCircleElement(const CCircleElement &inSource) :
			CGraphicElement(inSource)
{
	mRadius = inSource.mRadius;
}

// ---------------------------------------------------------------------------------
//		 CCircleElement::setPosition
// ---------------------------------------------------------------------------------

void CCircleElement::setPosition(double inX, double inY)
{
	mX = inX;
	mY = inY;
}

// ---------------------------------------------------------------------------------
//		 CCircleElement::setTopLeftPosition
// ---------------------------------------------------------------------------------

void CCircleElement::setTopLeftPosition(double inX, double inY)
{
	mX = inX + mRadius;
	mY = inY + mRadius;
}

// ---------------------------------------------------------------------------------
//		 CCircleElement::setCentrePosition
// ---------------------------------------------------------------------------------

void CCircleElement::setCentrePosition(double inX, double inY)
{
	mX = inX;
	mY = inY;
}

// ---------------------------------------------------------------------------------
//		 CCircleElement::setSize
// ---------------------------------------------------------------------------------

void CCircleElement::setSize(double inWidth, double inHeight)
{	
	mWidth = inWidth;
	mHeight = inWidth;
	mRadius = inWidth / 2.0;
}

// ---------------------------------------------------------------------------------
//		 CCircleElement::getLeft
// ---------------------------------------------------------------------------------

double CCircleElement::getLeft()
{	
	return mX - mRadius;
}

// ---------------------------------------------------------------------------------
//		 CCircleElement::getTop
// ---------------------------------------------------------------------------------

double CCircleElement::getTop()
{	
	return mY - mRadius;
}

// ---------------------------------------------------------------------------------
//		 CCircleElement::getRight
// ---------------------------------------------------------------------------------

double CCircleElement::getRight()
{	
	return mX + mRadius;
}

// ---------------------------------------------------------------------------------
//		 CCircleElement::getBottom
// ---------------------------------------------------------------------------------

double CCircleElement::getBottom()
{	
	return mY + mRadius;
}

#pragma mark === CEllipseElement ===
// ---------------------------------------------------------------------------------
//		 CEllipseElement::CEllipseElement
// ---------------------------------------------------------------------------------

CEllipseElement::CEllipseElement(double inX, double inY, 
									double inRadiusX, double inRadiusY) :
			CGraphicElement()
{
	mX = inX;
	mY = inY;
	mWidth = inRadiusX * 2;
	mHeight = inRadiusY * 2;
	mRadiusX = inRadiusX;
	mRadiusY = inRadiusY;
}

// ---------------------------------------------------------------------------------
//		 CEllipseElement::CEllipseElement
// ---------------------------------------------------------------------------------

CEllipseElement::CEllipseElement(const CEllipseElement &inSource) :
			CGraphicElement(inSource)
{
	mRadiusX = inSource.mRadiusX;
	mRadiusY = inSource.mRadiusY;
}

// ---------------------------------------------------------------------------------
//		 CEllipseElement::setPosition
// ---------------------------------------------------------------------------------

void CEllipseElement::setPosition(double inX, double inY)
{
	mX = inX;
	mY = inY;
}

// ---------------------------------------------------------------------------------
//		 CEllipseElement::setTopLeftPosition
// ---------------------------------------------------------------------------------

void CEllipseElement::setTopLeftPosition(double inX, double inY)
{
	mX = inX + mRadiusX;
	mY = inY + mRadiusY;
}

// ---------------------------------------------------------------------------------
//		 CEllipseElement::setCentrePosition
// ---------------------------------------------------------------------------------

void CEllipseElement::setCentrePosition(double inX, double inY)
{
	mX = inX;
	mY = inY;
}

// ---------------------------------------------------------------------------------
//		 CEllipseElement::setSize
// ---------------------------------------------------------------------------------

void CEllipseElement::setSize(double inWidth, double inHeight)
{	
	mWidth = inWidth;
	mHeight = inHeight;
	mRadiusX = inWidth / 2.0;
	mRadiusY = inHeight / 2.0;
}

// ---------------------------------------------------------------------------------
//		 CEllipseElement::getLeft
// ---------------------------------------------------------------------------------

double CEllipseElement::getLeft()
{	
	return mX - mRadiusX;
}

// ---------------------------------------------------------------------------------
//		 CEllipseElement::getTop
// ---------------------------------------------------------------------------------

double CEllipseElement::getTop()
{	
	return mY - mRadiusY;
}

// ---------------------------------------------------------------------------------
//		 CEllipseElement::getRight
// ---------------------------------------------------------------------------------

double CEllipseElement::getRight()
{	
	return mX + mRadiusX;
}

// ---------------------------------------------------------------------------------
//		 CEllipseElement::getBottom
// ---------------------------------------------------------------------------------

double CEllipseElement::getBottom()
{	
	return mY + mRadiusY;
}

#pragma mark === CTextGraphicElement ===
// ---------------------------------------------------------------------------------
//		 CTextGraphicElement::CTextGraphicElement
// ---------------------------------------------------------------------------------

CTextGraphicElement::CTextGraphicElement(double inX, double inY, 
											char *inText,
											TextAnchorType inAnchor) :
			CGraphicElement()
{
	mX = inX;
	mY = inY;
	
	mText = NULL;
	setText(inText);
	
	mHasTextAnchor = inAnchor != kDefaultTextAnchor;
	mTextAnchor = inAnchor;
	mHasFontSize = false;
	mFontSize = 0.0;
	mHasFontFace = false;
	mFontFace = NULL;
}

// ---------------------------------------------------------------------------------
//		 CTextGraphicElement::CTextGraphicElement
// ---------------------------------------------------------------------------------

CTextGraphicElement::CTextGraphicElement(const CTextGraphicElement &inSource) :
			CGraphicElement(inSource)
{	
	setText(inSource.mText);
	mTextAnchor = inSource.mTextAnchor;
	mHasTextAnchor = inSource.mHasTextAnchor;
	setFontStyle(inSource.mFontSize, inSource.mFontFace);
	mHasFontSize = inSource.mHasFontSize;
	mHasFontFace = inSource.mHasFontFace;
}

// ---------------------------------------------------------------------------------
//		 CTextGraphicElement::CTextGraphicElement
// ---------------------------------------------------------------------------------

CTextGraphicElement::~CTextGraphicElement()
{
	if (mText != NULL)
		delete [] mText;

	if (mFontFace != NULL)
		delete [] mFontFace;
}

// ---------------------------------------------------------------------------------
//		 CTextGraphicElement::setPosition
// ---------------------------------------------------------------------------------

void CTextGraphicElement::setPosition(double inX, double inY)
{
	mX = inX;
	mY = inY;
}

// ---------------------------------------------------------------------------------
//		 CTextGraphicElement::setTopLeftPosition
// ---------------------------------------------------------------------------------
// Width and Height must have been set first

void CTextGraphicElement::setTopLeftPosition(double inX, double inY)
{
	mX = inX;
	mY = inY + mHeight;
}

// ---------------------------------------------------------------------------------
//		 CTextGraphicElement::setCentrePosition
// ---------------------------------------------------------------------------------
// Width and Height must have been set first

void CTextGraphicElement::setCentrePosition(double inX, double inY)
{
	mX = inX - (mWidth / 2.0);
	mY = inY + (mHeight / 2.0);
}

// ---------------------------------------------------------------------------------
//		 CTextGraphicElement::setText
// ---------------------------------------------------------------------------------

void CTextGraphicElement::setText(char *inText)
{
	if (mText != NULL)
		delete [] mText;
		
	mText = NULL;
	
	if (inText != NULL) {
		int len = strlen(inText);
		if (len > 0) {
			NewArray(mText, char, len+1);
			strcpy(mText, inText);
		}
	}
}

// ---------------------------------------------------------------------------------
//		 CTextGraphicElement::getLeft
// ---------------------------------------------------------------------------------

double CTextGraphicElement::getLeft()
{	
	return mX;
}

// ---------------------------------------------------------------------------------
//		 CTextGraphicElement::getTop
// ---------------------------------------------------------------------------------

double CTextGraphicElement::getTop()
{	
	return mY - mHeight;
}

// ---------------------------------------------------------------------------------
//		 CTextGraphicElement::getRight
// ---------------------------------------------------------------------------------

double CTextGraphicElement::getRight()
{	
	return mX + mWidth;
}

// ---------------------------------------------------------------------------------
//		 CTextGraphicElement::getBottom
// ---------------------------------------------------------------------------------

double CTextGraphicElement::getBottom()
{	
	return mY;
}

// ---------------------------------------------------------------------------------
//		 CTextGraphicElement::setTextAnchor
// ---------------------------------------------------------------------------------

void CTextGraphicElement::setTextAnchor(TextAnchorType inAnchor)
{
	mHasTextAnchor = inAnchor != kDefaultTextAnchor;
	mTextAnchor = inAnchor;
}

// ---------------------------------------------------------------------------------
//		 CTextGraphicElement::setFontStyle
// ---------------------------------------------------------------------------------

void CTextGraphicElement::setFontStyle(double inFontSize, 
										char *inFontFace)
{
	setFontSize(inFontSize);
	setFontFace(inFontFace);
}

// ---------------------------------------------------------------------------------
//		 CTextGraphicElement::clearFontStyle
// ---------------------------------------------------------------------------------

void CTextGraphicElement::clearFontStyle()
{
	setFontSize(0.0);
	setFontFace(NULL);
}

// ---------------------------------------------------------------------------------
//		 CTextGraphicElement::setFontSize
// ---------------------------------------------------------------------------------

void CTextGraphicElement::setFontSize(double inFontSize)
{
	mFontSize = inFontSize;
	if (mFontSize > 0.0)
		mHasFontSize = true;
	else
		mHasFontSize = false;
}


// ---------------------------------------------------------------------------------
//		 CTextGraphicElement::setFontFace
// ---------------------------------------------------------------------------------

void CTextGraphicElement::setFontFace(char *inFontFace)
{
	if (mFontFace != NULL)
		delete [] mFontFace;
		
	mFontFace = NULL;
	
	if (inFontFace != NULL) {
		int len = strlen(inFontFace);
		if (len > 0) {
			NewArray(mFontFace, char, len+1);
			strcpy(mFontFace, inFontFace);
		}
		mHasFontFace = true;
	} else
		mHasFontFace = false;

}

#pragma mark === CPathSubelement ===
// ---------------------------------------------------------------------------------
//		 CPathSubelement::CPathSubelement
// ---------------------------------------------------------------------------------

CPathSubelement::CPathSubelement()
{
}

// ---------------------------------------------------------------------------------
//		 CPathSubelement::CPathSubelement
// ---------------------------------------------------------------------------------

CPathSubelement::CPathSubelement(PathSubelementType inType, double inX, double inY)
{
	mType = inType;
	mX1 = inX;
	mY1 = inY;
}

// ---------------------------------------------------------------------------------
//		 CPathSubelement::CPathSubelement
// ---------------------------------------------------------------------------------

CPathSubelement::CPathSubelement(PathSubelementType inType, double inX1, double inY1,
									double inX2, double inY2)
{
	mType = inType;
	mX1 = inX1;
	mY1 = inY1;
	mX2 = inX2;
	mY2 = inY2;
}

// ---------------------------------------------------------------------------------
//		 CPathSubelement::CPathSubelement
// ---------------------------------------------------------------------------------

CPathSubelement::CPathSubelement(PathSubelementType inType, double inX1, double inY1,
									double inX2, double inY2, double inX3, double inY3)
{
	mType = inType;
	mX1 = inX1;
	mY1 = inY1;
	mX2 = inX2;
	mY2 = inY2;
	mX3 = inX3;
	mY3 = inY3;
}

#pragma mark === CPathElement ===
// ---------------------------------------------------------------------------------
//		 CPathElement::CPathElement
// ---------------------------------------------------------------------------------

CPathElement::CPathElement(double inX, double inY) :
			CGraphicElement()
{
	mX = inX;
	mY = inY;
	mLeft = inX;
	mTop = inY;
	mRight = inX;
	mBottom = inY;
	calcBounds(inX, inY);
	
	mSubelements = NULL;
	mNumSubelements = 0;
	mCapacity = 0;
	
	mPathClosed = false;
}

// ---------------------------------------------------------------------------------
//		 CPathElement::CPathElement
// ---------------------------------------------------------------------------------

CPathElement::CPathElement(const CPathElement &inSource) :
			CGraphicElement(inSource)
{
	mLeft = inSource.mLeft;
	mTop = inSource.mTop;
	mRight = inSource.mRight;
	mBottom = inSource.mBottom;
	mLastX = inSource.mLastX;
	mLastY = inSource.mLastY;
	
	mSubelements = NULL;
	mNumSubelements = 0;
	mCapacity = 0;

	checkCapacity(inSource.mNumSubelements);
	for (int i = 0; i < inSource.mNumSubelements; i++) 
		mSubelements[mNumSubelements] = inSource.mSubelements[i];

	mPathClosed = inSource.mPathClosed;
}

// ---------------------------------------------------------------------------------
//		 CPathElement::~CPathElement
// ---------------------------------------------------------------------------------

CPathElement::~CPathElement() 
{
	if (mSubelements != NULL)
		delete [] mSubelements;
}

// ---------------------------------------------------------------------------------
//		 CPathElement::setPosition
// ---------------------------------------------------------------------------------

void CPathElement::setPosition(double inX, double inY)
{
	mX = inX;
	mY = inY;
}

// ---------------------------------------------------------------------------------
//		 CPathElement::setTopLeftPosition
// ---------------------------------------------------------------------------------

void CPathElement::setTopLeftPosition(double inX, double inY)
{
	mX = inX;
	mY = inY;
}

// ---------------------------------------------------------------------------------
//		 CPathElement::setCentrePosition
// ---------------------------------------------------------------------------------

void CPathElement::setCentrePosition(double inX, double inY)
{
	mX = inX - (mWidth / 2.0);
	mY = inY - (mHeight / 2.0);
}

// ---------------------------------------------------------------------------------
//		 CPathElement::moveTo
// ---------------------------------------------------------------------------------

void CPathElement::moveTo(double inX, double inY) 
{
	CPathSubelement sub(kMoveToSubelement, inX, inY);
	addSublement(sub);
	calcBounds(inX, inY);
}
		
// ---------------------------------------------------------------------------------
//		 CPathElement::moveBy
// ---------------------------------------------------------------------------------

void CPathElement::moveBy(double inDX, double inDY) 
{
	CPathSubelement sub(kMoveBySubelement, inDX, inDY);
	addSublement(sub);
	calcBounds(mLastX + inDX, mLastY + inDY);
}
		
// ---------------------------------------------------------------------------------
//		 CPathElement::lineTo
// ---------------------------------------------------------------------------------

void CPathElement::lineTo(double inX, double inY) 
{
	CPathSubelement sub(kLineToSubelement, inX, inY);
	addSublement(sub);
	calcBounds(inX, inY);
}
		
// ---------------------------------------------------------------------------------
//		 CPathElement::lineBy
// ---------------------------------------------------------------------------------

void CPathElement::lineBy(double inDX, double inDY) 
{
	CPathSubelement sub(kLineBySubelement, inDX, inDY);
	addSublement(sub);
	calcBounds(mLastX + inDX, mLastY + inDY);
}
		
// ---------------------------------------------------------------------------------
//		 CPathElement::horizLineTo
// ---------------------------------------------------------------------------------

void CPathElement::horizLineTo(double inX) 
{
	CPathSubelement sub(kHLineToSubelement, inX, 0.0);
	addSublement(sub);
	calcBounds(inX, mLastY);
}
		
// ---------------------------------------------------------------------------------
//		 CPathElement::horizLineBy
// ---------------------------------------------------------------------------------

void CPathElement::horizLineBy(double inDX) 
{
	CPathSubelement sub(kHLineBySubelement, inDX, 0.0);
	addSublement(sub);
	calcBounds(mLastX + inDX, mLastY);
}
		
// ---------------------------------------------------------------------------------
//		 CPathElement::vertLineTo
// ---------------------------------------------------------------------------------

void CPathElement::vertLineTo(double inY) 
{
	CPathSubelement sub(kVLineToSubelement, 0.0, inY);
	addSublement(sub);
	calcBounds(mLastX, inY);
}
		
// ---------------------------------------------------------------------------------
//		 CPathElement::vertLineBy
// ---------------------------------------------------------------------------------

void CPathElement::vertLineBy(double inDY) 
{
	CPathSubelement sub(kVLineBySubelement, 0.0, inDY);
	addSublement(sub);
	calcBounds(mLastX, mLastY + inDY);
}
		
// ---------------------------------------------------------------------------------
//		 CPathElement::curveTo
// ---------------------------------------------------------------------------------

void CPathElement::curveTo(double inX, double inY, 
							double inCX1, double inCY1,
							double inCX2, double inCY2) 
{
	CPathSubelement sub(kCurveToSubelement, inX, inY, inCX1, inCY1, inCX2, inCY2);
	addSublement(sub);
	calcBounds(inX, inY);
}
		
// ---------------------------------------------------------------------------------
//		 CPathElement::curveBy
// ---------------------------------------------------------------------------------

void CPathElement::curveBy(double inDX, double inDY, 
							double inCX1, double inCY1,
							double inCX2, double inCY2) 
{
	CPathSubelement sub(kCurveBySubelement, inDX, inDY, inCX1, inCY1, inCX2, inCY2);
	addSublement(sub);
	calcBounds(mLastX + inDX, mLastY + inDY);
}
		
// ---------------------------------------------------------------------------------
//		 CPathElement::arc
// ---------------------------------------------------------------------------------

void CPathElement::arc(double inCX, double inCY, double inAngle1, double inAngle2) 
{
	CPathSubelement sub(kArcSubelement, inCX, inCY, inAngle1, inAngle2);
	addSublement(sub);
}
		
/*
// ---------------------------------------------------------------------------------
//		 CPathElement::arcTo
// ---------------------------------------------------------------------------------

void CPathElement::arcTo(double inX, double inY, 
							double inRX, double inRY, double inRotate,
							bool inLargeArcFlag, bool inSweepFlag) 
{
	CPathSubelement sub(kArcToSubelement, inX, inY, inRX, inRY, inRotate, inLargeArcFlag, inSweepFlag);
	addSublement(sub);
}
		
// ---------------------------------------------------------------------------------
//		 CPathElement::arcBy
// ---------------------------------------------------------------------------------

void CPathElement::arcBy(double inDX, double inDY, 
							double inRX, double inRY, double inRotate,
							bool inLargeArcFlag, bool inSweepFlag) 
{
	CPathSubelement sub(kArcBySubelement, inDX, inDY, inRX, inRY, inRotate, inLargeArcFlag, inSweepFlag);
	addSublement(sub);
}
*/		
// ---------------------------------------------------------------------------------
//		 CPathElement::addSublement
// ---------------------------------------------------------------------------------

void CPathElement::addSublement(CPathSubelement inSubelement) 
{
	checkCapacity(mNumSubelements + 1);
	mSubelements[mNumSubelements] = inSubelement;
	mNumSubelements++;
}
		
// ---------------------------------------------------------------------------------
//		 CPathElement::calcBounds
// ---------------------------------------------------------------------------------

void CPathElement::calcBounds(double inX, double inY)
{
	if (inX < mLeft) mLeft = inX;
	if (inY < mTop) mTop = inY;
	if (inX > mRight) mRight = inX;
	if (inY > mBottom) mBottom = inY;
	mLastX = inX;
	mLastY = inY;
	
	mWidth = mRight - mLeft;
	mHeight = mBottom - mTop;
}

// ---------------------------------------------------------------------------------
//		 CPathElement::getSubelement
// ---------------------------------------------------------------------------------

CPathSubelement CPathElement::getSubelement(int index) 
{ 
	if (index < 0 || index >= mNumSubelements)
		throw CIndexOutOfBounds();
		
	return mSubelements[index]; 
}

// ---------------------------------------------------------------------------------
//		CPathElement::checkCapacity
// ---------------------------------------------------------------------------------

void CPathElement::checkCapacity(int requiredCapacity)
{
	if (requiredCapacity > mCapacity) {
		CPathSubelement *newSubelements;
		int i;
		
		// Increase capacity
		while (mCapacity < requiredCapacity)
			mCapacity += 50;
		
		// Allocate the new list.
		NewArray(newSubelements, CPathSubelement, mCapacity);
		
		if (mSubelements != NULL) {
			// Copy the existing list into the new list.
			for (i=0; i<mNumSubelements; i++)
				newSubelements[i] = mSubelements[i];
	
			// Free the old list.
			delete [] mSubelements;
		}
		
		// Point the old list to the new one.
		mSubelements = newSubelements;
	}
}

#pragma mark === CComplexGraphicElement ===
// ---------------------------------------------------------------------------------
//		 CComplexGraphicElement::CComplexGraphicElement
// ---------------------------------------------------------------------------------

CComplexGraphicElement::CComplexGraphicElement() :
			CGraphicElement()
{
	mElements = NULL;
	mNumElements = 0;
	mCapacity = 0;
	mLeft = 0.0;
	mTop = 0.0;
	mRight = 0.0;
	mBottom = 0.0;
}

// ---------------------------------------------------------------------------------
//		 CComplexGraphicElement::CComplexGraphicElement
// ---------------------------------------------------------------------------------

CComplexGraphicElement::CComplexGraphicElement(const CComplexGraphicElement &inSource) :
			CGraphicElement(inSource)
{
	mElements = NULL;
	mNumElements = 0;
	mCapacity = 0;

	mLeft = inSource.mLeft;
	mTop = inSource.mTop;
	mRight = inSource.mRight;
	mBottom = inSource.mBottom;

	GraphicType gType;
	CGraphicElement *element;
	for (int i = 0; i < inSource.mNumElements; i++) {
		gType = inSource.mElements[i]->getGraphicType();
		
		switch (gType) {
			case kLineGraphicElement:
				element = new CLineElement(*dynamic_cast<CLineElement*>(inSource.mElements[i]));
			break;
			case kRectGraphicElement:
				element = new CRectElement(*dynamic_cast<CRectElement*>(inSource.mElements[i]));
			break;
			case kRoundedRectGraphicElement:
				element = new CRoundedRectElement(*dynamic_cast<CRoundedRectElement*>(inSource.mElements[i]));
			break;
			case kCircleGraphicElement:
				element = new CCircleElement(*dynamic_cast<CCircleElement*>(inSource.mElements[i]));
			break;
			case kEllipseGraphicElement:
				element = new CEllipseElement(*dynamic_cast<CEllipseElement*>(inSource.mElements[i]));
			break;
			case kTextGraphicElement:
				element = new CTextGraphicElement(*dynamic_cast<CTextGraphicElement*>(inSource.mElements[i]));
			break;
			case kPathGraphicElement:
				element = new CPathElement(*dynamic_cast<CPathElement*>(inSource.mElements[i]));
			break;
			case kGroupGraphicElement:
				element = new CGroupElement(*dynamic_cast<CGroupElement*>(inSource.mElements[i]));
			break;
			case kViewPortGraphicElement:
				element = new CViewPortElement(*dynamic_cast<CViewPortElement*>(inSource.mElements[i]));
			break;
			case kFigureGraphicElement:
				element = new CFigureElement(*dynamic_cast<CFigureElement*>(inSource.mElements[i]));
			break;
		}
		
		add(element);
	}
}

// ---------------------------------------------------------------------------------
//		 CComplexGraphicElement::~CComplexGraphicElement
// ---------------------------------------------------------------------------------

CComplexGraphicElement::~CComplexGraphicElement() 
{
	if (mElements != NULL)
		delete [] mElements;
}

// ---------------------------------------------------------------------------------
//		 CComplexGraphicElement::setTopLeftPosition
// ---------------------------------------------------------------------------------

void CComplexGraphicElement::setTopLeftPosition(double inX, double inY)
{
	mX = inX;
	mY = inY;
}

// ---------------------------------------------------------------------------------
//		 CComplexGraphicElement::setCentrePosition
// ---------------------------------------------------------------------------------

void CComplexGraphicElement::setCentrePosition(double inX, double inY)
{
	double cx = mWidth / 2.0;
	double cy = mHeight / 2.0;
	double dx = inX - cx;
	double dy = inY - cy;
	
	mX += dx;
	mY += dy;
}

// ---------------------------------------------------------------------------------
//		 CComplexGraphicElement::setSize
// ---------------------------------------------------------------------------------

void CComplexGraphicElement::setSize(double inWidth, double inHeight)
{	
	mWidth = inWidth;
	mHeight = inHeight;
}

// ---------------------------------------------------------------------------------
//		 CComplexGraphicElement::add
// ---------------------------------------------------------------------------------

void CComplexGraphicElement::add(CGraphicElement *inElement) 
{
	checkCapacity(mNumElements + 1);
	mElements[mNumElements] = inElement;
	mNumElements++;
	
	calcBounds(inElement);
}
		
// ---------------------------------------------------------------------------------
//		 CComplexGraphicElement::getElement
// ---------------------------------------------------------------------------------

CGraphicElement *CComplexGraphicElement::getElement(int index) 
{ 
	if (index < 0 || index >= mNumElements)
		throw CIndexOutOfBounds();
		
	return mElements[index]; 
}

// ---------------------------------------------------------------------------------
//		 CComplexGraphicElement::calcBounds
// ---------------------------------------------------------------------------------

void CComplexGraphicElement::calcBounds(CGraphicElement *inElement)
{
	if (inElement->getLeft() < mLeft) mLeft = inElement->getLeft();
	if (inElement->getTop() < mTop) mTop = inElement->getTop();
	if (inElement->getRight() > mRight) mRight = inElement->getRight();
	if (inElement->getBottom() > mBottom) mBottom = inElement->getBottom();
}

// ---------------------------------------------------------------------------------
//		CComplexGraphicElement::checkCapacity
// ---------------------------------------------------------------------------------

void CComplexGraphicElement::checkCapacity(int requiredCapacity)
{
	if (requiredCapacity > mCapacity) {
		CGraphicElement **newElements;
		int i;
		
		// Increase capacity
		while (mCapacity < requiredCapacity)
			mCapacity += 50;
		
		// Allocate the new list.
		NewArray(newElements, CGraphicElement *, mCapacity);
		
		if (mElements != NULL) {
			// Copy the existing list into the new list.
			for (i=0; i<mNumElements; i++)
				newElements[i] = mElements[i];
	
			// Free the old list.
			delete [] mElements;
		}
		
		// Point the old list to the new one.
		mElements = newElements;
	}
}

#pragma mark === CGroupGraphicElement ===
// ---------------------------------------------------------------------------------
//		 CGroupElement::CGroupElement
// ---------------------------------------------------------------------------------

CGroupElement::CGroupElement() :
			CComplexGraphicElement()
{
}

// ---------------------------------------------------------------------------------
//		 CGroupElement::CGroupElement
// ---------------------------------------------------------------------------------

CGroupElement::CGroupElement(const CGroupElement &inSource) :
			CComplexGraphicElement(inSource)
{
}

// ---------------------------------------------------------------------------------
//		 CGroupElement::~CGroupElement
// ---------------------------------------------------------------------------------

CGroupElement::~CGroupElement() 
{
	if (mElements != NULL)
		delete [] mElements;
}

#pragma mark === CViewPortElement ===
// ---------------------------------------------------------------------------------
//		 CViewPortElement::CViewPortElement
// ---------------------------------------------------------------------------------

CViewPortElement::CViewPortElement() :
			CComplexGraphicElement()
{
	mHasViewBox = false;
	mViewBoxMinX = 0.0;
	mViewBoxMinY = 0.0;
	mViewBoxWidth = 1.0;
	mViewBoxHeight = 1.0;

	mPreserveAspectRatio = kNoneAspect;
	mSliceViewBox = false;
		
	mPageUnits = kNoPageUnits;
	mWidth = 1.0;
	mHeight = 1.0;
}

// ---------------------------------------------------------------------------------
//		 CViewPortElement::CViewPortElement
// ---------------------------------------------------------------------------------

CViewPortElement::CViewPortElement(const CViewPortElement &inSource) :
			CComplexGraphicElement(inSource)
{
	mHasViewBox = inSource.mHasViewBox;
	mViewBoxMinX = inSource.mViewBoxMinX;
	mViewBoxMinY = inSource.mViewBoxMinY;
	mViewBoxWidth = inSource.mViewBoxWidth;
	mViewBoxHeight = inSource.mViewBoxHeight;

	mPreserveAspectRatio = inSource.mPreserveAspectRatio;
	mSliceViewBox = inSource.mSliceViewBox;
		
	mPageUnits = inSource.mPageUnits;
}

// ---------------------------------------------------------------------------------
//		 CViewPortElement::~CViewPortElement
// ---------------------------------------------------------------------------------

CViewPortElement::~CViewPortElement() 
{
}

// ---------------------------------------------------------------------------------
//		 CViewPortElement::setSize
// ---------------------------------------------------------------------------------

void CViewPortElement::setSize(double inWidth, double inHeight)
{	
	CComplexGraphicElement::setSize(inWidth, inHeight);
}

// ---------------------------------------------------------------------------------
//		 CViewPortElement::setSize
// ---------------------------------------------------------------------------------

void CViewPortElement::setSize(PageUnitsType inPageUnits, double inWidth, double inHeight)
{
	mPageUnits = inPageUnits;
	setSize(inWidth, inHeight); 
}

// ---------------------------------------------------------------------------------
//		 CViewPortElement::setViewBox
// ---------------------------------------------------------------------------------

void CViewPortElement::setViewBox(double inMinX, double inMinY, 
									double inWidth, double inHeight)
{
	mHasViewBox = true;
	mViewBoxMinX = inMinX;
	mViewBoxMinY = inMinY;
	mViewBoxWidth = inWidth;
	mViewBoxHeight = inHeight;
}

#pragma mark === CFigureElement ===
// ---------------------------------------------------------------------------------
//		 CFigureElement::CFigureElement
// ---------------------------------------------------------------------------------

CFigureElement::CFigureElement(char *inName, char *inTitle) :
			CViewPortElement()
{
	mName = NULL;
	mTitle = NULL;
	
	setName(inName);
	setTitle(inTitle);
	
	// By default, make it about A4
	mPageUnits = kCMPageUnits;
	mWidth = 20.0;
	mHeight = 28.0;
}

// ---------------------------------------------------------------------------------
//		 CFigureElement::CFigureElement
// ---------------------------------------------------------------------------------

CFigureElement::CFigureElement(const CFigureElement &inSource) :
			CViewPortElement(inSource)
{
	mName = NULL;
	mTitle = NULL;
	
	setName(inSource.mName);
	setTitle(inSource.mTitle);
}

// ---------------------------------------------------------------------------------
//		 CFigureElement::~CFigureElement
// ---------------------------------------------------------------------------------

CFigureElement::~CFigureElement() 
{
	if (mName != NULL)
		delete [] mName;

	if (mTitle != NULL)
		delete [] mTitle;
}

// ---------------------------------------------------------------------------------
//		 CFigureElement::setName
// ---------------------------------------------------------------------------------

void CFigureElement::setName(char *inName)
{
	if (mName != NULL) {
		delete [] mName;
		mName = NULL;
	}
	
	if (inName != NULL) {
		int len = strlen(inName);
		if (len > 0) {
			NewArray(mName, char, len+1);
			strcpy(mName, inName);
		}
	}
}

// ---------------------------------------------------------------------------------
//		 CFigureElement::setTitle
// ---------------------------------------------------------------------------------

void CFigureElement::setTitle(char *inTitle)
{
	if (mTitle != NULL) {
		delete [] mTitle;
		mTitle = NULL;
	}
	
	if (inTitle != NULL) {
		int len = strlen(inTitle);
		if (len > 0) {
			NewArray(mTitle, char, len+1);
			strcpy(mTitle, inTitle);
		}
	}
}

// ---------------------------------------------------------------------------------
//		 CFigureElement::fitFigureInBounds
// ---------------------------------------------------------------------------------

void CFigureElement::fitFigureInBounds()
{
	double contentsWidth = mRight - mLeft;
	double contentsHeight = mBottom - mTop;
	
	double scaleX = mWidth / contentsWidth;
	double scaleY = mHeight / contentsHeight;
	double ofsX, ofsY;
	
	if (mPreserveAspectRatio != kNoneAspect) {
		double scale = (scaleX < scaleY ? scaleX : scaleY);
		
		double w = contentsWidth * scale;
		double h = contentsHeight * scale;
	
		switch (mPreserveAspectRatio) {
			case kXMinYMinAspect: 
			case kXMinYMidAspect:
			case kXMinYMaxAspect: 
				ofsX = -mLeft * scale;
			break;
			case kXMidYMinAspect:
			case kXMidYMidAspect:
			case kXMidYMaxAspect:
				ofsX = (-mLeft * scale) + ((mWidth - w) / 2.0);
			break;
			case kXMaxYMinAspect:
			case kXMaxYMidAspect:
			case kXMaxYMaxAspect:
				ofsX = -mRight * scale;
			break;
		}
		switch (mPreserveAspectRatio) {
			case kXMinYMinAspect: 
			case kXMidYMinAspect:
			case kXMaxYMinAspect:
				ofsY = -mTop * scale;
			break;
			case kXMinYMidAspect:
			case kXMidYMidAspect:
			case kXMaxYMidAspect:
				ofsY = (-mTop * scale) + ((mHeight - h) / 2.0);
			break;
			case kXMinYMaxAspect:
			case kXMidYMaxAspect:
			case kXMaxYMaxAspect:
				ofsY = -mBottom * scale;
			break;
		}
		
		scaleX = scale;
		scaleY = scale;
	} else {
		ofsX = -mLeft * scaleX;
		ofsY = -mTop * scaleY;
	}

	setScale(scaleX, scaleY);
	setTranslation(ofsX, ofsY);
}

