/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CHTMLReporter.cpp

				CHTMLReporter Class
	
 ******************************************************************************/

#include <string.h>
#include <iostream>
#include <fstream>

#include "CExceptions.h"

#include "CWriter.h"
#include "CTextWriter.h"
#include "CReportRecord.h"
#include "CGraphicsRecord.h"
#include "CHTMLReporter.h"
#include "CSVGReporter.h"

#pragma mark === CHTMLReporter ===

// ---------------------------------------------------------------------------------
//		 CHTMLReporter
// ---------------------------------------------------------------------------------

CHTMLReporter::CHTMLReporter(CWriter *inWriter, bool inOwner) : CReporter(inWriter, inOwner)
{
	mIsReporting = false;
	mFigureCount = 0;
}

CHTMLReporter::~CHTMLReporter()
{
	if (mIsReporting)
		finishReport();
}

// ---------------------------------------------------------------------------------
//		 CHTMLReporter::startReport
// ---------------------------------------------------------------------------------

void CHTMLReporter::startReport(char *inTitle)
{
	writer->writeString("<html>\n"); 
	writer->writeString("<head>\n"); 
	
	if (inTitle != NULL && strlen(inTitle) > 0) {
		writer->writeString("<title>"); 
		writer->writeString(inTitle); 
		writer->writeString("</title>\n"); 
	}
	writer->writeString("</head>\n"); 
	writer->writeString("<body>\n"); 
	
	mIsReporting = true;
}

// ---------------------------------------------------------------------------------
//		 CHTMLReporter::finishReport
// ---------------------------------------------------------------------------------

void CHTMLReporter::finishReport()
{
	writer->writeString("</body>\n"); 	
	writer->writeString("</html>\n"); 
	
	mIsReporting = false;
}

// ---------------------------------------------------------------------------------
//		 CHTMLReporter::writeReport
// ---------------------------------------------------------------------------------

void CHTMLReporter::writeReport(CReportRecord *inReportRecord)
{
	ThrowIfNull(inReportRecord);
	
	char *title = inReportRecord->getTitle();
	
	if (mIsReporting)
		startReport(title);

	if (title != NULL && strlen(title) > 0) {
		writer->writeString("<H1>"); 
		writer->writeString(title); 
		writer->writeString("</H1>\n"); 
	}

	CReporter::writeReport(inReportRecord);
}

// ---------------------------------------------------------------------------------
//		 CHTMLReporter::writeOpenStyle
// ---------------------------------------------------------------------------------

void CHTMLReporter::writeOpenStyle(CStyledElement *inElement)
{
	if (inElement->isItalic())
		writer->writeString("<em>");
		
	if (inElement->isBold())
		writer->writeString("<strong>");
}

// ---------------------------------------------------------------------------------
//		 CHTMLReporter::writeCloseStyle
// ---------------------------------------------------------------------------------

void CHTMLReporter::writeCloseStyle(CStyledElement *inElement)
{
	if (inElement->isItalic())
		writer->writeString("</em>");
		
	if (inElement->isBold())
		writer->writeString("</strong>");
}

// ---------------------------------------------------------------------------------
//		 CHTMLReporter::writeOpenLink
// ---------------------------------------------------------------------------------

void CHTMLReporter::writeOpenLink(CReportElement *inElement)
{
	if (inElement->getLink() != NULL) {
		writer->writeString("<a href=\"");
		writer->writeString(inElement->getLink());
		writer->writeString("\">");
	}
}

// ---------------------------------------------------------------------------------
//		 CHTMLReporter::writeCloseLink
// ---------------------------------------------------------------------------------

void CHTMLReporter::writeCloseLink(CReportElement *inElement)
{
	if (inElement->getLink() != NULL)
		writer->writeString("</a>");
}

// ---------------------------------------------------------------------------------
//		 CHTMLReporter::writeSimpleElement
// ---------------------------------------------------------------------------------

void CHTMLReporter::writeSimpleElement(char *inTag, ElementAlignment inAlign, CSimpleElement *inElement, bool inIndent)
{
	writeOpenStyle(inElement);
	writer->writeString("<"); 
	writer->writeString(inTag);
	
	if (inAlign != kAlignElementDefault) {
		writer->writeString(" align=\""); 	
		switch (inAlign) {
			case kAlignElementLeft:	writer->writeString("left\""); break;
			case kAlignElementCentre:	writer->writeString("center\""); break;
			case kAlignElementRight:	writer->writeString("right\""); break;
			case kAlignElementJustify:	writer->writeString("justify\""); break;
		}
	}		
	if (inIndent)
		writer->writeString(">\n");
	else
		writer->writeString(">");
	
	writeElement(inElement);
	
	if (inIndent)
		writer->writeString("\n</"); 
	else
		writer->writeString("</"); 
	writer->writeString(inTag);
	writer->writeString(">\n"); 
	writeCloseStyle(inElement);
}

// ---------------------------------------------------------------------------------
//		 CHTMLReporter::writeComplexElement
// ---------------------------------------------------------------------------------

void CHTMLReporter::writeComplexElement(char *inTag, ElementAlignment inAlign, CComplexElement *inElement, bool inIndent)
{
	writeOpenStyle(inElement);
	writer->writeString("<"); 
	writer->writeString(inTag);
	
	if (inElement->getAnchor() != NULL) {
		writer->writeString(" id=\"");
		writer->writeString(inElement->getAnchor());
		writer->writeString("\"");
	}
	
	if (inAlign != kAlignElementDefault) {
		writer->writeString(" align=\""); 	
		switch (inAlign) {
			case kAlignElementLeft:	writer->writeString("left\""); break;
			case kAlignElementCentre:	writer->writeString("center\""); break;
			case kAlignElementRight:	writer->writeString("right\""); break;
			case kAlignElementJustify:	writer->writeString("justify\""); break;
		}
	}		
	if (inIndent)
		writer->writeString(">\n");
	else
		writer->writeString(">");
	
	if (inElement->getElementType() == kParagraphElement) {
		int i, n = inElement->getNumElements();
		for (i = 0; i < n; i++) {
			writeElement(inElement->getElement(i));
			writer->writeString(" ");
		}
	} else
		writeElement(inElement);
	
	if (inIndent)
		writer->writeString("\n</"); 
	else
		writer->writeString("</"); 
	writer->writeString(inTag);
	writer->writeString(">\n"); 
	writeCloseStyle(inElement);
}

// ---------------------------------------------------------------------------------
//		 CHTMLReporter::writeFilteredString
// ---------------------------------------------------------------------------------
// Writes a string, filtering for characters that are HTML important.

void CHTMLReporter::writeFilteredString(const char *st)
{
	const char *P = st;
	while (*P != '\0') {
		switch (*P) {
			case '"': writer->writeString("&quot;"); break;
			case '&': writer->writeString("&amp;"); break; 
			case '<': writer->writeString("&lt;"); break;
			case '>': writer->writeString("&gt;"); break; 
			default:
				writer->writeChar(*P);
		}
		P++;
	}
}

// ---------------------------------------------------------------------------------
//		 CHTMLReporter::writeText
// ---------------------------------------------------------------------------------

void CHTMLReporter::writeText(CTextElement *inElement)
{
	writeOpenLink(inElement);
	writeOpenStyle(inElement);
	writeFilteredString(inElement->getText());
	writeCloseStyle(inElement);
	writeCloseLink(inElement);
}

// ---------------------------------------------------------------------------------
//		 CHTMLReporter::writeLineBreak
// ---------------------------------------------------------------------------------

void CHTMLReporter::writeLineBreak(CLineBreakElement *inElement)
{
	writer->writeString("<br>\n");
}

// ---------------------------------------------------------------------------------
//		 CHTMLReporter::writeInteger
// ---------------------------------------------------------------------------------

void CHTMLReporter::writeInteger(CIntegerElement *inElement)
{
	writeOpenLink(inElement);
	writeOpenStyle(inElement);
	writer->writeInteger(inElement->getValue());
	writeCloseStyle(inElement);
	writeCloseLink(inElement);
}

// ---------------------------------------------------------------------------------
//		 CHTMLReporter::writeReal
// ---------------------------------------------------------------------------------

void CHTMLReporter::writeReal(CRealElement *inElement)
{
	writeOpenLink(inElement);
	writeOpenStyle(inElement);
	writer->writeDouble(inElement->getValue(), inElement->getPrecision(), inElement->getIsSciFormat());
	writeCloseStyle(inElement);
	writeCloseLink(inElement);
}

// ---------------------------------------------------------------------------------
//		 CHTMLReporter::writeToken
// ---------------------------------------------------------------------------------

void CHTMLReporter::writeToken(CTokenElement *inElement)
{
	writeOpenLink(inElement);
	writeOpenStyle(inElement);
	int i, n = inElement->getNumElements();
	for (i = 0; i < n; i++) 
		writeElement(inElement->getElement(i));
	writeCloseStyle(inElement);
	writeCloseLink(inElement);
}

// ---------------------------------------------------------------------------------
//		 CHTMLReporter::writeParagraph
// ---------------------------------------------------------------------------------

void CHTMLReporter::writeParagraph(CParagraphElement *inElement)
{
	writeOpenLink(inElement);
	switch (inElement->getHeadingLevel()) {
		case kNormalText: 		writeComplexElement("p", inElement->getAlignment(), inElement); break;
		case kHeadingLevel1: 	writeComplexElement("h1", inElement->getAlignment(), inElement); break;
		case kHeadingLevel2: 	writeComplexElement("h2", inElement->getAlignment(), inElement); break;
		case kHeadingLevel3: 	writeComplexElement("h3", inElement->getAlignment(), inElement); break;
		case kHeadingLevel4: 	writeComplexElement("h4", inElement->getAlignment(), inElement); break;
		case kHeadingLevel5: 	writeComplexElement("h5", inElement->getAlignment(), inElement); break;
		case kHeadingLevel6: 	writeComplexElement("h6", inElement->getAlignment(), inElement); break;
	}
	writeCloseLink(inElement);
}

// ---------------------------------------------------------------------------------
//		 CHTMLReporter::writePreformatted
// ---------------------------------------------------------------------------------

void CHTMLReporter::writePreformatted(CPreformattedElement *inElement)
{
	writeOpenLink(inElement);
	writeComplexElement("pre", inElement->getAlignment(), inElement);
	writeCloseLink(inElement);
}

// ---------------------------------------------------------------------------------
//		 CHTMLReporter::writeTable
// ---------------------------------------------------------------------------------

void CHTMLReporter::writeTable(CTableElement *inElement)
{
	CComplexElement *cell;
	
	writeOpenLink(inElement);
	writeOpenStyle(inElement);

	writer->writeString("<table>\n"); 
	int i, j;
	for (i = 0; i < inElement->getNumRows(); i++) {
		writer->writeString("<tr>\n"); 
		for (j = 0; j < inElement->getNumCols(); j++) {
			cell = inElement->getCell(i, j);
			if (cell != NULL)
				writeComplexElement("td", cell->getAlignment(), cell);
		}
		writer->writeString("</tr>\n");
	}
	writer->writeString("</table>\n");

	writeCloseStyle(inElement);
	writeCloseLink(inElement);
}

// ---------------------------------------------------------------------------------
//		 CHTMLReporter::writeList
// ---------------------------------------------------------------------------------

void CHTMLReporter::writeList(CListElement *inElement)
{
	CComplexElement *cell;
	
	writeOpenLink(inElement);
	writeOpenStyle(inElement);

	switch (inElement->getListType()) {
		case kBullettedList: 	writer->writeString("<ul>\n"); break;
		case kNumberedList: 	writer->writeString("<ol>\n"); break;
		case kDefinitionList: 	writer->writeString("<dl>\n"); break;
	}

	int i, n = inElement->getNumElements();
	if (inElement->getListType() == kDefinitionList) {
		i = 0;
		while (i < n) {
			cell = inElement->getCell(i);
			writeComplexElement("dt", cell->getAlignment(), cell);
			
			i++;
			if (i < n) {
				cell = inElement->getCell(i);
				writeComplexElement("dd", cell->getAlignment(), cell);
			} else
				writer->writeString("<dd></dd>\n");
			i++;
		}
	} else {
		for (i = 0; i < n; i++) {
			cell = inElement->getCell(i);
			writeComplexElement("li", cell->getAlignment(), cell);
		}
	}

	switch (inElement->getListType()) {
		case kBullettedList: 	writer->writeString("</ul>\n"); break;
		case kNumberedList: 	writer->writeString("</ol>\n"); break;
		case kDefinitionList: 	writer->writeString("</dl>\n"); break;
	}

	writeCloseStyle(inElement);
	writeCloseLink(inElement);
}

// ---------------------------------------------------------------------------------
//		 CHTMLReporter::writeFigure
// ---------------------------------------------------------------------------------

void CHTMLReporter::writeFigure(CFigureElement *inElement)
{
	char filename[256];
	
	mFigureCount++;

	if (inElement->getName() != NULL)
		sprintf(filename, "figure_%d_%s.svg", mFigureCount, inElement->getName());
	else
		sprintf(filename, "figure_%d.svg", mFigureCount);
	
	writeOpenLink(inElement);

	writer->writeString("<OBJECT WIDTH=\"");
	writer->writeDouble(inElement->getWidth());
	writer->writeString("\" HEIGHT=\"");
	writer->writeDouble(inElement->getHeight());
	writer->writeString("\" DATA=\"");
	writer->writeString(filename);
	writer->writeString("\" TYPE=\"image/svg+xml\"");
	writer->writeString("\" PLUGINSPAGE=\"http://www.adobe.com/svg/viewer/install/\">");
	writer->writeString("This is an SVG figure - <A HREF=\"http://www.adobe.com/svg/viewer/install/\">click to get SVG plug-in</A></OBJECT>\n");

	writeCloseLink(inElement);

	std::ofstream* stream = new std::ofstream(filename); 
	if (stream==NULL || stream->fail() || stream->bad()) {
		CComplexElement *e = new CComplexElement();
		e->add("Unable create file - '");
		e->add(filename);
		e->add("'");
		
		throw CException(e);
	}
    CTextWriter* writer = new CTextWriter(stream, true);
   	CSVGReporter *reporter = new CSVGReporter(writer, true);
	reporter->writeFigure(inElement);

	delete reporter;
}

