/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CLinePlot.cpp
 
		CLinePlot Class
	 
 ******************************************************************************/

#include "CGraph.h"
#include "CLinePlot.h"
#include "CDataSeries.h"
#include "CLogAxis.h"
#include "CDoubleArray.h"

#include "CGraphicsRecord.h"

#pragma mark === Construction & Destruction ===

// ---------------------------------------------------------------------------------
//		 CLinePlot()
// ---------------------------------------------------------------------------------

CLinePlot::CLinePlot( ) : CPlot( )
{
	mLineGraphic = new CLineElement(0.0,0.0,0.0,0.0);
	mLineGraphic->setStrokeStyle(1.0, kRoundLineCap, kRoundLineJoin);
	mLineGraphic->setStrokeColour(kBlackColour);
	mLineGraphic->setFillColour(kNoColour);
}

#pragma mark === Drawing ===

//------------------------------------------------------------------------------------
// 		CLinePlot::DrawDataSeries
//-------------------------------------------------------------------------------------

void CLinePlot::DrawDataSeries(CComplexGraphicElement *ioGraphic, 
								CDataSeries *inData,
								double inXTolerance, 
								double inYTolerance)
{
	CDoubleArray *xData = inData->GetXData();
	CDoubleArray *yData = inData->GetYData();
	
	CPathElement *path = new CPathElement(0.0, 0.0);
	
	if (mLineGraphic != NULL)
		path->setStyle(*mLineGraphic);
		
	double lastX, lastY;
	double nextX, nextY;
	
	bool done = true;
	
	int j = 0;
	do {
		lastX = xData->getValue(j);
		lastY = yData->getValue(j);
		j++;
		
		try {
			path->setPosition(	mGraph->ConvertX(lastX), 
								mGraph->ConvertY(lastY));
			done = true;
			
		} catch (CIllegalLogValue) {
		
			done = false;						
		}			
	} while (!done);
	
	int n = xData->getNumValues();
	for (int i = j; i < n; i++) {
		nextX = xData->getValue(i);
		nextY = yData->getValue(i);
		
		if ((nextX - lastX) >= inXTolerance || 
			(nextY - lastY) >= inYTolerance) {
			try {
				path->lineTo(	mGraph->ConvertX(nextX), 
								mGraph->ConvertY(nextY));
			} catch (CIllegalLogValue) {
			}
			
			lastX = nextX;
			lastY = nextY;		
		}
	}
	path->setPathClosed(false);
	ioGraphic->add(path);
}


