/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CLogAxis.h
 
		CLogAxis Class
	 
 ******************************************************************************/

#ifndef _CLOGAXIS_H_
#define _CLOGAXIS_H_

#include "CAxis.h"
		
#include "CExceptions.h"

//: Thrown when a zero or negative value appears on a log axis.
class CIllegalLogValue : public CException {  
	public:
		CIllegalLogValue() : CException() { };
};

class CLogAxis : public CAxis {

protected:												
	//====<<  FIELDS  >>=============================================================

	int numExtraMinorTicks;
	
public:
	//====<<  METHODS  >>============================================================
		
	//----<<  INITIALIZATION & DISPOSAL  >>------------------------------------------

 						CLogAxis( );
 						CLogAxis(AxisFlag inMinAxisFlag, AxisFlag inMaxAxisFlag);
 						CLogAxis(AxisFlag inMinAxisFlag, AxisFlag inMaxAxisFlag, 
 								bool inIsDescrete);

	//----<<  ACCESSING  >>----------------------------------------------------------
		
	virtual	bool		GetUseScientific() { return true; }

	virtual	void		SetRange(CDoubleArray *inData);
	virtual	void		AddRange(CDoubleArray *inData);
	virtual	void		SetRange(double inMinValue, double inMaxValue);
	virtual	void		AddRange(double inMinValue, double inMaxValue);
	
	virtual double 		Transform(double inValue);
	
	virtual int 		GetNumMinorTicks(int majorTickNo);
	
	virtual double 		GetMajorTick(int majorTickNo);
	virtual double 		GetMinorTick(int minorTickNo, int majorTickNo);

protected:
	//----<<  CALIBRATING  >>--------------------------------------------------------

	virtual	void		SetMinTick();
	virtual	void		SetMaxTick();
	virtual	void		SetMajorTick();
	virtual	void		SetMinorTick();
	virtual	void		HandleAxisFlags();
	
};

#endif /* _CLOGAXIS_H_ */
