/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CObject.cpp

 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = Global

//	The base for all data objects

#include "CObject.h"
#include "CString.h"

#include "CExceptions.h"

#pragma mark === CObject ===

// ---------------------------------------------------------------------------------
//		CObject::CObject
// ---------------------------------------------------------------------------------

//: the constructor
//  Constructs an CObject with no name, description or owner
CObject::CObject()
{
	mReferenceCount = 1;
	mName = NULL;
}

// ---------------------------------------------------------------------------------
//		CObject::~CObject
// ---------------------------------------------------------------------------------

//: The destructor
//  Deletes the name and description if present
CObject::~CObject()
{
	if (mName != NULL)
		delete mName;
}

// ---------------------------------------------------------------------------------
//		CObject::retain
// ---------------------------------------------------------------------------------

//: Adds a reference to this object
//  Increments the references count by 1. This is a mechanism by which other
//  objects can register that they are using an object. When they are finished
//  they should call release. If the references count hits 0 the object deletes itself.

void CObject::retain()
{
	mReferenceCount++;
}

// ---------------------------------------------------------------------------------
//		CObject::release
// ---------------------------------------------------------------------------------

//: Removes a reference to this object
//  Decrements the references count by 1. This is a mechanism by which other
//  objects can register that they are using an object. When they are finished
//  they should call release. If the references count hits 0 the object deletes itself.

void CObject::release()
{
	mReferenceCount--;
	if (mReferenceCount < 1)
		delete this;
}

// ---------------------------------------------------------------------------------
//		CObject::setName
// ---------------------------------------------------------------------------------

//: Sets the name field of this object
void CObject::setName(char *inName)
{
	if (mName != NULL)
		mName->setString(inName);
	else
		NewClass(mName, CString(inName));
}

// ---------------------------------------------------------------------------------
//		CObject::getName
// ---------------------------------------------------------------------------------
		
//: Returns the name of this object
//  If null, then this object has no name.
char* CObject::getName()
{
	if (mName != NULL)
		return mName->getString();
	else
		return NULL;
}

// ---------------------------------------------------------------------------------
//		CObject::getName
// ---------------------------------------------------------------------------------
		
//: Copies the name of this object into a string
//  Writes the name of this object into ioName which should contain enough space.
void CObject::getName(char *outName)
{
	if (mName != NULL)
		mName->getString(outName);
	else
		outName[0] = '\0';
}

