/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CObject.h

		Interface for the CObject Class

 ******************************************************************************/
//! author = "Andrew Rambaut"
//! library = Core
//! group = Global
 
#ifndef _COBJECT_H_
#define _COBJECT_H_

class CString;

/******************************************************************************
	class CObject
 ******************************************************************************/
#pragma mark CObject class

//: The base for all data objects

class CObject {
	public:
		virtual char*		getClassName() const { return "Object"; }
		//: Returns the name of this class - "Object"
		//  A class name is character string which gives a unique name for
		//	CObject and its descendants.
	
	protected:
		short				mReferenceCount;	//: Number of objects referencing this one
		
		CString				*mName;

	public:
	 						CObject();		//: The constructor
	 	virtual				~CObject();	//: The destructor

		virtual void 		retain();
		//: Adds a reference to this object
		
		virtual void 		release();
		//: Removes a reference to this object

		virtual short 		getReferenceCount() { return mReferenceCount; }
		//: Returns the number of references to this object

		virtual void 		setName(char *inName);
		//: Sets the name field of this object

		virtual char*		getName();
		//: Returns a pointer to the name of this object

		virtual void		getName(char *outName);
		//: Copies the name of this object into a string

};

#endif /* _COBJECT_H_ */
