/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CPlot.h
 
		CPlot Class
	 
 ******************************************************************************/

#ifndef _CPLOT_H_
#define _CPLOT_H_

#include "CGraphicsRecord.h"

class CGraph;
class CAxis;
class CDataSeries;

enum PlotMarkType {
	kNoMarks,
	kCrossMarks,
	kPlusMarks,
	kCircleMarks,
	kSquareMarks,
	kDiamondMarks
};

class CPlot {

	//====<<  FIELDS  >>=============================================================

protected:												
	CGraph 				*mGraph;
	
	CAxis				*mXAxis, *mYAxis;
	CDataSeries			*mData;
	bool				mIsDataOwner;
	
	bool 				mShowOrigin;
	
	CGraphicElement* 	mMarkGraphic;
	CGraphicElement* 	mLineGraphic;

public:
	//====<<  METHODS  >>============================================================
		
	//----<<  INITIALIZATION & DISPOSAL  >>------------------------------------------

 						CPlot( );
 	virtual				~CPlot( );

	//----<<  ACCESSING  >>----------------------------------------------------------
		
	virtual	void		SetAxes(CAxis *inXAxis, CAxis *inYAxis);
	virtual	void		SetGraph(CGraph *inGraph);
	
	virtual	void		SetData(CDataSeries *inData);
	virtual	CDataSeries *GetData() { return mData; }
	
	virtual	void		SetShowOrigin(bool inShowOrigin) { mShowOrigin = inShowOrigin; }

	virtual	void		SetLineStyle(double inWidth, StrokeLineCapType inLineCap, 
										StrokeLineJoinType inLineJoin, CColourRec &inColour);

	virtual	void		SetMarkStyle(PlotMarkType inMarkType,
										double inMarkWidth, double inStrokeWidth, 
										CColourRec &inStrokeColour, CColourRec &inFillColour);

	virtual	void		SetMarkGraphic(CGraphicElement* inMarkGraphic);
	virtual	CGraphicElement*	GetMarkGraphic() { return mMarkGraphic; }
	virtual	CGraphicElement*	GetLineGraphic() { return mLineGraphic; }


	//----<<  DRAWING  >>--------------------------------------------------------
	
	virtual	void		DrawMark(CComplexGraphicElement *ioGraphic, CGraphicElement *inMarkGraphic,
									double x, double y);
	virtual	void		DrawLine(CComplexGraphicElement *ioGraphic, CGraphicElement *inLineStyle,
									double x1, double y1, double x2, double y2);
	virtual	void		DrawRect(CComplexGraphicElement *ioGraphic, CGraphicElement *inRectStyle,
									double x1, double y1, double x2, double y2);
	
	virtual	void		DrawDataSeries(CComplexGraphicElement *ioGraphic, 
									CDataSeries *inData, 
									double inXTolerance = 0.0, 
									double inYTolerance = 0.0 ) = 0;
	
	virtual	void		DrawPlot(CComplexGraphicElement *ioGraphic, 
									double inXTolerance = 0.0, 
									double inYTolerance = 0.0 );
};

#endif /* _CPLOT_H_ */
