/******************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CRandom.h
 	
	Random Number Generator.

 ******************************************************************/


//  includes:
#include <ctype.h>
#include <math.h>

#include "CRandom.h"

// methods:

#pragma mark _________ CRandom _________

// ===========================================================================================
// 								CRandom static class 
// ===========================================================================================

long int		CRandom::mSeed = -1;
int				CRandom::mZRndU = 137;
unsigned int	CRandom::mWRndU = 13757;


// -------------------------------------------------------------------------------------------
//   CRandom::setSeed
// -------------------------------------------------------------------------------------------

void CRandom::setSeed(long int inSeed)
{
   mZRndU = 170 * (inSeed % 178) + 137;
   mWRndU = (unsigned int) inSeed;
   mSeed = inSeed;
}

// -------------------------------------------------------------------------------------------
//   CRandom::slowUniform
// -------------------------------------------------------------------------------------------
double CRandom::slowUniform()
{
   static int x_rndu=11, y_rndu=23;
   double r;
   
   x_rndu = 171*(x_rndu%177) -  2*(x_rndu/177);
   y_rndu = 172*(y_rndu%176) - 35*(y_rndu/176);
   mZRndU = 170*(mZRndU%178) - 63*(mZRndU/178);
   if (x_rndu<0) x_rndu+=30269;
   if (y_rndu<0) y_rndu+=30307;
   if (mZRndU<0) mZRndU+=30323;
   r = x_rndu/30269.0 + y_rndu/30307.0 + mZRndU/30323.0;
   
   return (r-(int)r);
}

// -------------------------------------------------------------------------------------------
//   CRandom::fastUniform
// -------------------------------------------------------------------------------------------
double CRandom::fastUniform()
{
   mWRndU *= 127773;
   return ldexp((double)mWRndU, -32);
}

 // -------------------------------------------------------------------------------------------
//   CRandom::logRan
// -------------------------------------------------------------------------------------------
double CRandom::logRan(double x)
{
	double y;
	do {
		y = CRandom::fRan();
	} while (y == 0.0);
	return (-x * log(y));

}

// -------------------------------------------------------------------------------------------
//   CRandom::sampleBinomial
// -------------------------------------------------------------------------------------------
int CRandom::sampleBinomial(int n, double p)
//: Sample from n trials each of which has probability p of success and return the number
// of successes.
{
	int i, j = 0;
	for (i = 0; i < n; i++)
		j += CRandom::bRan(p);
	return j;
}


// -------------------------------------------------------------------------------------------
//   CRandom::sampleLogNormal
// -------------------------------------------------------------------------------------------
double CRandom::sampleLogNormal(double mean, double stdDev)
{
	double temp = CRandom::sampleNormal(mean, stdDev);
	return exp(temp);
}


// -------------------------------------------------------------------------------------------
//   CRandom::sampleNormal
// -------------------------------------------------------------------------------------------
double CRandom::sampleNormal(double mean, double stdDev)
{
	double
		x1, x2, fac, rsq, bm;
	static double
		gset;
	static int
		iset = 0;
	
	if (iset == 0) {
		do {
			x1 = 2*CRandom::fRan() - 1.0;
			x2 = 2*CRandom::fRan() - 1.0;
			rsq = x1*x1+x2*x2;
		} while (rsq >= 1.0 || rsq == 0.0);
		fac = sqrt(-2*log(rsq)/rsq);
		gset = x1*fac;
		iset = 1;
		bm = x2*fac;
	}
	else {
		iset = 0;
		bm = gset;
	}
	bm = bm*stdDev + mean;
	return bm;
}


// -------------------------------------------------------------------------------------------
//   CRandom::sampleUniform
// -------------------------------------------------------------------------------------------
double CRandom::sampleUniform(double minVal, double maxVal)
//: return a random variate drawn from the uniform continuous distribution 
// on the interval (minVal, maxVal)
{
	return (CRandom::fRan(maxVal-minVal) + minVal);
}
