/******************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CRandom.h
 	
	Random Number Generator.
	
 ******************************************************************/

#ifndef _CRANDOM_H_
#define _CRANDOM_H_

#pragma mark // CRandom

class CRandom {
protected:
	static long int			mSeed;
	
	static int				mZRndU;
	static unsigned int		mWRndU;

public:
	static void				setSeed(long int inSeed);
	static long				getSeed() { return mSeed; }
	
	static double			slowUniform();
	static double			fastUniform();

	static double			fRan(double d = 1.0) 
		{
		#ifdef FAST_RANDOM
			return (long int) (CRandom::slowUniform() * d );
		#else
			return (long int) (CRandom::slowUniform() * d );
		#endif
		}

	static long int			iRan(long i) { return (long int) (CRandom::fRan() * i ); }
	//: return a randomly chosen number from the uniform discrete distribution on [0,x-1].

	static long int			iRan(int i) { return CRandom::iRan((long int) i); }
	//: return a randomly chosen number from the uniform discrete distribution on [0,x-1].

	static unsigned long	ulRan(unsigned long i) { return (unsigned long) CRandom::fRan() * i; }
	//: return a randomly chosen number from the uniform discrete distribution on [0,x-1].

	static double			logRan(double x);
	
	static bool				bRan(double p = 0.5) { return ( CRandom::fRan() < p ? true : false ); }	
	//: return a true with probability p.
	
	static int				sampleBinomial(int n, double p);
	//: Sample from n trials each of which has probability p of success and return the number of successes.

	static double			sampleLogNormal(double mean = 0.0, double stdDev = 1.0);
	//: Sample from the log normal distribution
	
	static double			sampleNormal(double mean = 0.0, double stdDev = 1.0);
	//: Sample from the normal distribution
	
	static double			sampleUniform(double minVal, double maxVal);
	//: return a random variate drawn from the uniform continuous distribution on the interval (minVal, maxVal)
};

#endif /* _CRANDOM_H_ */
