/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CReader.h

		Interface for the CReader Class
					
 ******************************************************************************/
 
#ifndef _CREADER_H_
#define _CREADER_H_

#include "CObject.h"

class CWriter;

/******************************************************************************
	class CReader
 ******************************************************************************/
#pragma mark CReader class

class CReader : public CObject {
	protected:
		bool 					firstBranch;
		char					startComment,  stopComment, lineComment, writeComment;
		CWriter					*commentWriter;
		
		int						mLineNumber;

	public:
	 							CReader();						
	 														
	protected:
		void 					initReader();
		
	public:
	 	virtual					~CReader();

		virtual void 			setCommentDelimiters(char start, char stop, char line = 0xFF, char write = 0xFF);
		virtual char 			getStartComment() { return startComment; };
		virtual char 			getStopComment() { return stopComment; };
		virtual char 			getLineComment() { return lineComment; };
		virtual void 			setCommentWriter(CWriter *inCommentWriter) 
									{ commentWriter = inCommentWriter; }

		virtual void 			setLineNumber(int inLineNumber) { mLineNumber = inLineNumber; }
		virtual int 			getLineNumber() { return mLineNumber; }

		// Abstract functions which must be implemented
		virtual void 			markPosition() = 0;
		virtual void 			restorePosition() = 0;
		
		virtual char 			readChar() = 0;
		virtual char 			nextChar() = 0;
		virtual void 			unreadChar(char ch) = 0;
		
		// Higher level functions that utilise the above
		virtual void 			skipComments();
		virtual void 			skipToEndOfLine();
		virtual int 			skipSpace();
		virtual int 			skipSpace(char *delimiters);
		virtual char 			skipSpace(char *delimiters, bool returnDelimiter);
		virtual int 			skipSpace(char *delimiters1, char *delimiters2);
		virtual char 			skipSpace(char *delimiters1, char *delimiters2, bool returnDelimiter);
		virtual char 			skipUntil(char delimiter);
		virtual char 			skipUntil(char *delimiters);
		virtual char 			skipWhile(char *delimiters);
		
		virtual int 			readLine(char *line, int maxLen, bool upper=false, bool skipBlank = true);
		virtual int 			readInteger();
		virtual char 			readInteger(int *d, char *delimiters);
		virtual double 			readDouble();
		virtual char 			readDouble(double *f, char *delimiters);
		
		virtual char 			readToken(char *token, int maxLen, bool upper=false, bool skipDelimiters = false);
		virtual char 			readToken(char *token, int maxLen, char *delimiters, bool upper=false, bool skipDelimiters = false);
		virtual char 			readString(char *token, int maxLen, char *delimiters, bool upper=false, bool ignoreQuotes = false);
		virtual void 			findToken(char *token, bool upper=false);
		virtual void 			findToken(char *token, char *delimiters, bool upper=false);
		
		virtual int 			readTaxonLabel(char *token, int maxLen, int maxIndex);
};

#endif /* _CREADER_H_ */
