/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CReportRecord.cpp

				CReportRecord Class
	
 ******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "CExceptions.h"

#include "CString.h"
#include "CReportRecord.h"

// ---------------------------------------------------------------------------------
//		 CReportElement::CReportElement
// ---------------------------------------------------------------------------------
#pragma mark === CReportElement ===

CReportElement::CReportElement() 
{
	mLink = NULL;
}

// ---------------------------------------------------------------------------------
//		 CReportElement::~CReportElement
// ---------------------------------------------------------------------------------

CReportElement::~CReportElement() 
{
	if (mLink != NULL)
		mLink->release();
}

// ---------------------------------------------------------------------------------
//		 CReportElement::setLink
// ---------------------------------------------------------------------------------

void CReportElement::setLink(const char *inLink)
{
	if (inLink != NULL) {
		if (mLink == NULL)
			mLink = new CString(inLink);
		else
			mLink->setString(inLink);
	}
}

// ---------------------------------------------------------------------------------
//		 CReportElement::getLink
// ---------------------------------------------------------------------------------

char* CReportElement::getLink()
{
	if (mLink != NULL)
		return mLink->getString();
	else
		return NULL;
}

// ---------------------------------------------------------------------------------
//		 CStyledElement::CStyledElement
// ---------------------------------------------------------------------------------
#pragma mark === CStyledElement ===

CStyledElement::CStyledElement() : CReportElement()
{
	mIsItalic = false;
	mIsBold = false;
}

// ---------------------------------------------------------------------------------
//		 CStyledElement::~CStyledElement
// ---------------------------------------------------------------------------------

CStyledElement::~CStyledElement() 
{
}

// ---------------------------------------------------------------------------------
//		 CStyledElement::setStyle
// ---------------------------------------------------------------------------------

void CStyledElement::setStyle(bool inIsItalic, bool inIsBold)
{
	mIsItalic = inIsItalic;
	mIsBold = inIsBold;
}

// ---------------------------------------------------------------------------------
//		 CSimpleElement::CSimpleElement
// ---------------------------------------------------------------------------------
#pragma mark === CSimpleElement ===

CSimpleElement::CSimpleElement(bool inIsItalic, bool inIsBold) :
			CStyledElement()
{
	setStyle(inIsItalic, inIsBold);
}

// ---------------------------------------------------------------------------------
//		 CSimpleElement::~CSimpleElement
// ---------------------------------------------------------------------------------

CSimpleElement::~CSimpleElement() 
{

}

// ---------------------------------------------------------------------------------
//		 CTextElement::CTextElement
// ---------------------------------------------------------------------------------
#pragma mark === CTextElement ===

CTextElement::CTextElement(const char *inText, bool inIsItalic, bool inIsBold) :
			CSimpleElement(inIsItalic, inIsBold)
{
	mText = NULL;
	setText(inText);
	

}

// ---------------------------------------------------------------------------------
//		 CTextElement::CTextElement
// ---------------------------------------------------------------------------------

CTextElement::CTextElement(const char *inText, const char *inLink, bool inIsItalic, bool inIsBold) :
			CSimpleElement(inIsItalic, inIsBold)
{
	mText = NULL;
	setText(inText);
	setLink(inLink);
}

// ---------------------------------------------------------------------------------
//		 CTextElement::~CTextElement
// ---------------------------------------------------------------------------------

CTextElement::~CTextElement() 
{
	if (mText != NULL)
		mText->release();
}

// ---------------------------------------------------------------------------------
//		 CTextElement::setText
// ---------------------------------------------------------------------------------

void CTextElement::setText(const char *inText)
{
	if (inText != NULL) {
		if (mText == NULL)
			mText = new CString(inText);
		else
			mText->setString(inText);
	}
}

// ---------------------------------------------------------------------------------
//		 CTextElement::getText
// ---------------------------------------------------------------------------------

char* CTextElement::getText()
{
	if (mText != NULL)
		return mText->getString();
	else
		return NULL;
}

// ---------------------------------------------------------------------------------
//		 CIntegerElement::CIntegerElement
// ---------------------------------------------------------------------------------
#pragma mark === CIntegerElement ===

CIntegerElement::CIntegerElement(int inValue, bool inIsItalic, bool inIsBold) :
			CSimpleElement(inIsItalic, inIsBold)
{
	mValue = inValue;
}

// ---------------------------------------------------------------------------------
//		 CIntegerElement::CIntegerElement
// ---------------------------------------------------------------------------------

CIntegerElement::CIntegerElement(int inValue, const char *inLink, bool inIsItalic, bool inIsBold) :
			CSimpleElement(inIsItalic, inIsBold)
{
	mValue = inValue;
	setLink(inLink);
}

// ---------------------------------------------------------------------------------
//		 CIntegerElement::CIntegerElement
// ---------------------------------------------------------------------------------

CIntegerElement::CIntegerElement(long inValue, bool inIsItalic, bool inIsBold) :
			CSimpleElement(inIsItalic, inIsBold)
{
	mValue = inValue;
}

// ---------------------------------------------------------------------------------
//		 CIntegerElement::CIntegerElement
// ---------------------------------------------------------------------------------

CIntegerElement::CIntegerElement(long inValue, const char *inLink, bool inIsItalic, bool inIsBold) :
			CSimpleElement(inIsItalic, inIsBold)
{
	mValue = inValue;
	setLink(inLink);
}

// ---------------------------------------------------------------------------------
//		 CIntegerElement::~CIntegerElement
// ---------------------------------------------------------------------------------

CIntegerElement::~CIntegerElement() 
{
}

// ---------------------------------------------------------------------------------
//		 CRealElement::CRealElement
// ---------------------------------------------------------------------------------
#pragma mark === CRealElement ===

CRealElement::CRealElement(double inValue, bool inIsItalic, bool inIsBold) :
			CSimpleElement(inIsItalic, inIsBold)
{
	mValue = inValue;
	mPrecision = -1;
	mSciFormat = false;
}

CRealElement::CRealElement(double inValue, int inPrecision, bool inIsItalic, bool inIsBold) :
			CSimpleElement(inIsItalic, inIsBold)
{
	mValue = inValue;
	mPrecision = inPrecision;
	mSciFormat = false;
}

CRealElement::CRealElement(double inValue, int inPrecision, bool inSciFormat, bool inIsItalic, bool inIsBold) :
			CSimpleElement(inIsItalic, inIsBold)
{
	mValue = inValue;
	mPrecision = inPrecision;
	mSciFormat = inSciFormat;
}

// ---------------------------------------------------------------------------------
//		 CRealElement::CRealElement
// ---------------------------------------------------------------------------------

CRealElement::CRealElement(double inValue, const char *inLink, bool inIsItalic, bool inIsBold) :
			CSimpleElement(inIsItalic, inIsBold)
{
	mValue = inValue;
	mPrecision = -1;
	mSciFormat = false;
	setLink(inLink);
}

CRealElement::CRealElement(double inValue, int inPrecision, const char *inLink, bool inIsItalic, bool inIsBold) :
			CSimpleElement(inIsItalic, inIsBold)
{
	mValue = inValue;
	mPrecision = inPrecision;
	mSciFormat = false;
	setLink(inLink);
}

CRealElement::CRealElement(double inValue, int inPrecision, bool inSciFormat, const char *inLink, bool inIsItalic, bool inIsBold) :
			CSimpleElement(inIsItalic, inIsBold)
{
	mValue = inValue;
	mPrecision = inPrecision;
	mSciFormat = inSciFormat;
	setLink(inLink);
}

// ---------------------------------------------------------------------------------
//		 CRealElement::~CRealElement
// ---------------------------------------------------------------------------------

CRealElement::~CRealElement() 
{
}

// ---------------------------------------------------------------------------------
//		 CComplexElement::CComplexElement
// ---------------------------------------------------------------------------------
#pragma mark === CComplexElement ===

CComplexElement::CComplexElement(ElementAlignment inAlignment, int inIndentLevel) :
			CStyledElement()
{
	mElements = new CList();
	
	mAlignment = inAlignment;	
	mIndentLevel = inIndentLevel;
	
	mAnchor = NULL;
}

// ---------------------------------------------------------------------------------
//		 CComplexElement::~CComplexElement
// ---------------------------------------------------------------------------------

CComplexElement::~CComplexElement() 
{
	mElements->release();

	if (mAnchor != NULL)
		mAnchor->release();
}

// ---------------------------------------------------------------------------------
//		 CComplexElement::setAnchor
// ---------------------------------------------------------------------------------

void CComplexElement::setAnchor(const char *inAnchor)
{
	if (inAnchor != NULL) {
		if (mAnchor == NULL)
			mAnchor = new CString(inAnchor);
		else
			mAnchor->setString(inAnchor);
	}
}

// ---------------------------------------------------------------------------------
//		 CComplexElement::getAnchor
// ---------------------------------------------------------------------------------

char* CComplexElement::getAnchor()
{
	if (mAnchor != NULL)
		return mAnchor->getString();
	else
		return NULL;
}

// ---------------------------------------------------------------------------------
//		 CComplexElement::addElement
// ---------------------------------------------------------------------------------

void CComplexElement::addElement(CReportElement *inElement) 
{
	mElements->add(inElement);
}
		
// ---------------------------------------------------------------------------------
//		 CComplexElement::add
// ---------------------------------------------------------------------------------

void CComplexElement::add(CReportElement *inElement) 
{
	if (inElement->isComplexElement()) {
		CComplexElement *e = (CComplexElement *)inElement;
		int i, n = e->getNumElements();
		for (i = 0; i < n; i++)
			add(e->getElement(i));
	} else {
		addElement(inElement);
	}
}
		
// ---------------------------------------------------------------------------------
//		 CComplexElement::add
// ---------------------------------------------------------------------------------

void CComplexElement::add(char *inText) 
{
	add(new CTextElement(inText));
}
		
// ---------------------------------------------------------------------------------
//		 CComplexElement::add
// ---------------------------------------------------------------------------------

void CComplexElement::add(int inValue) 
{
	add(new CIntegerElement(inValue));
}
		
// ---------------------------------------------------------------------------------
//		 CComplexElement::add
// ---------------------------------------------------------------------------------

void CComplexElement::add(double inValue) 
{
	add(new CRealElement(inValue));
}
		
// ---------------------------------------------------------------------------------
//		 CComplexElement::getElement
// ---------------------------------------------------------------------------------

CReportElement *CComplexElement::getElement(int index) 
{ 
	return dynamic_cast<CReportElement*>(mElements->getObject(index)); 
}

// ---------------------------------------------------------------------------------
//		 CTokenElement::CTokenElement
// ---------------------------------------------------------------------------------
#pragma mark === CTokenElement ===

CTokenElement::CTokenElement(bool inIsItalic, bool inIsBold) :
			CComplexElement(kAlignElementDefault, 0)
{
	mIsItalic = inIsItalic;
	mIsBold = inIsBold;
}

// ---------------------------------------------------------------------------------
//		 CTokenElement::~CTokenElement
// ---------------------------------------------------------------------------------

CTokenElement::~CTokenElement() 
{
}

// ---------------------------------------------------------------------------------
//		 CTokenElement::add
// ---------------------------------------------------------------------------------

void CTokenElement::add(CSimpleElement *inElement) 
{
	ThrowIf(inElement->isComplexElement());
	
	// Elements within a Token cannot have their own style...
	inElement->setStyle(false, false);
	
	CComplexElement::add(inElement);
}
		
// ---------------------------------------------------------------------------------
//		 CTokenElement::add
// ---------------------------------------------------------------------------------

void CTokenElement::add(char *inText) 
{
	CComplexElement::add(inText);
}
		
// ---------------------------------------------------------------------------------
//		 CTokenElement::add
// ---------------------------------------------------------------------------------

void CTokenElement::add(int inValue) 
{
	CComplexElement::add(inValue);
}
		
// ---------------------------------------------------------------------------------
//		 CTokenElement::add
// ---------------------------------------------------------------------------------

void CTokenElement::add(double inValue) 
{
	CComplexElement::add(inValue);
}
		
// ---------------------------------------------------------------------------------
//		 CParagraphElement::CParagraphElement
// ---------------------------------------------------------------------------------
#pragma mark === CParagraphElement ===

CParagraphElement::CParagraphElement(HeadingLevel inHeadingLevel, 
										ElementAlignment inAlignment,
										int inIndentLevel) :
			CComplexElement(inAlignment, inIndentLevel)
{
	mHeadingLevel = inHeadingLevel;
}

// ---------------------------------------------------------------------------------
//		 CParagraphElement::CParagraphElement
// ---------------------------------------------------------------------------------

CParagraphElement::CParagraphElement( char *inText,
										HeadingLevel inHeadingLevel, 
										ElementAlignment inAlignment,
										int inIndentLevel) :
			CComplexElement(inAlignment, inIndentLevel)
{
	mHeadingLevel = inHeadingLevel;
	add(inText);
}

// ---------------------------------------------------------------------------------
//		 CParagraphElement::CParagraphElement
// ---------------------------------------------------------------------------------

CParagraphElement::CParagraphElement( int inValue,
										HeadingLevel inHeadingLevel, 
										ElementAlignment inAlignment,
										int inIndentLevel) :
			CComplexElement(inAlignment, inIndentLevel)
{
	mHeadingLevel = inHeadingLevel;
	add(inValue);
}

// ---------------------------------------------------------------------------------
//		 CParagraphElement::CParagraphElement
// ---------------------------------------------------------------------------------

CParagraphElement::CParagraphElement( double inValue,
										HeadingLevel inHeadingLevel, 
										ElementAlignment inAlignment,
										int inIndentLevel) :
			CComplexElement(inAlignment, inIndentLevel)
{
	mHeadingLevel = inHeadingLevel;
	add(inValue);
}

// ---------------------------------------------------------------------------------
//		 CParagraphElement::~CParagraphElement
// ---------------------------------------------------------------------------------

CParagraphElement::~CParagraphElement() 
{
}

// ---------------------------------------------------------------------------------
//		 CParagraphElement::add
// ---------------------------------------------------------------------------------

void CParagraphElement::add(CReportElement *inElement) 
{
	CComplexElement::add(inElement);
}
		
// ---------------------------------------------------------------------------------
//		 CParagraphElement::add
// ---------------------------------------------------------------------------------

void CParagraphElement::add(char *inText) 
{
	CComplexElement::add(inText);
}
		
// ---------------------------------------------------------------------------------
//		 CParagraphElement::add
// ---------------------------------------------------------------------------------

void CParagraphElement::add(int inValue) 
{
	CComplexElement::add(inValue);
}
		
// ---------------------------------------------------------------------------------
//		 CParagraphElement::add
// ---------------------------------------------------------------------------------

void CParagraphElement::add(double inValue) 
{
	CComplexElement::add(inValue);
}
		
// ---------------------------------------------------------------------------------
//		 CParagraphElement::addLineBreak
// ---------------------------------------------------------------------------------

void CParagraphElement::addLineBreak() 
{
	add(new CLineBreakElement());
}
		
// ---------------------------------------------------------------------------------
//		 CPreformattedElement::CPreformattedElement
// ---------------------------------------------------------------------------------
#pragma mark === CPreformattedElement ===

CPreformattedElement::CPreformattedElement(int inIndentLevel) :
			CComplexElement(kAlignElementDefault, inIndentLevel)
{
}

// ---------------------------------------------------------------------------------
//		 CPreformattedElement::CPreformattedElement
// ---------------------------------------------------------------------------------

CPreformattedElement::CPreformattedElement( char *inText,
											int inIndentLevel) :
			CComplexElement(kAlignElementDefault, inIndentLevel)
{
	add(inText);
}

// ---------------------------------------------------------------------------------
//		 CPreformattedElement::~CPreformattedElement
// ---------------------------------------------------------------------------------

CPreformattedElement::~CPreformattedElement() 
{
}

// ---------------------------------------------------------------------------------
//		 CPreformattedElement::addLineBreak
// ---------------------------------------------------------------------------------

void CPreformattedElement::addLineBreak() 
{
	add(new CLineBreakElement());
}		
		
// ---------------------------------------------------------------------------------
//		 CTableElement::CTableElement
// ---------------------------------------------------------------------------------
#pragma mark === CTableElement ===

CTableElement::CTableElement(int inNumRows, int inNumCols, 
								int inNumTitleRows, int inNumTitleCols,
								ElementAlignment inAlignment,
								int inIndentLevel) :
			CComplexElement(inAlignment, inIndentLevel)
{
	mNumRows = inNumRows;
	mNumCols = inNumCols;
	mNumTitleRows = inNumTitleRows;
	mNumTitleCols = inNumTitleCols;
	
	int i, n = mNumRows * mNumCols;
	for (i = 0; i < n; i++)
		mElements->add(NULL);
}

// ---------------------------------------------------------------------------------
//		 CTableElement::~CTableElement
// ---------------------------------------------------------------------------------

CTableElement::~CTableElement() 
{
}

// ---------------------------------------------------------------------------------
//		 CTableElement::add
// ---------------------------------------------------------------------------------
// Add is disabled - use setCell instead...

void CTableElement::add(CReportElement *inElement) 
{ 
	throw CIndexOutOfBounds();
}

// ---------------------------------------------------------------------------------
//		 CTableElement::add
// ---------------------------------------------------------------------------------
// Add is disabled - use setCell instead...

void CTableElement::add(char *inText) 
{ 
	throw CIndexOutOfBounds();
}

// ---------------------------------------------------------------------------------
//		 CTableElement::add
// ---------------------------------------------------------------------------------
// Add is disabled - use setCell instead...

void CTableElement::add(int inValue) 
{ 
	throw CIndexOutOfBounds();
}

// ---------------------------------------------------------------------------------
//		 CTableElement::add
// ---------------------------------------------------------------------------------
// Add is disabled - use setCell instead...

void CTableElement::add(double inValue) 
{ 
	throw CIndexOutOfBounds();
}

// ---------------------------------------------------------------------------------
//		 CTableElement::setCell
// ---------------------------------------------------------------------------------

void CTableElement::setCell(int inRow, int inCol, CReportElement* inElement, 
							ElementAlignment inAlignment) 
{
	int index = (inRow * mNumCols) + inCol;	
		
	if (inElement->getElementType() == kParagraphElement) {
		mElements->setObject(index, inElement);
	} else {
		CParagraphElement *element;
		NewClass(element, CParagraphElement(kNormalText, inAlignment));
		element->add(inElement);
		mElements->setObject(index, element);
	}
}
		
// ---------------------------------------------------------------------------------
//		 CTableElement::setCell
// ---------------------------------------------------------------------------------

void CTableElement::setCell(int inRow, int inCol, char *inText, 
							ElementAlignment inAlignment) 
{
	setCell(inRow, inCol, new CTextElement(inText), inAlignment);
}

// ---------------------------------------------------------------------------------
//		 CTableElement::setCell
// ---------------------------------------------------------------------------------

void CTableElement::setCell(int inRow, int inCol, int inValue, 
							ElementAlignment inAlignment) 
{
	setCell(inRow, inCol, new CIntegerElement(inValue), inAlignment);
}

// ---------------------------------------------------------------------------------
//		 CTableElement::setCell
// ---------------------------------------------------------------------------------

void CTableElement::setCell(int inRow, int inCol, double inValue, 
							ElementAlignment inAlignment) 
{
	setCell(inRow, inCol, new CRealElement(inValue), inAlignment);
}

// ---------------------------------------------------------------------------------
//		 CTableElement::getCell
// ---------------------------------------------------------------------------------

CComplexElement *CTableElement::getCell(int inRow, int inCol) 
{ 
	int index = (inRow * mNumCols) + inCol;		
	return (CComplexElement *)getElement(index); 
}

// ---------------------------------------------------------------------------------
//		 CListElement::CListElement
// ---------------------------------------------------------------------------------
#pragma mark === CListElement ===

CListElement::CListElement(ListType inListType,
							int inIndentLevel) :
			CTableElement(0, 1, 0, 0, kAlignElementDefault, inIndentLevel)
{
	mListType = inListType;
}

// ---------------------------------------------------------------------------------
//		 CListElement::~CListElement
// ---------------------------------------------------------------------------------

CListElement::~CListElement() 
{
}

// ---------------------------------------------------------------------------------
//		 CListElement::add
// ---------------------------------------------------------------------------------

void CListElement::add(CReportElement *inElement) 
{
	mElements->add(NULL);
	setCell(mNumRows, 0, inElement);
	mNumRows++;
}
		
// ---------------------------------------------------------------------------------
//		 CListElement::add
// ---------------------------------------------------------------------------------

void CListElement::add(char *inText) 
{
	add(new CTextElement(inText));
}
		
// ---------------------------------------------------------------------------------
//		 CListElement::add
// ---------------------------------------------------------------------------------

void CListElement::add(int inValue) 
{
	add(new CIntegerElement(inValue));
}
		
// ---------------------------------------------------------------------------------
//		 CListElement::add
// ---------------------------------------------------------------------------------

void CListElement::add(double inValue) 
{
	add(new CRealElement(inValue));
}
		
// ---------------------------------------------------------------------------------
//		 CListElement::getCell
// ---------------------------------------------------------------------------------

CComplexElement *CListElement::getCell(int inRow) 
{ 
	return CTableElement::getCell(inRow, 0); 
}

// ---------------------------------------------------------------------------------
//		 CReportRecord::CReportRecord
// ---------------------------------------------------------------------------------
#pragma mark === CReportRecord ===

CReportRecord::CReportRecord(char *inTitle) :
			CComplexElement(kAlignElementDefault)
{
	mTitle = NULL;
	
	setTitle(inTitle);
}

// ---------------------------------------------------------------------------------
//		 CReportRecord::~CReportRecord
// ---------------------------------------------------------------------------------

CReportRecord::~CReportRecord() 
{
	if (mTitle != NULL)
		mTitle->release();
	
}

// ---------------------------------------------------------------------------------
//		 CReportRecord::setTitle
// ---------------------------------------------------------------------------------

void CReportRecord::setTitle(char *inTitle)
{
	if (inTitle != NULL) {
		if (mTitle == NULL)
			mTitle = new CString(inTitle);
		else
			mTitle->setString(inTitle);
	}
}

// ---------------------------------------------------------------------------------
//		 CReportRecord::getTitle
// ---------------------------------------------------------------------------------

char* CReportRecord::getTitle()
{
	if (mTitle != NULL)
		return mTitle->getString();
	else
		return NULL;
}

// ---------------------------------------------------------------------------------
//		 CReportRecord::add
// ---------------------------------------------------------------------------------

void CReportRecord::add(CReportElement *inElement) 
{
	CComplexElement::addElement(inElement);
}
		
