/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CReporter.cpp

				CReporter Class
	
 ******************************************************************************/

// CReporter provides a common base to all classes parsing file formats.

#include "CExceptions.h"

#include "CWriter.h"
#include "CReporter.h"
#include "CReportRecord.h"
#include "CGraphicsRecord.h"

#pragma mark === CReporter ===

// ---------------------------------------------------------------------------------
//		 CReporter
// ---------------------------------------------------------------------------------

CReporter::CReporter(CWriter *inWriter, bool inOwner)
{
	writer = inWriter;
	ThrowIfNull(writer);
	
	isOwner = inOwner;
}

CReporter::~CReporter()
{
	if (isOwner && writer != NULL)
		delete writer;
}


// ---------------------------------------------------------------------------------
//		 CReporter::writeReport
// ---------------------------------------------------------------------------------

void CReporter::writeReport(CReportRecord *inReportRecord)
{
	ThrowIfNull(inReportRecord);
	
	int i, n = inReportRecord->getNumElements();
	for (i = 0; i < n; i++) 
		writeElement(inReportRecord->getElement(i));
}

// ---------------------------------------------------------------------------------
//		 CReporter::writeElement
// ---------------------------------------------------------------------------------

void CReporter::writeElement(CReportElement *inElement)
{
	ThrowIfNull(inElement);
	
	ElementType type = inElement->getElementType();
	
	switch (type) {
		case kReport: break; // shouldn't occur
		
		case kTextElement:
			writeText(dynamic_cast<CTextElement *>(inElement));
		break;
		case kLineBreakElement:
			writeLineBreak(dynamic_cast<CLineBreakElement *>(inElement));
		break;
		case kIntegerElement:
			writeInteger(dynamic_cast<CIntegerElement *>(inElement));
		break;
		case kRealElement:
			writeReal(dynamic_cast<CRealElement *>(inElement));
		break;
		case kTokenElement:
			writeToken(dynamic_cast<CTokenElement *>(inElement));
		break;
		case kParagraphElement:
			writeParagraph(dynamic_cast<CParagraphElement *>(inElement));
		break;
		case kPreformattedElement:
			writePreformatted(dynamic_cast<CPreformattedElement *>(inElement));
		break;
		case kTableElement:
			writeTable(dynamic_cast<CTableElement *>(inElement));
		break;
		case kListElement:
			writeList(dynamic_cast<CListElement *>(inElement));
		break;
		
		case kGraphicElement: 
			GraphicType gType = dynamic_cast<CGraphicElement*>(inElement)->getGraphicType();
			if (gType == kFigureGraphicElement)
				writeFigure(dynamic_cast<CFigureElement *>(inElement));
		break;
	}	
}

// ---------------------------------------------------------------------------------
//		 CReporter::writeText
// ---------------------------------------------------------------------------------

void CReporter::writeText(CTextElement *inElement)
{
}

// ---------------------------------------------------------------------------------
//		 CReporter::writeLineBreak
// ---------------------------------------------------------------------------------

void CReporter::writeLineBreak(CLineBreakElement *inElement)
{
}

// ---------------------------------------------------------------------------------
//		 CReporter::writeInteger
// ---------------------------------------------------------------------------------

void CReporter::writeInteger(CIntegerElement *inElement)
{
}

// ---------------------------------------------------------------------------------
//		 CReporter::writeReal
// ---------------------------------------------------------------------------------

void CReporter::writeReal(CRealElement *inElement)
{
}

// ---------------------------------------------------------------------------------
//		 CReporter::writeToken
// ---------------------------------------------------------------------------------

void CReporter::writeToken(CTokenElement *inElement)
{
}

// ---------------------------------------------------------------------------------
//		 CReporter::writeParagraph
// ---------------------------------------------------------------------------------

void CReporter::writeParagraph(CParagraphElement *inElement)
{
}

// ---------------------------------------------------------------------------------
//		 CReporter::writePreformatted
// ---------------------------------------------------------------------------------

void CReporter::writePreformatted(CPreformattedElement *inElement)
{
}

// ---------------------------------------------------------------------------------
//		 CReporter::writeTable
// ---------------------------------------------------------------------------------

void CReporter::writeTable(CTableElement *inElement)
{
}

// ---------------------------------------------------------------------------------
//		 CReporter::writeList
// ---------------------------------------------------------------------------------

void CReporter::writeList(CListElement *inElement)
{
}

// ---------------------------------------------------------------------------------
//		 CReporter::writeFigure
// ---------------------------------------------------------------------------------

void CReporter::writeFigure(CFigureElement *inElement)
{
}

