/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CString.h

		Interface for the CString Class
					
 ******************************************************************************/
//! author="Andrew Rambaut"
//! group=Core
 
 
#ifndef _CSTRING_H_
#define _CSTRING_H_

#include "CObject.h"

/******************************************************************************
	class CString
 ******************************************************************************/
#pragma mark CString class

//:	Implements a dynamically resizing list of strings

class CString : public CObject {
	protected:
		int 				mLength; //: The length of the string
		int					mCapacity;	//: The capacity of the list
		int					mIncrement;	//: The amount by which capacity increments

		char*				mString;	//: The string

	public:
							CString();	//: The constructor
							CString(int inInitial);	//: The constructor
							CString(const char *inString);	//: The constructor
	 	virtual				~CString();	//: The destructor		
	 	
		virtual void		setInitial(int inInitial, int inIncrement = 20);
		//: Set the initial capacity and (optionally) the increment

		virtual void		setCapacity(int inCapacity);
		//: Set the capacity
		
		virtual int			getCapacity() { return mCapacity; };
		//: Returns the capacity

		virtual int			getIncrement() { return mIncrement; };
		//: Sets the amount that capacity increments by
		virtual void		setIncrement(int inIncrement);
		//: Returns the increment
		
		int					getLength() { return mLength; };
		//: Returns the length of the string
		
		virtual char*		getString() { return mString; };
		//: Returns the string
		
		virtual void		getString(char *outString);
		//: Copies the string into outString.
		
		virtual char		getChar(int index);
		//: Returns the char that is at an index.
		
		virtual void		setString(const char *inString);
		//: Sets the string

		virtual void		append(const char *inString);
		//: Adds a string to the end of the string
		
		virtual void		clear();
		//: Removes the string at index and moves others up 
		
		virtual char		operator[](int index) { return getChar(index); }

	protected:
		virtual void		checkCapacity(int requiredCapacity);
		//: Checks that the string list has enough space and resizes it as necessary
	
};

#endif /* _CSTRING_H_ */
