/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CStringArray.h

		Interface for the CStringArray Class
					
 ******************************************************************************/
//! author="Andrew Rambaut"
//! group=Core
 
 
#ifndef _CSTRING_ARRAY_H_
#define _CSTRING_ARRAY_H_

#include "CObject.h"

/******************************************************************************
	class CStringArray
 ******************************************************************************/
#pragma mark CStringArray class

//:	Implements a dynamically resizing list of strings

class CStringArray : public CObject {
	protected:
		int 				mNumStrings; //: The number of strings in the list
		int					mCapacity;	//: The capacity of the list
		int					mIncrement;	//: The amount by which capacity increments

		char**				mStrings;	//: An array of strings

	public:
							CStringArray(int inInitial = 50, 
										int inIncrement = 20);	//: The constructor
	 	virtual				~CStringArray();	//: The destructor		
	 	
		virtual void		setInitial(int inInitial, int inIncrement = 20);
		//: Set the initial capacity and (optionally) the increment

		int					getNumStrings() { return mNumStrings; };
		//: Returns the number of strings in the list
		
		virtual char**		getString() { return mStrings; };
		//: Returns the list of strings
		
		virtual void		setCapacity(int inCapacity);
		//: Set the capacity
		
		virtual int			getCapacity() { return mCapacity; };
		//: Returns the capacity

		virtual int			getIncrement() { return mIncrement; };
		//: Sets the amount that capacity increments by
		virtual void		setIncrement(int inIncrement);
		//: Returns the increment
		
		virtual char*		getString(int index);
		//: Returns the string that is at an index.
		
		virtual void		setString(int index, char *inString);
		//: Sets the string at an index

		virtual int			add(char *inString);
		//: Adds a string to the end of the list
		
		virtual void		insert(int index, char *inString);
		//: Inserts a string into the list at a position
		
		virtual char*		remove(int index);
		//: Removes the string at index and moves others up 
		
		virtual void		removeAll();
		//: Removes all the strings in the list

		virtual char*		operator[](int index) { return getString(index); }

		virtual int			findString(char *inString);
		//: Finds the index of a string in the list

	protected:
		virtual void		checkCapacity(int requiredCapacity);
		//: Checks that the string list has enough space and resizes it as necessary
	
};

#endif /* _CSTRING_ARRAY_H_ */
