/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	CTextProgress.cpp

		Class: CTextProgress

 ******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "CTextProgress.h"


#pragma mark === CTextProgress ===

// ---------------------------------------------------------------------------------
//		 CTextProgress
// ---------------------------------------------------------------------------------

CTextProgress::CTextProgress(std::ostream* inStream, int inBarLength) : CProgress()
{
	setStream(inStream);

	barLength = inBarLength;
	
	hasDrawnBar = false;
}

// ---------------------------------------------------------------------------------
//		 ~CTextProgress
// ---------------------------------------------------------------------------------

CTextProgress::~CTextProgress()
{
}

// ---------------------------------------------------------------------------------
//		 CTextProgress::setStream
// ---------------------------------------------------------------------------------

void CTextProgress::setStream(std::ostream *inStream)
{
	stream=inStream;
}


// ---------------------------------------------------------------------------------
//		 CTextProgress::reset
// ---------------------------------------------------------------------------------

void CTextProgress::reset()
{
	if (hasDrawnBar)
		complete();
		
	hasDrawnBar = false;
}

// ---------------------------------------------------------------------------------
//		 CTextProgress::complete
// ---------------------------------------------------------------------------------

void CTextProgress::complete()
{
	while (dots < barLength) {
		*stream << 'o';
		dots++;
	}

	*stream << '|' << std::endl << std::endl << std::flush;

	hasDrawnBar = false;
}

// ---------------------------------------------------------------------------------
//		 CTextProgress::drawBar
// ---------------------------------------------------------------------------------

void CTextProgress::drawBar()
{
	int i;

	*stream << "0%|";
	for (i=0; i<(barLength); i++)
		*stream << '_';
	*stream << "|100%" << std::endl << "  |" << std::flush;
	dots=0;	
	
	hasDrawnBar = true;
}

// ---------------------------------------------------------------------------------
//		 CTextProgress::description
// ---------------------------------------------------------------------------------

void CTextProgress::description(char* message)
{
	if (suspend)
		return;
		
	reset();
	
	*stream << message << std::endl << std::flush;
};

// ---------------------------------------------------------------------------------
//		 CTextProgress::progress
// ---------------------------------------------------------------------------------

bool CTextProgress::progress(int count, int total)
{
	if (suspend)
		return false;
		
	if (!hasDrawnBar)
		drawBar();
		
	int subLength = barLength / numSubdivisions;

	double percent=((double)count)/total;
	int curr=(int)(subLength*percent) + (currSubdivision * subLength);
	
	while (curr>dots) {
		*stream << 'o';
		dots++;
	}

	if (dots==barLength) {
		*stream << '|' << std::endl << std::endl;
		dots++;
	}
	*stream << std::flush;

	return false; 
};

// ---------------------------------------------------------------------------------
//		 CTextProgress::progress
// ---------------------------------------------------------------------------------

bool CTextProgress::progress(int count, double value)
{
	if (suspend)
		return false;
		
	*stream << count << " - " << value << std::endl << std::flush;

	return false; 
};

