/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CTextReader.cpp

				CTextReader Class
				
 ******************************************************************************/

// CTextReader provides is descended from Reader, using a C++ stream
// to implement reading.

#include "CExceptions.h"
#include "CTextReader.h"

#pragma mark === CTextReader ===

// ---------------------------------------------------------------------------------
//		 CTextReader
// ---------------------------------------------------------------------------------

CTextReader::CTextReader() : CReader(), markPos(0)
{
	setStream(NULL, false);
}

CTextReader::CTextReader(std::istream *inStream, bool inOwner) : CReader(), markPos(0)
{
	setStream(inStream, inOwner);
}


// ---------------------------------------------------------------------------------
//		 ~CTextReader
// ---------------------------------------------------------------------------------

CTextReader::~CTextReader()
{
	if (isOwner && stream != NULL)
		delete stream;
}

// ---------------------------------------------------------------------------------
//		 CTextReader::setStream
// ---------------------------------------------------------------------------------

void CTextReader::setStream(std::istream *inStream, bool inOwner)
{
	stream=inStream;
	markPos=0;
	isOwner = inOwner;
	storedChar = '\0';
	
	mLineNumber = 1;
}

#pragma mark === Low Level Reading ===

// ---------------------------------------------------------------------------------
//		 CTextReader::markPosition
// ---------------------------------------------------------------------------------

// Store the current position in the file. 

void CTextReader::markPosition()
{
	markPos=stream->tellg();
}

// ---------------------------------------------------------------------------------
//		 CTextReader::restorePosition
// ---------------------------------------------------------------------------------

// Reset the file to the stored position. 

void CTextReader::restorePosition()
{
	stream->seekg(markPos);
}

// ---------------------------------------------------------------------------------
//		 CTextReader::readChar
// ---------------------------------------------------------------------------------

// Returns the next character from the file. 

char CTextReader::readChar()
{
	char ch;
		
	if (storedChar != '\0') {
		ch = storedChar;
		storedChar = '\0';
	} else {
		stream->get(ch);
		if (stream->fail())
			throw CEndOfFile();

		if (ch == '\n' || ch == '\r')
			mLineNumber++;
	}
			
	return ch;
};


// ---------------------------------------------------------------------------------
//		 CTextReader::nextChar
// ---------------------------------------------------------------------------------

// Returns the next character from the file without moving the file pointer on. 

char CTextReader::nextChar()
{	
	if (storedChar != '\0') {
		return storedChar;
	} else {
		return stream->peek();
	}
			
};


// ---------------------------------------------------------------------------------
//		 CTextReader::unreadChar
// ---------------------------------------------------------------------------------

// Puts a character back in to the buffer. Ignores the value of ch.

void CTextReader::unreadChar(char ch) 
{
	storedChar = ch;
}



