/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CTextWriter.cpp

				CTextWriter Class

 ******************************************************************************/


#include <iostream>

#include "CTextWriter.h"

#pragma mark === CTextWriter ===

// ---------------------------------------------------------------------------------
//		 CTextWriter
// ---------------------------------------------------------------------------------

CTextWriter::CTextWriter() : CWriter()
{
	setStream(NULL, false);
}

CTextWriter::CTextWriter(std::ostream* inStream, bool inOwner) : CWriter()
{
	setStream(inStream, inOwner);
}

// ---------------------------------------------------------------------------------
//		 ~CTextWriter
// ---------------------------------------------------------------------------------

CTextWriter::~CTextWriter()
{
	if (isOwner && stream != NULL)
		delete stream;
}

// ---------------------------------------------------------------------------------
//		 CTextWriter::setStream
// ---------------------------------------------------------------------------------

void CTextWriter::setStream(std::ostream *inStream, bool inOwner)
{
	stream = inStream;
	isOwner = inOwner;
}

#pragma mark === Low Level Writing ===

// ---------------------------------------------------------------------------------
//		 CTextWriter::writeChar
// ---------------------------------------------------------------------------------

void CTextWriter::writeChar(char ch)
{
	if (ch=='\r' || ch=='\n')
		*stream << std::endl;
	else
		*stream << ch;
}


// ---------------------------------------------------------------------------------
//		 CTextWriter::writeText
// ---------------------------------------------------------------------------------

void CTextWriter::writeText(const char *st, int len)
{
	stream->write(st, len);
}


// ---------------------------------------------------------------------------------
//		 CTextWriter::writeInteger
// ---------------------------------------------------------------------------------

void CTextWriter::writeInteger(int d)
{
	*stream << d;
}


// ---------------------------------------------------------------------------------
//		 CTextWriter::writeInteger
// ---------------------------------------------------------------------------------

void CTextWriter::writeInteger(int d, int width)
{
	stream->width(width);
	*stream << d;
}


// ---------------------------------------------------------------------------------
//		 CTextWriter::writeDouble
// ---------------------------------------------------------------------------------


void CTextWriter::writeDouble(double f)
{
	stream->precision(8);
	*stream << f;
}


// ---------------------------------------------------------------------------------
//		 CTextWriter::writeDouble
// ---------------------------------------------------------------------------------

void CTextWriter::writeDouble(double f, int precision, bool scientific)
{
	if (precision >= 0) 
		stream->precision(precision);
	if (scientific) 
		stream->setf(std::ios::scientific);
	else
		stream->setf(std::ios::fixed);
	*stream << f;
}


// ---------------------------------------------------------------------------------
//		 CTextWriter::writeEndLine
// ---------------------------------------------------------------------------------

void CTextWriter::writeEndLine(int n)
{
	int i;
	
	for (i=0; i<n; i++)
		*stream << std::endl;
	*stream << std::flush;
}

