/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CVariate.h

		Interface for the CVariate Class
					
 ******************************************************************************/
//! author="Andrew Rambaut"
//! group=Core
 
 
#ifndef _CVARIATE_H_
#define _CVARIATE_H_

#include "CDoubleArray.h"

/******************************************************************************
	class CVariate
 ******************************************************************************/
#pragma mark CVariate class

class CVariate : public CDoubleArray {
	protected:
		double				weight;
		
	public:
							CVariate(int inInitial = 50, 
										int inIncrement = 20);	//: The constructor
	 	virtual				~CVariate();	//: The destructor		
	 	
	virtual void			setWeight(double inWeight) { weight = inWeight; }

	virtual double			getRange() { return getMaxValue() - getMinValue(); }
	virtual double			getSum();
	virtual double			getSumOfSquares();
			
	virtual double			getMean();
	virtual double			getMedian();
	virtual double			getVariance();
	
	virtual double			getStandardDeviation();
	virtual double			getSD() { return getStandardDeviation(); }

	virtual double			getStandardError();
	virtual double			getSE() { return getStandardError(); }

	virtual double			getPercentile(double inPercentile);

	virtual int				getSampleSize() { return getNumValues(); }
	virtual double			getSampleVariance();
	virtual double			getSampleStandardDeviation();
	virtual double			getSSD() { return getSampleStandardDeviation(); }

	virtual double			getCoefficientOfVariation();
	virtual double			getCoV() { return getCoefficientOfVariation(); }
};

#endif /* _CVARIATE_H_ */
