/******************************************************************************

ZooCore C++ Class Library (ZooCoreLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of ZooCoreLib C++ Class Library.

    ZooCoreLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ZooCoreLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 CWriter.cpp

				CWriter Class
				
 ******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "CWriter.h"


#pragma mark === CWriter ===

// ---------------------------------------------------------------------------------
//		 CWriter
// ---------------------------------------------------------------------------------

CWriter::CWriter()
{
}

// ---------------------------------------------------------------------------------
//		 ~CWriter
// ---------------------------------------------------------------------------------

CWriter::~CWriter()
{
}

#pragma mark === Higher Level Writing ===
// These functions are used by MEParser objects to write file formats

// ---------------------------------------------------------------------------------
//		 CWriter::writeString
// ---------------------------------------------------------------------------------

void CWriter::writeString(const char *st)
{
	writeText(st, strlen(st));
}


// ---------------------------------------------------------------------------------
//		 CWriter::writePString
// ---------------------------------------------------------------------------------

void CWriter::writePString(const char *st)
{
	writeText(st+1, st[0]);
}


// ---------------------------------------------------------------------------------
//		 CWriter::getLabelLength
// ---------------------------------------------------------------------------------

int CWriter::getLabelLength(const char *st, bool allowPunctuation)
{
	bool quote = false;
	const char *P=st;
	
	while (*P!='\0' && !quote) {
		if ( isspace(*P) || (!allowPunctuation && ispunct(*P) && *P != '_') )
			quote = true;
		P++;
	}
	
	int len = strlen(st);
	if (quote)
		len += 2;

	return len;
}


// ---------------------------------------------------------------------------------
//		 CWriter::writeLabel
// ---------------------------------------------------------------------------------

// This differs from writeString because if st contains spaces (or optionally,
// punctuation) then it is written in quotes. The underscore character is an 
// exception because programs like PAUP use it to represent a space and don't
// expect quotes round labels that use it.

void CWriter::writeLabel(const char *st, bool allowPunctuation)
{
	bool quote = false;
	const char *P=st;
	
	while (*P!='\0' && !quote) {
		if ( isspace(*P) || (!allowPunctuation && ispunct(*P) && *P != '_') )
			quote = true;
		P++;
	}
	
	if (quote) {
		writeChar('\'');
		writeText(st, strlen(st));
		writeChar('\'');
	} else {
		writeText(st, strlen(st));
	}
}


// ---------------------------------------------------------------------------------
//		 CWriter::writeSpaces
// ---------------------------------------------------------------------------------

void CWriter::writeSpaces(int n)
{
	int i;
	
	for (i=0; i<n; i++)
		writeChar(' ');
}


// ---------------------------------------------------------------------------------
//		 CWriter::writeTabs
// ---------------------------------------------------------------------------------

void CWriter::writeTabs(int n)
{
	int i;
	
	for (i=0; i<n; i++)
		writeChar('\t');
}

// ---------------------------------------------------------------------------------
//		 CWriter::writeEndLine
// ---------------------------------------------------------------------------------

void CWriter::writeEndLine(int n)
{
	int i;
	
	for (i=0; i<n; i++)
		writeChar('\r');
}



