/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEAlignment.h

		Interface for the MEAlignment Class
					
 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "Advanced Sequences"
 
#ifndef _MEALIGNMENT_H_
#define _MEALIGNMENT_H_

#include "MEBasicAlignment.h"

class MESequence;
class MEFeature;
class MEFeatureList;
class MEStream;
class MEBitset;

/******************************************************************************
	class MEAlignment
 ******************************************************************************/
#pragma mark MEAlignment class

class MEAlignment : public MEBasicAlignment {
	public:
		virtual long		getID() const { return 'MAli'; }
		//: The ID of this object - 'MAli'.
		
		virtual char*		getClassName() const { return "Alignment"; }
		//: Returns the name of this class - "Alignment"
		
	protected:
		MEFeatureList				*featureList;
		
	public:	
	 								MEAlignment();	
	 								MEAlignment(MEBasicAlignment *src);	
	 	virtual						~MEAlignment();

		virtual void 				clone(MEObject *src);

		virtual MEObject*			createObject();

		virtual void				setAlignmentType(AlignmentType inType);
		virtual void				setAlignmentType(SequenceType inType);
		virtual void				setSequenceTypes(SequenceType inType);
		
		virtual int					getNumSites();
		virtual int					getNumSites(AlignmentType inType);
		virtual int					getNumStates();

		virtual MEBasicSequence*	newSequence(SequenceType inType);

		virtual void				insertGap(int inTaxon, int inSite, int inLength);
		virtual void				removeSites(int inTaxon, int inSite, int inLength);

		virtual int					getSeq2AlignSite(int inTaxon, int inSequenceSite);
		virtual int					getSeq2GapSite(int inTaxon, int inSequenceSite);
		virtual int					getGap2AlignSite(int inTaxon, int inGappedSite);
		virtual int					getGap2AlignLength(int inTaxon, int inGappedLength);
		
		virtual int					getAlign2SeqSite(int inTaxon, int inAlignmentSite);
		virtual int					getAlign2GapSite(int inTaxon, int inAlignmentSite);
		virtual int					getAlign2GapLength(int inTaxon, int inAlignmentLength);
		virtual int					getGap2SeqSite(int inTaxon, int inGappedSite);
		
		virtual int					getNumSites(int inTaxon);
		virtual int					getStringLength(int inTaxon);
		virtual int					getStringLength(int inTaxon, int inNumSites);
		virtual void				getSequenceString(int inTaxon, char *outStr);
		virtual void				getSequenceString(int inTaxon, char *outStr, int inNumSites);
		virtual void				getSequenceString(int inTaxon, char *outStr, int inStartSite, int inNumSites);
		virtual void				getGapString(int inTaxon, char *outStr, int inNumSites);
		virtual void				getSequenceCodes(int inTaxon, short *outCodes);

		virtual void				setSequenceFrame(int inTaxon, int inFrame);
		virtual int					getSequenceFrame(int inTaxon);

		virtual char				getState(int inTaxon, int inSite);

		virtual bool				getSiteIsGapped(int inSite);
		virtual bool				getSiteIsConstant(int inSite);

		// Mike:
		virtual void				putSplit(MEBitset *b, int idx, char *splitMap);
		
		virtual void				getPattern(int inSite, char *outPattern);
		virtual void				getPattern(int inTaxon, int inSite, char *outPattern);

		virtual double*				getEmpiricalFrequencies(double *inFrequencies = NULL);
		virtual double*				getEqualFrequencies(double *inFrequencies = NULL);

		virtual void				setFeatureList(MEFeatureList *inFeatureList, bool isOwner);
		virtual MEFeatureList*		getFeatureList() { return featureList; };
		virtual MEFeatureList*		newFeatureList();
		
		virtual MEFeature*			newFeature();
		virtual int					getNumFeatures();
		virtual MEFeature*			getFeature(int index);
		virtual MEFeature*			getFirstFeature();
		virtual MEFeature*			getNextFeature();
		virtual void				addFeature(MEFeature *inFeature);
		virtual void				insertFeature(int index, MEFeature *inFeature);
		virtual MEFeature*			removeFeature(int index);
		virtual void				removeAllFeatures();

		virtual void				readObject(MEStream *stream);
		virtual void				writeObject(MEStream *stream);
};

#endif /* _MEALIGNMENT_H_ */

