/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEBasicAlignment.h

		Interface for the MEBasicAlignment Class
					
 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "Basic Sequences"
 
#ifndef _MEBASICALIGNMENT_H_
#define _MEBASICALIGNMENT_H_

#include "MEGlobal.h"
#include "METaxonList.h"

class MEBasicSequence;
class MEStream;

/******************************************************************************
	class MEBasicAlignment
 ******************************************************************************/
#pragma mark MEBasicAlignment class

class MEBasicAlignment : public METaxonList {
	public:
		virtual long		getID() const { return 'MBAl'; }
		//: The ID of this object - 'MBAl'.
	
		virtual char*		getClassName() const { return "Basic Alignment"; }
		//: Returns the name of this class - "Basic Alignment"
		
	protected:	
		int 						numSites;
		bool						numSitesKnown;
		
		int 						numStates;
		
		AlignmentType 				alignmentType;
		
		double						*siteWeights;
		int							numWeights;
		
	public:	
	 								MEBasicAlignment();	
	 	virtual						~MEBasicAlignment();

		virtual void 				clone(MEObject *src);

		virtual MEObject*			createObject();

		virtual int					getNumSites();
		virtual int					getNumSites(int inTaxon);

		virtual int					getMaxNumSites();
		virtual int					getMinNumSites();
		virtual bool				hasEqualNumSites();

		virtual int					getNumStates();
		virtual int					countNumStates();

		virtual void				setAlignmentType(AlignmentType inType);
		virtual AlignmentType		getAlignmentType() { return alignmentType; };

		virtual int					getNumSequences() { return numObjects; };
		
		virtual MEBasicSequence*	getSequence(int inTaxon);
		virtual char*				getSequenceLabel(int inTaxon);
		virtual char*				getSequenceInfo(int inTaxon);
		virtual char*				getSequenceName(int inTaxon);
		virtual char*				getSequenceAccession(int inTaxon);
		virtual char*				getSequenceDescription(int inTaxon);
		
		virtual MEBasicSequence*	getFirstSequence();
		virtual MEBasicSequence*	getNextSequence();
		
		virtual int					addSequence(MEBasicSequence *inSequence);
		virtual void				insertSequence(int inTaxon, MEBasicSequence *inSequence);
		virtual MEBasicSequence*	removeSequence(int inTaxon);
		virtual void				removeAll();

		virtual void				getIndelRange(int inTaxon, int inSite, int *outFirst, int *outLast);
		virtual void				getVerticalIndelRange(int inTaxon, int inSite, int *outFirst, int *outLast);
		
		virtual void				appendSequence(int inTaxon, char *inSequence, int inLength);
		virtual void				appendSequence(int inTaxon, MEBasicSequence *inSequence);
		virtual void				appendSite(int inTaxon, char inSite);
		
		virtual void				insertSequence(int inTaxon, char *inSequence, int inLength, int inSite);
		virtual void				insertSequence(int inTaxon, MEBasicSequence *inSequence, int inSite);
		
		virtual void				insertGap(int inTaxon, int inSite, int inLength);
		virtual void				removeSites(int inTaxon, int inSite, int inLength);
		
		virtual int					countInitialGap(int inTaxon);
		virtual int					removeInitialGap(int inTaxon);

		virtual void				trimGapsFromEnd();
		virtual void				removeGaps();
		virtual void				removeGaps(int inTaxon);

		virtual char				getState(int inTaxon, int inSite);
		virtual char				getStateNumber(int inTaxon, int inSite);
		virtual void				setState(int inTaxon, int inSite, char inState);

		virtual void				getSiteString(int inSite, char *outSiteString);
		virtual void				setSiteString(int inSite, char *inSiteString);
		virtual void				appendSiteString(char *inSiteString);

		virtual void				getBinarySiteString(int inSite, char *inStateMap, 
														char *outSiteString);

		virtual void				getPattern(int inSite, char *outPattern);

		virtual bool				getIsMismatch(int inTaxon, int inSite, char inState);

		virtual MEBasicSequence*	newSequence(SequenceType inType);

		virtual int					getStringLength(int inTaxon);
		virtual int					getStringLength(int inTaxon, int inNumSites);
		virtual void				getSequenceString(int inTaxon, char *outStr);
		virtual void				getSequenceString(int inTaxon, char *outStr, int inNumSites);
		virtual void				getSequenceString(int inTaxon, char *outStr, int inStartSite, int inNumSites);
		
		virtual double				getSiteWeight(int site);
		virtual double*				getSiteWeights();
		
		virtual double				getMaxSiteWeight();
		virtual double				getMinSiteWeight();
		
		virtual void				setSiteWeight(int site, double weight);
		virtual void				setAllSiteWeights(double weight);
		virtual void				setAllSiteWeights(double *inWeights);

		virtual void				resizeSiteWeights();

		virtual SequenceType		inferSequenceType();
		
		virtual void				sortBySiteWeights(bool down = true);
		
		virtual void				readObject(MEStream *stream);
		virtual void				readData(MEStream *stream);
		virtual void				writeObject(MEStream *stream);
};

#endif /* _MEBASICALIGNMENT_H_ */

