/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEBasicSequence.h

		Interface for the MEBasicSequence Class
					
 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "Basic Sequences"
 
#ifndef _MEBASICSEQUENCE_H_
#define _MEBASICSEQUENCE_H_

#include "MEGlobal.h"
#include "METaxon.h"

class MEStream;

/******************************************************************************
	class MEBasicSequence
 ******************************************************************************/
#pragma mark MEBasicSequence class

class MEBasicSequence : public METaxon {
	public:
		virtual long		getID() const { return 'MBSq'; }
		//: The ID of this object - 'MBSq'.
		
		virtual char*		getClassName() const { return "Basic Sequence"; }
		//: Returns the name of this class - "Basic Sequence"
		
	protected:
		char				*acc, 			// A string for the accession number / ID
							*seq;			// A character array for the sequence
							
		int					length, 		// Length of the sequence
							capacity;		// Capacity of the sequence character array
							
		SequenceType		seqType;		// Type of the sequence
		AlignmentType		alignmentType;	// Current alignment type
		
		
		int					numSites;		// Number of sites in the current alignmentType
		bool				numSitesKnown;	// Are the number of sites known?
		
		int					numStates;		// Number of states for this sequence
		
		bool				seqChanged;		// The sequence has changed flag
											// This is only set by MEBasicSequence,
											// not used.
	public:
	 					MEBasicSequence();
	 	virtual			~MEBasicSequence();

		virtual void 	clone(MEObject *src);
		
		virtual void 	cloneExceptSequence(MEBasicSequence *src);

		virtual void 	setLength(int inLength);
		virtual int		getLength() { return length; };
		
		virtual int		getCapacity() { return capacity; };

		virtual int		getNumStates();
		virtual int		getNumSites() { numSitesKnown = true; return length; };

		virtual bool	getNumSitesKnown() { return numSitesKnown; }
		virtual bool	getSequenceChanged() { return seqChanged; }
		
		virtual void 			setSequenceType(SequenceType inSequenceType);
		virtual SequenceType	getSequenceType() { return seqType; }
		bool			getIsNucleotide() { return seqType==kDNASequence || seqType==kRNASequence; }
		bool			getIsAminoAcid() { return seqType==kAminoAcidSequence; }
		
		virtual void 	setAlignmentType(AlignmentType inType);
		virtual AlignmentType 	getAlignmentType() { return alignmentType; }
		
		virtual void 	setAccession(char *inAccession);
		virtual char*	getAccession();
		virtual void	getAccession(char *inAccession);

		virtual void	setSite(int inSite, char inState);
		virtual char	getSite(int inSite);
		virtual void	getSites(int inStartSite, int inNumSites, char *outStr);
		
		virtual bool	getIsMismatch(int inSite, char inState);
		
		virtual void 	setSequence(char *inSequence, int inLength);
		virtual char*	getSequence();

		virtual void	appendSequence(char *inSequence, int inLength);
		virtual void	appendSequence(MEBasicSequence *inSequence);
		virtual void	appendSite(char inSite);
		
		virtual void	insertSequence(char *inSequence, int inLength, int inSite);
		virtual void	insertSequence(MEBasicSequence *inSequence, int inSite);
		
		virtual void	insertGap(int inSite, int inLength);
		virtual void	removeSites(int inSite, int inLength);

		virtual	void	trimGapsFromEnd();
		virtual void	removeGaps();
		virtual void	removeGaps(int inFirstSite, int inLastSite);

		virtual void	getIndelRange(int site, int *first, int *last);
		
		virtual int		filterSequence(const char *src, char dest);
		virtual int		filterMatches(const char *match, const char *src);
		virtual int		countState(char inStates);
		virtual int		countStates(const char *inStates);
		
		virtual void	readObject(MEStream *stream);
		virtual void	writeObject(MEStream *stream);
};

#endif /* _MEBASICSEQUENCE_H_ */

