/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEBinaryStream.h

		Interface for the MEBinaryStream Class
					
 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "Read/Write"
 
#ifndef _MEBINARY_STREAM_H_
#define _MEBINARY_STREAM_H_

#include "MEStream.h"

class MEObject;
class MEUndead;
class MEList;
class MEIntArray;
class MEDoubleArray;
class METree;
class MELink;
union ObjectID;

/******************************************************************************
	class MEBinaryStream
 ******************************************************************************/
#pragma mark MEBinaryStream class

class MEBinaryStream : public MEStream
{
	public:
	 							MEBinaryStream();						
	 	virtual					~MEBinaryStream();

		virtual char 			readChar() = 0;
		virtual int 			readInteger() = 0;
		virtual double 			readDouble() = 0;		
		virtual void 			readString(char *str) = 0;		
		virtual void 			readString(char *str, int length) = 0;		

		virtual bool			getLabel(char *label);
		virtual bool			get(char *label, int *i);
		virtual bool			get(char *label, double *d);
		virtual bool			get(char *label, bool *b);
		virtual bool			get(char *label, char *ch);
		virtual bool			get(char *label, char *str, int maxLen);		
		virtual bool			get(char *label, ObjectID *id);
		virtual bool			get(char *label, int *d, const char **stringList, const int numStrings);
		virtual MEObject*		get(char *label);

		virtual void 			getList(MEList *inList);
		virtual void 			getIntArray(MEIntArray *inList);
		virtual void 			getDoubleArray(MEDoubleArray *inList);
		virtual void			getTree(METree *inTree);
		virtual void			getMatrix(MEMatrix *inMatrix);

	protected:
		virtual MELink*			getTip(METree *inTree);
		virtual MELink*			getNode(METree *inTree, int numBranches);
		virtual MELink*			getBranch(METree *inTree, MELink *inLink);

	public:
		virtual void			writeChar(const char c) = 0;
		virtual void			writeInteger(const int d) = 0;
		virtual void			writeDouble(const double f) = 0;
		virtual void			writeString(const char *str) = 0;
		virtual void			writeText(const char *str, int length) = 0;
		
		virtual void			put(char *label, int i);
		virtual void			put(char *label, double d);
		virtual void			put(char *label, bool b);
		virtual void			put(char *label, char ch);
		virtual void			put(char *label, const char *str);
		virtual void			put(char *label, const char *str, int length);
		virtual void			put(char *label, const ObjectID id);
		virtual void			put(char *label, MEObject *inObject);
	
		virtual void			putList(MEList *inList);
		virtual void			putIntArray(MEIntArray *inList);
		virtual void			putDoubleArray(MEDoubleArray *inList);
		virtual void			putTree(METree *inTree);
		virtual void			putMatrix(MEMatrix *inMatrix);

	protected:
		virtual void 			putBranch(METree *inTree, MELink *inLink);
		virtual void 			putNode(METree *inTree, MELink *inLink);
};

#endif /* _MEBINARY_STREAM_H_ */
