/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEBirthDeathSimulator.h

		Interface for the MEBirthDeathSimulator Class
					
 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "Trees"
 
#ifndef _MEBIRTH_DEATH_SIMULATOR_H_
#define _MEBIRTH_DEATH_SIMULATOR_H_

#include "MEExceptions.h"
#include "MEPhylogeny.h"
#include "MEPhylogenySimulator.h"

//: The base to a number of TreeSimulator related exceptions
class METreeSimulatorError : public MEException { 
	public:
		METreeSimulatorError(const char *aMsg) : MEException(aMsg) { };
		METreeSimulatorError(const char *aMsg1, const char *aMsg2) : MEException(aMsg1, aMsg2) { };
};

//: Throw when the input file format is unknown
class MEAllExtinctTreeSimError : public METreeSimulatorError { 
	public:
		MEAllExtinctTreeSimError() : METreeSimulatorError("All lineages have gone extinct") { };
};

// =========================================================================
//	class MEBirthDeathSimulator
// =========================================================================

class MEBirthDeathSimulator : public MEPhylogenySimulator {
	protected:
		double				mBirthRate;
		double				mDeathRate;
		double				mSumRate;
		double				mRelativeBirthRate;
					
		long				mExtantStop;
		double				mTimeStop;		
		
		bool				mPureBirth;
		
	public:
		
							MEBirthDeathSimulator();
		virtual				~MEBirthDeathSimulator();
			
		virtual void		setPureBirth(double inBirthRate);
		virtual void		setBirthDeathRates(double inBirthRate, double inDeathRate);

		virtual void		setExtantStop(long inExtantStop);
		virtual void		setTimeStop(long inTimeStop);

		virtual void		simulate();

	protected:
		virtual bool		event();

		virtual void		mutateLineage(long inParent, long inOffspring) { }
		virtual void		cloneLineage(long inParent, long inOffspring) { }

		virtual MEPhylogeny*	newPhylogeny() { return new MEPhylogeny(); }
		virtual MEGenotype*	newGenotype() { return 0; }

		virtual bool		checkStopConditions();
		virtual double		timeToNextEvent();
		virtual bool		eventIsBirth();
		
	public:
		virtual void		describe(CReportRecord *ioReport);
};

#endif /* _MEBIRTH_DEATH_SIMULATOR_H_ */
