/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEBitset.h

		MEBitset Class
					
 ******************************************************************************/
// MEBitset.h - hacked from Rod Page's VClass code (with permission)
#ifndef MEBITSET_H
#define MEBITSET_H

// includes:
#include <stdlib.h>
#include <string.h>
#include <fstream.h>
#include <iostream.h>
#include "MEObject.h"
#include "MEMaths.h"

// accessed classes:
class permutation;

// constants:
#define LONGBITSET
//#define kMaxBitsetElements	1024
#define kDefaultBitsetElements	128
// default					1024
#ifdef LONGBITSET
	typedef unsigned long cell;
#else
	typedef unsigned short cell;
#endif
#define kBitsPerCell (sizeof(cell)*8)
#define BLOCK	(int)(kMaxBitsetElements / kBitsPerCell)

enum SetRels {IDENTITY, DISJOINT, SUBSET, SUPERSET, OVERLAPPING};


// Set implemented as a bit set
class MEBitset : public MEObject {
public:
	virtual long			getID() const { return 'MEBs'; }
	virtual char*			getClassName() const { return "Bit Set"; }

	MEBitset(int maxelements = kDefaultBitsetElements);
	MEBitset(char *str);
	
	virtual ~MEBitset();

	// boolean functions:

	virtual void				accommodate(int i);
	
	// add_:
	virtual MEBitset&		add(int i); 				// Add i to the set
	virtual MEBitset&		add(long int i);
	virtual MEBitset&		add(const MEBitset &b); 	// Add the set b
	virtual MEBitset&		add(char *str);
	virtual MEBitset&		add(char *str, int length);
	virtual MEBitset&		addToCell(int i, cell x);

	//  c:
	virtual bool				checkcard() const;
	virtual MEBitset&		clear();
	virtual bool				contains(MEBitset &b) const;
	virtual bool				contains(int i) const;
	virtual bool				containsLargerThan(int i) const { return (getLastElement() > i); }
	virtual MEBitset&		copyBits(int i) { return copyBits((long) i); }
	virtual MEBitset&		copyBits(long i);
	virtual MEBitset&		copyFrom(const MEBitset &b);
	virtual MEBitset&		copyFrom(const permutation &p);
	virtual MEBitset&		copyFrom(char *str);
	virtual MEBitset&		copyFrom(char *str, int length);
	virtual MEBitset&		copyFromSplitIndex(unsigned long int l);
	virtual int				countElements() const; 	// Count the number of elements

	// del:
	virtual MEBitset&		del(int i);
	virtual MEBitset&		del(const MEBitset &b);
	virtual MEBitset&		del(char *str);
	virtual MEBitset&		delFromCell(int i, cell x);

	virtual bool 			equals(const MEBitset &b) const;
	virtual bool 			equals(int x) const { return ((contains(x)) && (getCard() == 1)); }
	virtual bool 			equals(char *str) const;

	// fill:
	virtual MEBitset&		fill(int n);
	virtual MEBitset&		fill(char *str, char *map);
	virtual MEBitset&		fillFrom(const permutation &p);
	virtual MEBitset&		fillRandom(int k);
								// choose k elements at random and put them in the set
	virtual MEBitset&		fillRandomUpTo(int k, double pr = 0.5);
								// choose at most k elements at random and put them in the set.
	
	//  get_:
	virtual bool				getBit(int x) const;		// get the i-th bit (0 or 1)
	virtual int				getCard() const;		// get cardinality of set
	virtual cell				getCell(int i) const;		// 
	virtual int  				getFirstElement() const;	// position of first non-zero bit
	virtual int				getIndex() const { return idx; }		// return an integer index of this bitset
	virtual int  				getIthElement(int i) const;	// position of i-th non-zero bit
	virtual int				getLastElement() const;
	virtual int				getMaxCapacity() const { return size*kBitsPerCell; }
	virtual int				getMaxElements() const { return maxElements; }
	virtual int  				getNextElement() const;		//
	virtual int				getNextLargestFrom(int i) const;
	virtual int				getNumCells() const { return getSize(); }
	virtual int				getPreviousElement() const;
	virtual int				getRandomElement() const 
								{ return getIthElement(MERandom::iRan(getCard())); }
	virtual int				getSize() const { return size; }	// get size of the array,
														//  NOT the cardinality of the set!

	virtual bool				hasMore() const { return (Count < card); }

	virtual bool				intersects(const MEBitset &b) const;
	virtual MEBitset&		intersectWith(const MEBitset& bs);
	
	// logical functions:
	virtual bool 			isDisjointFrom(const MEBitset &b) const;
	virtual bool				isEmpty() const { countElements(); return (getCard() == 0); }
	virtual bool				isLexAfter(MEBitset &b) const;
	virtual bool				isLexBefore(MEBitset &b) const;
	virtual bool 			isNull() const { return isEmpty(); }
	virtual bool 			isSubsetOf(MEBitset &b) const;
	virtual bool 			isSupersetOf(MEBitset &b) const;

	virtual bool				nonEmpty() const { return (getCard() != 0); }
	
	//  operators:
	// = set assignment
	virtual MEBitset&		operator=(const MEBitset &b) { return copyFrom(b);	}
	virtual MEBitset&		operator=(int i) { clear(); return add(i); }
	virtual MEBitset&		operator=(long int i) { clear(); return add(i); }
	virtual MEBitset&		operator=(char *str) { clear(); return add(str); }
	virtual MEBitset&		operator=(const permutation &p) { return copyFrom(p); }
	
	// += set to union with
	virtual MEBitset&		operator+=(int i) { return add(i); }	// put to union with
	virtual MEBitset&		operator+=(const MEBitset &b) { return add(b); }
	virtual MEBitset&		operator+=(char *str) { return add(str); }
	
	// -= set minus
	virtual MEBitset&		operator-=(int i) { return del(i); }
	virtual MEBitset&		operator-=(const MEBitset &b) { return del(b); }
	virtual MEBitset&		operator-=(char *str) { return del(str); }
	
	// []
	virtual int				operator[](int i) const { return getIthElement(i); }
	
	// ==
	virtual bool 			operator==(const MEBitset &b) { return equals(b); }
	virtual bool				operator==(char *str) { return equals(str); }	
	
	// <=, >= subset and superset
	virtual bool 			operator<=(MEBitset &b) const { return isSubsetOf(b); }
	virtual bool			operator>=(MEBitset &b) const { return isSupersetOf(b); }
	virtual bool			operator>=(int i) const { return contains(i); }
	
	// *= intersect with
	virtual MEBitset&		operator*=(const MEBitset& bs) { return setIntersectionWith(bs); }	// make intersection with

	// | has non-empty intersection with
	virtual bool			operator|(MEBitset &b) const { return intersects(b); }
	virtual bool			operator|(int i) const { return contains(i); }

	friend ostream&			operator<<(ostream& o, MEBitset &b) { return o << b.write(); }

	// p:
	virtual void			putAsString(char *str);
	
	//  r:
	virtual MEBitset*		randomSubset(int x, MEBitset *S = NULL);
	virtual void				resetCounter() const { setCount(0); }	// go back to beginning of list
	virtual MEBitset&		resize(int newmaxelements);

	// set:
	virtual MEBitset&		setCard(int c);
	virtual MEBitset&		setCell(int i, cell s);
	virtual MEBitset&		setComplement(int n);
	virtual void				setCount(int c) const;
	virtual MEBitset&		setIndex(int i) { idx = i; return *this; }
	virtual MEBitset&		setIntersectionOf(const MEBitset& bs1, const MEBitset& bs2);
	virtual MEBitset&		setIntersectionWith(const MEBitset& bs);
	virtual MEBitset&		setMaxElements(int m) { maxElements = m; return *this; }
	virtual MEBitset&		setNextLex();
	virtual MEBitset&		setUnionOf(const MEBitset &bs1, const MEBitset& bs2);
	virtual MEBitset&		setUnionWith(const MEBitset& bs);
	
	virtual char*			write(char *str = NULL) const;
	
private:

	mutable bool
		_cardKnown;			// flag of whether we know the cardinality of the set
		
	cell
		*set;		// the bit set itself
		
	unsigned long
		idx;		//  4/9/97 
	int
		maxElements,
		size;		// the length of the array in cells

	mutable int
		card,		// number of elements in the set
		CurrentElement,
		Count;
};

typedef MEBitset *MEBitsetPtr;
typedef MEBitsetPtr *bitsetHandle;
typedef cell *cellPtr;

int			requiredspace_bitset(int mel);

#endif


