/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEBitsetList.h

		Interface for the MEBitsetList Class

 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "Utilities"
 
#ifndef _MEBITSET_LIST_H_
#define _MEBITSET_LIST_H_

#include "MEList.h"
#include "MEBitset.h"

class MEBitset;

class MEStream;

/******************************************************************************
	class MEBitsetList
 ******************************************************************************/
#pragma mark MEBitsetList class

class MEBitsetList : public MEList {
public:
	virtual long		getID() const { return 'MBsL'; }
	//: The ID of this object - 'MBsL'.
	
	virtual char*		getClassName() const { return "Bitset List"; }
	//: Returns the name of this class - "Bitset List"
	
protected:
	bool
		_owningSets;

public:
 						MEBitsetList();						
		//: Construct an MEBitsetList.

 	virtual				~MEBitsetList();
		//: Destroy this list.
 	
 	virtual int				addBitset(MEBitset *inBitset, bool isOwner = false);

	virtual void			clear();
 		//: Clear all the elements from the list.  Delete them elsewhere!
 		
	virtual MEObject*		createObject();

	virtual MEBitsetList&	copyFrom(MEBitsetList &bsl);
		//: Copies the contents of one MEBitsetList into this one.

	virtual int				findBitsetPosition(MEBitset *b) { return findByContents(b); }
		//: Finds the index of the set in the list, by its contents (not its address).
	
	virtual MEBitset*		getBitset(int i);
		//: Retrieves a bitset from the list.
		
	virtual MEBitset*		getEmptySet();
		//: Get an empty set from the list.
		
	virtual int				getIndex(MEBitset *b) { return b->getIndex(); }
		//: Get the index stored by the desired bitset.
	
	virtual int				getNumBitsets() const { return numObjects; }
		//: Return the number of bitsets in the list.
			
	virtual void			insertBitset(int index, MEBitset *inBitset, bool isOwner = false);
	virtual MEBitset*		removeBitset(int index);

	virtual int				compare(MEObject *inObject1, MEObject *inObject2);

	virtual bool			owningSets() { return _owningSets; }
	virtual void			setOwningSets(bool o) { _owningSets = o; }
	
	virtual void			readObject(MEStream *stream);
	virtual void			writeObject(MEStream *stream);
};

#endif /* _MEBITSET_LIST_H_ */

