/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MECAICCreator.cpp

				MECAICCreator Class
	
 ******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "MEExceptions.h"
#include "CWriter.h"
#include "MECreator.h"
#include "MECAICCreator.h"
#include "MELink.h"
#include "METree.h"


/******************************************************************************
 MECAICCreator
 
 ******************************************************************************/

MECAICCreator::MECAICCreator() : MECreator()
{
	mMaxCodeLength = 1024;
	NewArray(mCodeString, char, mMaxCodeLength+1);
	mBranchLengthFile = false;
	mUseHeights = false;
}


MECAICCreator::MECAICCreator(CWriter *inWriter) : MECreator(inWriter)
{
	mMaxCodeLength = 1024;
	NewArray(mCodeString, char, mMaxCodeLength+1);
	mBranchLengthFile = false;
}


MECAICCreator::~MECAICCreator()
{
	if (mCodeString != NULL)
		delete [] mCodeString;
}

void MECAICCreator::writeNode(char *inCode, int inPosition, MELink *inNode, double inLength, METree *inTree)
{
	ThrowIf(inPosition >= mMaxCodeLength);
		
	if (inNode != NULL && inNode->getIsTip()) {
		if (!mBranchLengthFile) {
			char *name = inTree->getTipName(inNode);

			writer->writeString(inCode);
			writer->writeChar('\r');
			writer->writeString(name);
			writer->writeChar('\r');
		} else {
			writer->writeString(inCode);
			writer->writeChar('\t');
			writer->writeDouble(inLength);
			writer->writeChar('\t');
			if (mUseHeights) {
				writer->writeDouble(0.0);
			} else {
				writer->writeInteger(-9);
			}
			writer->writeChar('\r');
		}
	} else {
		if (mBranchLengthFile) {
			writer->writeString(inCode);
			writer->writeChar('\t');
			writer->writeDouble(inLength);
			writer->writeChar('\t');
			if (mUseHeights) {
				if (inNode == NULL) { // is the root
					writer->writeDouble(inTree->calcTreeHeight());
				} else {
					writer->writeDouble(inTree->calcNodeHeight(inNode));
				}
			} else {
				writer->writeInteger(-9);
			}
			writer->writeChar('\r');
		}
		
		MELink *descBranch;
		MELink *nextBranch=inTree->getFirstBranch(inNode);
		double length;
		
		inCode[inPosition] = 'A';
		inCode[inPosition+1] = '\0';
		while (nextBranch != NULL) {
			descBranch = inTree->getDescendantNode(nextBranch, &length);
			writeNode(inCode, inPosition+1, descBranch, length, inTree);
			nextBranch=inTree->getNextBranch(inNode, nextBranch);
			inCode[inPosition]++;
		}
		inCode[inPosition] = '\0';
	}
}


void MECAICCreator::createTree(METree *inTree)
{
	mCodeString[0] = '\0';
	writeNode(mCodeString, 0, NULL, 0.0, inTree);
}

