/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 MECodeTable.h

		Interface for the MECodeTable Class
					
 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "Advanced Sequences"
 
#ifndef _MECODETABLE_H_
#define _MECODETABLE_H_

#include "MEGlobal.h"
#include "MEObject.h"


class MEStream;

/******************************************************************************
	class MECodeTable
 ******************************************************************************/
#pragma mark MECodeTable class

class MECodeTable : public MEObject {
	public:
		virtual long		getID() const { return 'MCod'; }
		//: The ID of this object - 'MCod'.
		
		virtual char*		getClassName() const { return "Codon Table"; }
		//: Returns the name of this class - "Codon Table"
		
	protected:
		int						table[kNumNucleotides][kNumNucleotides][kNumNucleotides];
		GeneticCode				code;
		int						numStopCodons;
		
	public:
	 							MECodeTable();						
	 							MECodeTable(GeneticCode inCode);						
	 	virtual					~MECodeTable();
	 	
		virtual void			setCode(GeneticCode inCode);
		virtual GeneticCode		getCode() { return code; };
		virtual char*			getCodeName() { return (char*)kGeneticCodeNames[code]; };
		
		virtual void			setTable(char *inTable, char *inOrder="ACGT");
		virtual void			getTable(char *outTable);
		
		virtual int				translateCodon(char first, char second, char third);
		virtual int				translateCodon(int first, int second, int third);
									 							
		virtual int				getNumStopCodons() { return numStopCodons; }
		
	protected:
		int 					getUnambiguousCodon(const char *fst, const char *sec,
													const char *thd);
		
	public:
		virtual void			readObject(MEStream *stream);
		virtual void			writeObject(MEStream *stream);
};

#endif /* _MECODETABLE_H_ */
