/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MECodeTableList.cpp

				MECodeTableList Class
				
 ******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "MEGlobal.h"
#include "MEExceptions.h"

#include "MECodeTableList.h"
#include "MECodeTable.h"

#include "MEStream.h"


#pragma mark === MECodeTableList ===

MECodeTable* MECodeTableList::mStandardCodes[numGeneticCodes] = {
	new MECodeTable(kUniversalCode),
	new MECodeTable(kVertebrateMtCode),
	new MECodeTable(kYeastCode),
	new MECodeTable(kMoldProtozoanMtCode),
	new MECodeTable(kMycoplasmaCode),
	new MECodeTable(kInvertebrateMtCode),
	new MECodeTable(kCiliateCode),
	new MECodeTable(kEchinodermMtCode),
	new MECodeTable(kEuplotidNucCode),
	new MECodeTable(kBacterialCode),
	new MECodeTable(kAltYeastCode),
	new MECodeTable(kAscidianMtCode),
	new MECodeTable(kFlatwormMtCode),
	new MECodeTable(kBlepharismaNucCode)
};

// ---------------------------------------------------------------------------------
//		 MECodeTableList
// ---------------------------------------------------------------------------------

MECodeTableList::MECodeTableList() : MEList()
{
}

// ---------------------------------------------------------------------------------
//		 ~MECodeTableList
// ---------------------------------------------------------------------------------

MECodeTableList::~MECodeTableList()
{
}

// ---------------------------------------------------------------------------------
//		 MECodeTableList::getNumCodeTables
// ---------------------------------------------------------------------------------

int MECodeTableList::getNumCodeTables()
{
	return getNumObjects() + numGeneticCodes;
}

// ---------------------------------------------------------------------------------
//		 MECodeTableList::getNumUserCodeTables
// ---------------------------------------------------------------------------------

int MECodeTableList::getNumUserCodeTables()
{
	return getNumObjects();
}

// ---------------------------------------------------------------------------------
//		 MECodeTableList::getNumStandardCodeTables
// ---------------------------------------------------------------------------------

int MECodeTableList::getNumStandardCodeTables()
{
	return numGeneticCodes;
}

// ---------------------------------------------------------------------------------
//		 MECodeTableList::getCodeTable
// ---------------------------------------------------------------------------------

MECodeTable *MECodeTableList::getCodeTable(int index)
{
	if (index<numGeneticCodes)
		return getStandardCodeTable((GeneticCode)index);
	else
		return (MECodeTable *)getObject(index - numGeneticCodes);
}

// ---------------------------------------------------------------------------------
//		 MECodeTableList::getUserCodeTable
// ---------------------------------------------------------------------------------

MECodeTable *MECodeTableList::getUserCodeTable(int index)
{
	return (MECodeTable *)getObject(index);
}

// ---------------------------------------------------------------------------------
//		 MECodeTableList::getStandardCodeTable
// ---------------------------------------------------------------------------------

MECodeTable *MECodeTableList::getStandardCodeTable(GeneticCode index)
{
	return mStandardCodes[index];
}

// ---------------------------------------------------------------------------------
//		MECodeTableList::findIndex
// ---------------------------------------------------------------------------------

int	MECodeTableList::findIndex(MEObject *object)
{
	int i, n=getNumCodeTables();
	
	i=0;
	while (i<n && getCodeTable(i)!=object)
		i++;
	
	if (i==n)
		return -1;
	else
		return i;
}
		
// ---------------------------------------------------------------------------------
//		MECodeTableList::createObject
// ---------------------------------------------------------------------------------

MEObject* MECodeTableList::createObject()
{
	MEObject *obj;
	NewClass(obj, MECodeTable(kUniversalCode));
	return obj;
}

#pragma mark === Object I/O ===

// ---------------------------------------------------------------------------------
//		 MECodeTableList::readObject
// ---------------------------------------------------------------------------------

void MECodeTableList::readObject(MEStream *stream)
{
	MEList::readObject(stream);
}

// ---------------------------------------------------------------------------------
//		 MECodeTableList::writeObject
// ---------------------------------------------------------------------------------

void MECodeTableList::writeObject(MEStream *stream)
{
	MEList::writeObject(stream);
}


