/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MECreator.h

		Interface for the MECreator Class

 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "File Creating"

#ifndef _MECREATOR_H_
#define _MECREATOR_H_

// This is required to get the formatType enum
#include "MEParser.h"

// Whether the format is supported for output
extern const bool kOutputFormatSupported[numFormatTypes]; 

class CWriter;
class MEBasicAlignment;
class MELink;
class METree;
class METreeList;
class MEPhylogeny;
class MEPhylogenyList;
class MEDistanceMatrix;

/******************************************************************************
	class MECreator
 ******************************************************************************/
#pragma mark MECreator class

class MECreator {			// Class Declaration
	protected:
		CWriter *writer;
				
		bool interleaved;
		int	numColumns;
		bool repeatTaxa;

	public:
		MECreator();
		MECreator(CWriter *inWriter);
		
		virtual void			setWriter(CWriter *inWriter);
		virtual void			setIsInterleaved(bool inInterleaved) { interleaved = inInterleaved; }
		virtual void 			setNumColumns(int inNumColumns) { numColumns = inNumColumns; }
		virtual void			setRepeatTaxa(bool inRepeatTaxa) { repeatTaxa = inRepeatTaxa; }

		virtual ~MECreator();
				
	protected:
		virtual void 			writeSequence(char *inSequence, int maxLen, 
												int width, int indent, int space);
												
		virtual void			writeSequenceData(	MEBasicAlignment *inAlignment, 
													int inIndent, 
													int inTruncateNameLen,
													bool inUseTaxonNumbers,
													bool inIncludeInfo,
													bool inIncludeSpace,
													bool inDrawRuler,
													bool inRulerInComment,
													bool inUseMatchChar,
													int inSpaceEvery);

		virtual void 			writeBranch(METree *inTree, MELink *inLink);
		virtual void 			writeNode(METree *inTree, MELink *inLink);
		virtual void 			writeTree(METree *inTree);
		
		virtual void 			writeLineage(MEPhylogeny *inPhylogeny, long inNode, long &ioTip);
		virtual void 			writePhylogeny(MEPhylogeny *inPhylogeny);

	public:
		virtual void			createAlignment(MEBasicAlignment *inAlignment);
		
		virtual void			createTree(METree *inTree);
		virtual void			createTreeList(METreeList *inTreeList);
		
		virtual void			createPhylogeny(MEPhylogeny *inPhylogeny);
		virtual void			createPhylogenyList(MEPhylogenyList *inPhylogenyList);
		
		virtual void			createDistanceMatrix(MEDistanceMatrix *inDistanceMatrix);

		virtual void			finishCreation() = 0;
};

#endif /* _MECREATOR_H_ */
