/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MECreatorUtils.cpp

				MECreatorUtils Class
	
 ******************************************************************************/

// MECreatorUtils provides a set of static functions for automatically
// writing data to streams;

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "MEGlobal.h"
#include "MEExceptions.h"

#include "CWriter.h"
#include "MECreator.h"
#include "MENexusCreator.h"
#include "MEPhylipCreator.h"
#include "MEMolPhyCreator.h"
//#include "MEMegaCreator.h"
#include "MEFlatCreator.h"
#include "MENBRFCreator.h"
#include "MEFASTACreator.h"
#include "MEClustalCreator.h"
#include "MECAICCreator.h"
#include "MEPrettyCreator.h"

#include "MEBasicAlignment.h"
#include "METreeList.h"
#include "METree.h"

#include "MECreatorUtils.h"

// ---------------------------------------------------------------------------------
//		 MECreatorUtils::create
// ---------------------------------------------------------------------------------

void MECreatorUtils::create(CWriter *inWriter, FormatType format, bool interleaved, 
							MEBasicAlignment *inAlignment)
{
	MECreator *creator;
	
	creator=createCreator(inWriter, format, interleaved);
	creator->createAlignment(inAlignment);
	creator->finishCreation();
	
	delete creator;	
}


void MECreatorUtils::create(CWriter *inWriter, FormatType format, bool interleaved, 
							METree *inTree)
{
	MECreator *creator;
	
	creator=createCreator(inWriter, format, interleaved);
	creator->createTree(inTree);
	creator->finishCreation();
	
	delete creator;
}


void MECreatorUtils::create(CWriter *inWriter, FormatType format, bool interleaved, 
							METreeList *inTreeList)
{
	MECreator *creator;
	
	creator=createCreator(inWriter, format, interleaved);
	creator->createTreeList(inTreeList);
	creator->finishCreation();
	
	delete creator;
}


void MECreatorUtils::create(CWriter *inWriter, FormatType format, bool interleaved, 
							MEBasicAlignment *inAlignment, METree *inTree)
{
	MECreator *creator;
	
	creator=createCreator(inWriter, format, interleaved);
	creator->createAlignment(inAlignment);
	creator->createTree(inTree);
	creator->finishCreation();
	
	delete creator;
}


void MECreatorUtils::create(CWriter *inWriter, FormatType format, bool interleaved, 
							MEBasicAlignment *inAlignment, METreeList *inTreeList)
{
	MECreator *creator;
	
	creator=createCreator(inWriter, format, interleaved);
	creator->createAlignment(inAlignment);
	creator->createTreeList(inTreeList);
	creator->finishCreation();
	
	delete creator;
}


MECreator* MECreatorUtils::createCreator(CWriter *inWriter, FormatType format, bool interleaved)
{
	MECreator *creator=NULL;

	ThrowIfNot(format>kFormatUnknown && format<numFormatTypes);
	ThrowIfNot(kOutputFormatSupported[format]);
	
	switch (format) {
		case kFormatNEXUS: 
			creator=new MENexusCreator(inWriter);
		break;
		case kFormatPHYLIP: 
		case kFormatTree: 
			creator=new MEPhylipCreator(inWriter);
		break;
		case kFormatMOLPHY: 
			creator=new MEMolPhyCreator(inWriter);
		break;
		case kFormatMEGA: 
//			creator=new MEMegaCreator(inWriter);
		break;
		case kFormatFLAT: 
			creator=new MEFlatCreator(inWriter);
		break;
		case kFormatNBRF: 
			creator=new MENBRFCreator(inWriter);
		break;
		case kFormatFASTA: 
			creator=new MEFASTACreator(inWriter);
		break;
		case kFormatCLUSTAL: 
			creator=new MEClustalCreator(inWriter);
		break;
		case kFormatCAIC: 
			creator=new MECAICCreator(inWriter);
		break;
		case kFormatPretty: 
			creator=new MEPrettyCreator(inWriter);
		break;
	}
	
	creator->setIsInterleaved(interleaved);
	
	return creator;
}
