/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEDataMatrix.h

		Interface for the MEDataMatrix Class
					
 ******************************************************************************/
//! author = "Andrew Rambaut & Mike Charleston"
//! group = "Matrices"
 
#ifndef _MEDATA_MATRIX_H_
#define _MEDATA_MATRIX_H_

#include "MEGlobal.h"
#include "METaxonMatrix.h"

class MEList;
class MEDataColumn;
class MEStream;

class MEDataMatrix : public METaxonMatrix {
	public:
		virtual long		getID() const { return 'MDat'; }
		//: The ID of this object - 'MDat'.

		virtual char*		getClassName() const { return "Data Matrix"; }
		//: Returns the name of this class - "Data Matrix"
		
	protected:
		MEList*					columns;
			
	public:
	 							MEDataMatrix();						
	 							MEDataMatrix(int inDimension);						
	 							MEDataMatrix(int inNumRows, int inNumCols);	
	 										
	 	virtual					~MEDataMatrix();
	
		virtual void			setSize(int inNumRows, int inNumCols);
		
		virtual	MEList			*getColumnList();
		virtual MEObject		*getColumnObject(int col);

		virtual void			setElement(int row, int col, double f);
		virtual void			setElement(int index, double f);
		virtual double			getElement(int row, int col);
		virtual double			getElement(int index);
		
		virtual MEDataColumn	*getColumn(int index);

		virtual void			addColumn();
		virtual void			addColumn(char *inName);
		virtual void			addColumn(MEDataColumn *inColumn, bool isOwner=false);
		
		virtual void			addRow();
		virtual void			addRow(char *inName);
		
		virtual void			setColumnLogged(int index, bool inIsLogged);
		virtual bool			getIsColumnLogged(int index);

		virtual void			readObject(MEStream *stream);
		virtual void			readData(MEStream *stream);
		virtual void			writeObject(MEStream *stream);
		virtual void			writeData(MEStream *stream);
};

#endif /* _MEDATA_MATRIX_H_ */
