/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEDataTextStream.cpp

				MEDataTextStream Class
				
 ******************************************************************************/

// MEDataTextStream overrides METextStream to write a text format to a char buffer.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "MEGlobal.h"
#include "MEExceptions.h"
#include "MEList.h"
#include "MEDataTextStream.h"

#pragma mark === MEDataTextStream ===

// ---------------------------------------------------------------------------------
//		 MEDataTextStream
// ---------------------------------------------------------------------------------

MEDataTextStream::MEDataTextStream() : METextStream()
{
	buffer=NULL;
	setIsCounting(true);
}

MEDataTextStream::MEDataTextStream(char *inBuffer, int inLength) : METextStream()
{
	buffer=NULL;
	setBuffer(inBuffer, inLength);
}


// ---------------------------------------------------------------------------------
//		 ~MEDataTextStream
// ---------------------------------------------------------------------------------

MEDataTextStream::~MEDataTextStream()
{
}


// ---------------------------------------------------------------------------------
//		 MEDataTextStream::setBuffer
// ---------------------------------------------------------------------------------

void MEDataTextStream::setBuffer(char *inBuffer, int inLength)
{
	buffer=inBuffer;
	bufLen=inLength;
	bufPos=0;
	setIsCounting(false);
}

// ---------------------------------------------------------------------------------
//		 MEDataTextStream::setBuffer
// ---------------------------------------------------------------------------------

void MEDataTextStream::setIsCounting(bool inCounting)
{
	isCounting=inCounting;
	if (isCounting)
		writeCount=0;
	else {
		if (cache!=NULL)
			cache->removeAll();
	}
}


#pragma mark === Low Level Reading ===

// ---------------------------------------------------------------------------------
//		 MEDataTextStream::readChar
// ---------------------------------------------------------------------------------

// Returns the next character from the file. 

char MEDataTextStream::readChar()
{
	if (bufPos>=bufLen)
		throw CReadError("End of buffer reached");
		
	char ch=buffer[bufPos];
		
	bufPos++;

	return ch;
}


// ---------------------------------------------------------------------------------
//		 MEDataTextStream::nextChar
// ---------------------------------------------------------------------------------

// Returns the next character from the file without moving the file pointer on. 

char MEDataTextStream::nextChar()
{	
	if (bufPos>=bufLen)
		throw CReadError("End of buffer reached");
		
	return buffer[bufPos];
}


#pragma mark === Low Level Writing ===

// ---------------------------------------------------------------------------------
//		 MEDataTextStream::writeChar
// ---------------------------------------------------------------------------------

void MEDataTextStream::writeChar(char ch)
{
	if (isCounting) {
		writeCount++;
		return;
	}
	
	if (bufPos>=bufLen)
		throw CWriteError("End of buffer reached");
		
	buffer[bufPos] = ch;
	bufPos++;
}


// ---------------------------------------------------------------------------------
//		 MEDataTextStream::writeText
// ---------------------------------------------------------------------------------

void MEDataTextStream::writeText(const char *st, int len)
{
	for (int i=0; i<len; i++)
		writeChar(st[i]);
}


// ---------------------------------------------------------------------------------
//		 MEDataTextStream::writeInteger
// ---------------------------------------------------------------------------------

void MEDataTextStream::writeInteger(int d)
{
	char str[64];
	sprintf(str, "%ld", d);
	writeText(str, strlen(str));
}


// ---------------------------------------------------------------------------------
//		 MEDataTextStream::writeDouble
// ---------------------------------------------------------------------------------


void MEDataTextStream::writeDouble(double f)
{
	char str[64];
	sprintf(str, "%lf", f);
	writeText(str, strlen(str));
}


