/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEDistanceMatrix.h

		Interface for the MEDistanceMatrix Class

 ******************************************************************************/
//! author = "Andrew Rambaut & Mike Charleston"
//! group = "Matrices"
 
#ifndef _MEDISTANCE_MATRIX_H_
#define _MEDISTANCE_MATRIX_H_

#include "METaxonMatrix.h"

enum DistanceMethod {
	kUncorrectedDistance,
	kPDistance,
	kPoissonDistance,
	kLogDetDistance,
	numDistanceMethods
};

extern const char* kDistanceMethodNames[numDistanceMethods];

class MEAlignment;
class MEPatternList;
class METree;
class MEStream;

class MEDistanceMatrix : public METaxonMatrix {
	public:
		virtual long		getID() const { return 'MDis'; }
		//: The ID of this object - 'MDis'.
	
		virtual char*		getClassName() const { return "Distance Matrix"; }
		//: Returns the name of this class - "Distance Matrix"
		
	public:
	 						MEDistanceMatrix();						
	 						MEDistanceMatrix(int inDimension);						
	 										
	 	virtual				~MEDistanceMatrix();

		virtual void		setSize(int inNumRows, int inNumCols);
		virtual void 		setMatrixFromTree(METree *inTree);
		
		virtual void		calcDistanceMatrix();
		
		virtual void		readObject(MEStream *stream);
		virtual void		writeObject(MEStream *stream);
		
		virtual void		setDataSource(MEAlignment *inAlignment);
		virtual void		setDataSource(MEPatternList *inList);
		virtual void		setDataSource(MEDistanceMatrix *inMatrix);
		virtual void		setDistanceMethod(DistanceMethod dm);
		
		MEDistanceMatrix&	operator()(int r, int c);
		MEDistanceMatrix&	operator=(double d);
	
	bool
		_matrixIsKnown;
		
	DistanceMethod
		dMethod;
		
	MEAlignment			*sourceAlignment;
	MEDistanceMatrix	*sourceMatrix;
	MEPatternList		*sourcePatterns;
};

#endif /* _MEDISTANCE_MATRIX_H_ */
