/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	MEDistanceMeasure.cpp

		Interface for the MEDistanceMeasure Class
					    
 ******************************************************************************/

// includes:
// C/C++:
#include <stdlib.h>
#include <stdio.h>

// MolEvol.:
#include "MEExceptions.h"
#include "MEPatternList.h"
#include "MEDistanceMatrix.h"
#include "MEDistanceMeasure.h"



/******************************************************************************
	class MEDistanceMeasure
 ******************************************************************************/

// ---------------------------------------------------------------------------------
//   MEDistanceMeasure::
// ---------------------------------------------------------------------------------
MEDistanceMeasure::MEDistanceMeasure()
{

}

// ---------------------------------------------------------------------------------
//   MEDistanceMeasure::~MEDistanceMeasure
// ---------------------------------------------------------------------------------
MEDistanceMeasure::~MEDistanceMeasure()
{

}



// ---------------------------------------------------------------------------------
//   MEDistanceMeasure::MEDistanceMeasure
// ---------------------------------------------------------------------------------
MEDistanceMatrix* MEDistanceMeasure::getDistanceMatrix(MEPatternList *pat)
{
	// calculate the distance matrix: for this base class we use the ordinary
	// Hamming distance.
	MEDistanceMatrix *M;
	NewClass(M, MEDistanceMatrix(pat->getNumTaxa()));	
	return getDistanceMatrix(M, pat);
}
MEDistanceMatrix* MEDistanceMeasure::getDistanceMatrix(MEDistanceMatrix *M, MEPatternList *pat)
{
	// calculate the distance matrix: for this base class we use the ordinary
	// Hamming distance.
	char *str;
	double d;
	int i, j, k = pat->getNumPatterns(), n = pat->getNumTaxa(), t;
	M->setSize(n, n);
	M->fillDiagonal(0.0);
	for (i = 0; i < n; i++) {
		for (j = i+1; j < n; j++) {
			d = 0.0;
			for (t = 0; t < k; t++) {
				str = pat->getPattern(t);
				if (str[i] != str[j])
					d += pat->getPatternWeight(t);
			}
			M->setElement(i, j, d);
			M->setElement(j, i, d);
		}
	}
	
	return M;
}

