/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEDoubleArrayList.cpp

				MEDoubleArrayList Class
				
 ******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "MEGlobal.h"
#include "MEExceptions.h"

#include "MEDoubleArrayList.h"
#include "MEDoubleArray.h"

#include "MEStream.h"


#pragma mark === MEDoubleArrayList ===

// ---------------------------------------------------------------------------------
//		 MEDoubleArrayList
// ---------------------------------------------------------------------------------

MEDoubleArrayList::MEDoubleArrayList() : 
	MEList()
//: Construct an MEDoubleArrayList.
// This calls the base class (MEList) constructor.
{
}

// ---------------------------------------------------------------------------------
//		 ~MEDoubleArrayList
// ---------------------------------------------------------------------------------

MEDoubleArrayList::~MEDoubleArrayList()
//: Destroy this list.
{
}


// ---------------------------------------------------------------------------------
//		MEDoubleArrayList::createObject
// ---------------------------------------------------------------------------------
MEObject* MEDoubleArrayList::createObject()
//: Create a new MEDoubleArray for the list.
{
	MEObject *obj;
	NewClass(obj, MEDoubleArray());
	return obj;
}


// ---------------------------------------------------------------------------------
//		 MEDoubleArrayList::getDoubleArray
// ---------------------------------------------------------------------------------
MEDoubleArray* MEDoubleArrayList::getDoubleArray(int index)
//: Retrieves a bitset from the list.
// This also checks to see that the passed index is in the right range.
{
	return (MEDoubleArray *)getObject(index);
}


// ---------------------------------------------------------------------------------
//		 MEDoubleArrayList::addDoubleArray
// ---------------------------------------------------------------------------------
int MEDoubleArrayList::addDoubleArray(MEDoubleArray *inDoubleArray, bool isOwner)
//: Add a bitset to the MEDoubleArrayList and return its index in the list.
{
	return add(inDoubleArray, isOwner);
}


// ---------------------------------------------------------------------------------
//		 MEDoubleArrayList::insertDoubleArray
// ---------------------------------------------------------------------------------
void MEDoubleArrayList::insertDoubleArray(int index, MEDoubleArray *inDoubleArray, bool isOwner)
//: Insert a bitset into the list, advancing the positions of the lower-order bitsets if necessary.
// This is a simple wrapper for the base method.
{
	insert(index, inDoubleArray, isOwner);
}


// ---------------------------------------------------------------------------------
//		 MEDoubleArrayList::removeDoubleArray
// ---------------------------------------------------------------------------------
MEDoubleArray* MEDoubleArrayList::removeDoubleArray(int index)
//: Remove a bitset from the list, decrementing the positions of the lower-order bitsets if necessary.
// This is a simple wrapper for the base method.
{
	return (MEDoubleArray *)remove(index);
}


#pragma mark === Object I/O ===


// ---------------------------------------------------------------------------------
//		 MEDoubleArrayList::readObject
// ---------------------------------------------------------------------------------
void MEDoubleArrayList::readObject(MEStream *stream)
{
	MEList::readObject(stream);
}


// ---------------------------------------------------------------------------------
//		 MEDoubleArrayList::writeObject
// ---------------------------------------------------------------------------------
void MEDoubleArrayList::writeObject(MEStream *stream)
{
	MEList::writeObject(stream);
}


