/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEDoubleArrayList.h

		Interface for the MEDoubleArrayList Class

 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "Global"
 
#ifndef _MEDOUBLEARRAY_LIST_H_
#define _MEDOUBLEARRAY_LIST_H_

#include "MEList.h"
#include "MEDoubleArray.h"

class MEDoubleArray;

class MEStream;

/******************************************************************************
	class MEDoubleArrayList
 ******************************************************************************/
#pragma mark MEDoubleArrayList class

class MEDoubleArrayList : public MEList {
public:
	virtual long		getID() const { return 'MDAL'; }
	//: The ID of this object - 'MDAL'.
	
	virtual char*		getClassName() const { return "DoubleArray List"; }
	//: Returns the name of this class - "DoubleArray List"
	
public:
 								MEDoubleArrayList();						
		//: Construct an MEDoubleArrayList.

 	virtual					~MEDoubleArrayList();
		//: Destroy this list.
 	
 	virtual int				addDoubleArray(MEDoubleArray *inDoubleArray, bool isOwner = false);

	virtual MEObject*		createObject();

	virtual MEDoubleArray*		getDoubleArray(int i);
		
	virtual int				getNumDoubleArrays() const { return numObjects; }
			
	virtual void			insertDoubleArray(int index, MEDoubleArray *inDoubleArray, bool isOwner = false);
	virtual MEDoubleArray*		removeDoubleArray(int index);
	
	virtual void			readObject(MEStream *stream);
	virtual void			writeObject(MEStream *stream);
};

#endif /* _MEDOUBLEARRAY_LIST_H_ */

