/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEFASTAParser.cpp

				MEFASTAParser Class
	
 ******************************************************************************/

// This class parses NBRF/Pir flat format sequence files. 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "MEGlobal.h"
#include "MEExceptions.h"

#include "CReader.h"
#include "MEParser.h"
#include "MEFASTAParser.h"
#include "MEBasicSequence.h"
#include "MEBasicAlignment.h"


// ---------------------------------------------------------------------------------
//		 MEFASTAParser
// ---------------------------------------------------------------------------------

MEFASTAParser::MEFASTAParser(CReader *inReader) : MEParser(inReader)
{
	reader->setCommentDelimiters('\0', '\0', '\0');
}

// ---------------------------------------------------------------------------------
//		 MEFASTAParser::readAlignment
// ---------------------------------------------------------------------------------

void MEFASTAParser::readAlignment(MEBasicAlignment *inAlignment)
{	
	char name[MAX_NAME_LEN+1];
	char *P;
	MEBasicSequence *seq;

	try {
		char ch=reader->readChar();
		do {
			reader->readLine(name, getMaxNameLen());
			P=strchr(name, ','); // trim of the stuff after a comma (checksum, etc.)
			if (P!=NULL)
				*P='\0';
			
			seq=inAlignment->newSequence(seqType);
			seq->setName(name);
			ch=readSequenceUntil(seq, "0123456789", ">");
			inAlignment->addSequence(seq);
		} while (ch=='>');
	} catch (CEndOfFile) {
	}
	seqType=inAlignment->inferSequenceType();
}

