/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEFeature.cpp

				MEFeature Class

 ******************************************************************************/

// MEFeature is a simple class which provides a container for a feature in a
// gene alignment (exon, intron, etc.).

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "MEStream.h"
#include "MEObject.h"
#include "MEFeature.h"


#pragma mark === MEFeature ===

// ---------------------------------------------------------------------------------
//		 MEFeature
// ---------------------------------------------------------------------------------

MEFeature::MEFeature() : MEObject()
{
	featureType=kUnknownFeature;
	start=0;
	length=0;
	frame=0;
	score=-999;
}

// ---------------------------------------------------------------------------------
//		 ~MEFeature
// ---------------------------------------------------------------------------------

MEFeature::~MEFeature()
{
}

// ---------------------------------------------------------------------------------
//		 MEFeature::clone
// ---------------------------------------------------------------------------------

void MEFeature::clone(MEObject *src)
{
	MEObject::clone(src);
	
	MEFeature *feat=(MEFeature *)src;
	setFeatureType(feat->getFeatureType());
	setStart(feat->getStart());
	setLength(feat->getLength());
	setFrame(feat->getFrame());
	setScore(feat->getScore());
}

// ---------------------------------------------------------------------------------
//		 MEFeature::readObject
// ---------------------------------------------------------------------------------

void MEFeature::readObject(MEStream *stream)
{
	MEObject::readObject(stream);
	
	int i=kUnknownFeature;
	stream->get("Type", &i, kFeatureTypeNames, numFeatureTypes);
	featureType=(FeatureType)i;
	
	stream->get("Start", &start);
	stream->get("Length", &length);
	stream->get("Score", &score);
}

// ---------------------------------------------------------------------------------
//		 MEFeature::writeObject
// ---------------------------------------------------------------------------------

void MEFeature::writeObject(MEStream *stream)
{
	MEObject::writeObject(stream);
	
	stream->put("Type", kFeatureTypeNames[featureType]);
	stream->put("Start", start);
	stream->put("Length", length);
	stream->put("Score", score);
}


