/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEFeature.h

		Interface for the MEFeature Class
					
 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "Advanced Sequences"
 
#ifndef _MEFEATURE_H_
#define _MEFEATURE_H_

#include "MEGlobal.h"
#include "MEObject.h"

class MEStream;

/******************************************************************************
	class MEFeature
 ******************************************************************************/
#pragma mark MEFeature class

class MEFeature : public MEObject {
	public:
		virtual long		getID() const { return 'MFea'; }
		//: The ID of this object - 'MFea'.
		
		virtual char*		getClassName() const { return "Feature"; }
		//: Returns the name of this class - "Feature"
		
	protected:
		FeatureType			featureType;
		int					start, length, frame;
		double				score;
		
	public:
	 						MEFeature();						
	 	virtual				~MEFeature();
	 	
		virtual void 		clone(MEObject *src);
		//: Clones this object from an existing one

	 	FeatureType			getFeatureType() { return featureType; };
	 	void				setFeatureType(FeatureType inType) { featureType=inType; };
	 	
	 	int					getStart() { return start; };
	 	void				setStart(int inStart) { start=inStart; };
	 	
	 	int					getFinish() { return start + length -1; };
	 	void				setFinish(int inFinish) { length=inFinish - start + 1; };

	 	int					getLength() { return length; };
	 	void				setLength(int inLength) { length=inLength; };

	 	int					getFrame() { return frame; };
	 	void				setFrame(int inFrame) { frame=inFrame; };

	 	double				getScore() { return score; };
	 	void				setScore(double inScore) { score=inScore; };

		virtual void		readObject(MEStream *stream);
		virtual void		writeObject(MEStream *stream);
};

#endif /* _MEFEATURE_H_ */
