/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEFeatureList.cpp

				MEFeatureList Class
				
 ******************************************************************************/

// MEFeatureList is a simple list of features.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "MEExceptions.h"
#include "MEStream.h"
#include "MEFeature.h"
#include "MEList.h"
#include "MEFeatureList.h"

#pragma mark === MEFeatureList ===

// ---------------------------------------------------------------------------------
//		 MEFeatureList
// ---------------------------------------------------------------------------------

MEFeatureList::MEFeatureList() : 
	MEList() 
{
	sortType=kSortByName;
}

// ---------------------------------------------------------------------------------
//		 ~MEFeatureList
// ---------------------------------------------------------------------------------

MEFeatureList::~MEFeatureList()
{
}

// ---------------------------------------------------------------------------------
//		MEFeatureList::cloneList
// ---------------------------------------------------------------------------------

void MEFeatureList::cloneList(MEList *src)
{
	setSortType(((MEFeatureList *)src)->getSortType());
	MEList::cloneList(src);
}

// ---------------------------------------------------------------------------------
//		MEFeatureList::createObject
// ---------------------------------------------------------------------------------

MEObject* MEFeatureList::createObject()
{
	MEObject *obj;
	NewClass(obj, MEFeature());
	return obj;
}

// ---------------------------------------------------------------------------------
//		 MEFeatureList::getFeature
// ---------------------------------------------------------------------------------

MEFeature* MEFeatureList::getFeature(int index)
{
	return (MEFeature *)getObject(index);
}

// ---------------------------------------------------------------------------------
//		 MEFeatureList::addFeature
// ---------------------------------------------------------------------------------

int MEFeatureList::addFeature(MEFeature *inFeature)
{
	return add(inFeature, inFeature->getOwner()==NULL);
}

// ---------------------------------------------------------------------------------
//		 MEFeatureList::insertFeature
// ---------------------------------------------------------------------------------

void MEFeatureList::insertFeature(int index, MEFeature *inFeature)
{
	insert(index, inFeature, inFeature->getOwner()==NULL);
}

// ---------------------------------------------------------------------------------
//		 MEFeatureList::removeFeature
// ---------------------------------------------------------------------------------

MEFeature* MEFeatureList::removeFeature(int index)
{
	return (MEFeature *)remove(index);
}


// ---------------------------------------------------------------------------------
//		MEFeatureList::compare
// ---------------------------------------------------------------------------------

int MEFeatureList::compare(MEObject *inObject1, MEObject *inObject2)
{
	double n1, n2;
	switch (sortType) {
		case kSortByName:  return MEList::compare(inObject1, inObject2);
		case kSortByPosition:
			n1=((MEFeature *)inObject1)->getStart();
			n2=((MEFeature *)inObject2)->getStart();
		break;
		case kSortByLength:
			n1=((MEFeature *)inObject1)->getLength();
			n2=((MEFeature *)inObject2)->getLength();
		break;
		case kSortByScore:
			n1=((MEFeature *)inObject1)->getScore();
			n2=((MEFeature *)inObject2)->getScore();
		break;
	}
	
	if (n1<n2)
		return -1;
	else if (n1>n2)
		return 1;
	else 
		return 0;
}

// ---------------------------------------------------------------------------------
//		 MEFeatureList::readObject
// ---------------------------------------------------------------------------------

// Just call the MEList base function - there are no additional fields at present.
void MEFeatureList::readObject(MEStream *stream)
{
	MEList::readObject(stream);
}

// ---------------------------------------------------------------------------------
//		 MEFeatureList::writeObject
// ---------------------------------------------------------------------------------

// Just call the MEList base function - there are no additional fields at present.
void MEFeatureList::writeObject(MEStream *stream)
{
	MEList::writeObject(stream);
}


