/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEFeatureList.h

		Interface for the MEFeatureList Class
					
 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "Advanced Sequences"
 
#ifndef _MEFEATURELIST_H_
#define _MEFEATURELIST_H_

#include "MEList.h"

enum {
	kSortByName,
	kSortByPosition,
	kSortByLength,
	kSortByScore
};

class MEStream;
class MEFeature;

/******************************************************************************
	class MEFeatureList
 ******************************************************************************/
#pragma mark MEFeatureList class

class MEFeatureList : public MEList {
	
	public:
		virtual long		getID() const { return 'MFeL'; }
		//: The ID of this object - 'MFeL'.
		
		virtual char*		getClassName() const { return "Feature List"; }
		//: Returns the name of this class - "Feature List"
		
	protected:
		int							sortType;
		
	public:
	 								MEFeatureList();						
 	 	virtual						~MEFeatureList();
	 	
		virtual void 				cloneList(MEList *src);
		
		virtual void				setSortType(int inType) { sortType=inType; }
		virtual int					getSortType() { return sortType; }
		
		virtual int					getNumFeatures() { return numObjects; }
		
		virtual MEObject*			createObject();

		virtual MEFeature*			getFeature(int index);
		virtual int					addFeature(MEFeature *inFeature);
		virtual void				insertFeature(int index, MEFeature *inFeature);
		virtual MEFeature*			removeFeature(int index);

		virtual int					compare(MEObject *inObject1, MEObject *inObject2);
		
		virtual void				readObject(MEStream *stream);
		virtual void				writeObject(MEStream *stream);
};

#endif /* _MEFEATURELIST_H_ */
