/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEFlatParser.cpp

				MEFlatParser Class
	
 ******************************************************************************/

// This class parses GDE flat format sequence files. Lines starting with # give
// the name for a DNA sequence (the sequence follows on the next line). Lines
// starting with % give an amino acid sequence. 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "MEGlobal.h"
#include "MEExceptions.h"

#include "CReader.h"
#include "MEParser.h"
#include "MEFlatParser.h"
#include "MEBasicSequence.h"
#include "MEBasicAlignment.h"


// ---------------------------------------------------------------------------------
//		 MEFlatParser
// ---------------------------------------------------------------------------------

MEFlatParser::MEFlatParser(CReader *inReader) : MEParser(inReader)
{
	reader->setCommentDelimiters('\0', '\0', '\0');
}

// ---------------------------------------------------------------------------------
//		 MEFlatParser::readAlignment
// ---------------------------------------------------------------------------------

void MEFlatParser::readAlignment(MEBasicAlignment *inAlignment)
{	
	char name[MAX_NAME_LEN+1];
	MEBasicSequence *seq;

	seqType=kUnknownSequence;
	char ch=reader->readChar();
	do {
		reader->readLine(name, getMaxNameLen());
		
		if (ch=='#') {
			if (seqType==kUnknownSequence)
				seqType=kDNASequence;
			else if (seqType==kAminoAcidSequence)
				throw MEBadFormat("Unable to cope with both DNA and amino acids sequences");
		} else if (ch=='%') {
			if (seqType==kUnknownSequence)
				seqType=kAminoAcidSequence;
			else if (seqType==kDNASequence)
				throw MEBadFormat("Unable to cope with both DNA and amino acids sequences");
		} else
			throw MEBadFormat("First character of name should be '#' or '%'");
			
		seq=inAlignment->newSequence(seqType);
		seq->setName(name);
		ch=readSequenceUntil(seq, "0123456789", "#%");
		inAlignment->addSequence(seq);		
	} while (ch=='#' || ch=='%');
}

