/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEGCGParser.cpp

				MEGCGParser Class
	
 ******************************************************************************/

// This class parses GCG Rich Sequence format files. 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "MEGlobal.h"
#include "MEExceptions.h"

#include "CReader.h"
#include "MEParser.h"
#include "MEGCGParser.h"
#include "MEBasicSequence.h"
#include "MEBasicAlignment.h"


// ---------------------------------------------------------------------------------
//		 MEGCGParser
// ---------------------------------------------------------------------------------

MEGCGParser::MEGCGParser(CReader *inReader) : MEParser(inReader)
{
	strcpy(gap, "~.");
	strcpy(missing, "?");
	strcpy(match, "");
	reader->setCommentDelimiters('\0', '\0', '\0');
}

// ---------------------------------------------------------------------------------
//		 MEGCGParser::readAlignment
// ---------------------------------------------------------------------------------

void MEGCGParser::readAlignment(MEBasicAlignment *inAlignment)
{	
	char name[MAX_NAME_LEN+1];
	char desc[MAX_NAME_LEN+1];
	char token[33];
	char ch;
	int offset;
	bool hasSequence;
	MEBasicSequence *seq;

	try {
		seqType=kUnknownSequence;
		do {
			reader->skipUntil('{');
			hasSequence = false;
			offset = 0;
			do {
				ch = reader->readToken(token, 32, "}", false, false);
				if (strcmp(token, "name") == 0) {
					reader->readLine(name, getMaxNameLen(), false, false);
				} else if (strcmp(token, "descrip") == 0) {
					reader->readLine(desc, getMaxNameLen(), false, false);
				} else if (strcmp(token, "type") == 0) {
					reader->readToken(token, 32, true, false);
					if (strcmp(token, "DNA") == 0) {
						if (seqType==kUnknownSequence)
							seqType=kDNASequence;
						else if (seqType==kAminoAcidSequence)
							throw MEBadFormat("Unable to cope with both nucleotide and amino acids sequences");
					}  else if (strcmp(token, "RNA") == 0) {
						if (seqType==kUnknownSequence)
							seqType=kRNASequence;
						else if (seqType==kAminoAcidSequence)
							throw MEBadFormat("Unable to cope with both nucleotide and amino acids sequences");
					} else if (strcmp(token, "PROTEIN") == 0) {
						if (seqType==kUnknownSequence)
							seqType=kAminoAcidSequence;
						else if (seqType==kDNASequence || seqType==kRNASequence)
							throw MEBadFormat("Unable to cope with both nucleotide and amino acids sequences");
					} else
						throw MEBadFormat("Unknown sequence type");
				} else if (strcmp(token, "offset") == 0) {
					offset = reader->readInteger();
				} else if (strcmp(token, "sequence") == 0) {
					hasSequence = true;
				} else
					reader->readLine(token, 32, false, false);
			} while (!hasSequence);

			seq=inAlignment->newSequence(seqType);
			seq->setName(name);
			if (strlen(desc)>0)
				seq->setDescription(desc);

			ch=readSequenceUntil(seq, "0123456789", "}");
			
			if (offset > 0) 
				seq->insertGap(0, offset);
			
			inAlignment->addSequence(seq);
		} while (1);
	} catch (CEndOfFile) {
	}
}

