/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEIntArray.h

		Interface for the MEIntArray Class
					
 ******************************************************************************/
//! author="Andrew Rambaut"
//! group=Global
 
 
#ifndef _MEINT_ARRAY_H_
#define _MEINT_ARRAY_H_

#include "MEObject.h"

class MEStream;

/******************************************************************************
	class MEIntArray
 ******************************************************************************/
#pragma mark MEIntArray class

//:	Implements a dynamically resizing list of integers

class MEIntArray : public MEObject {
	public:
		virtual long		getID() const { return 'MInA'; }
		//: The ID of this object - 'MInL'.

		virtual char*		getClassName() const { return "Integer List"; }
		//: Returns the name of this class - "Integer List"
		
	protected:
		int 				numValues; 	//: The number of integers in the list
		int					capacity;	//: The capacity of the list
		int					increment;	//: The amount by which capacity increments
		int					iterator;	//: Used to iterate through the list

		int		 			*values;	//: An array of integers

	public:
							MEIntArray(int inInitial=INCREMENT_SIZE, 
									int inIncrement=INCREMENT_SIZE);	//: The constructor
	 	virtual				~MEIntArray();	//: The destructor		
	 	
		virtual void 		clone(MEObject *src);
		//: Clones this object from an existing one

		virtual void		setInitial(int inInitial, int inIncrement=INCREMENT_SIZE);
		//: Set the initial capacity and (optionally) the increment

		int					getNumValues() { return numValues; };
		//: Returns the number of integers in the list
		
		virtual int*		getValues() { return values; };
		//: Returns the list of MEObject pointers
		
		virtual void		setCapacity(int inCapacity);
		//: Set the capacity
		
		virtual int			getCapacity() { return capacity; };
		//: Returns the capacity

		virtual int			getIncrement() { return increment; };
		//: Sets the amount that capacity increments by
		virtual void		setIncrement(int inIncrement);
		//: Returns the increment
		
		virtual int			getValue(int index);
		//: Returns the integer that is at an index.
		
		virtual int			getSum();
		//: Returns the sum of all the integers
		
		virtual void		setValue(int index, int inValue);
		//: Sets the integer at an index

		virtual int			add(int inValue);
		//: Adds an integer to the end of the list
		
		virtual void		insert(int index, int inValue);
		//: Inserts an integer into the list at a position
		
		virtual int			remove(int index);
		//: Removes the integer at index and moves others up 
		
		virtual void		removeAll();
		//: Removes all the integer in the list

		virtual int			operator[](int index) { return getValue(index); }

		virtual int			findValue(int inValue);
		//: Finds the index of an integer in the list

	protected:
		virtual void		checkCapacity(int requiredCapacity);
		//: Checks that the integer list has enough space and resizes it as necessary
	
	public:
		virtual void		readObject(MEStream *stream);
		//: Reads the fields of this object

		virtual void		readData(MEStream *stream);
		//: Read the data of this object (i.e. the elements of the list)
		
		virtual void		writeObject(MEStream *stream);
		//: Writes the fields of this object

		virtual void		writeData(MEStream *stream);
		//: Write the data of this object (i.e. the elements of the list)
};

#endif /* _MEINT_ARRAY_H_ */
