/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MELink.h

		Interface for the MELink Class
					
 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "Trees"

#ifndef _MELINK_H_
#define _MELINK_H_

/******************************************************************************
	class MELink
 ******************************************************************************/
#pragma mark MELink class

class MEBitset;

class MELink 
{
	public:
		int					linkNo, 		// the link number 
		 					tipNo, 			// the tip number, -1 if not tip 
		 					taxonNo, 		// the taxon number, -1 if not tip 
							nodeNo, 		// the internal node number, -1 if tip 
							branchNo;		// index of the branch length in MLTree 
		
		short				flags;	 		// a general purpose set of flags 

		bool				isStem;			// the stem link points towards the root
		
		MELink 				*next,			// the next link in node (if not tip) 
							*branch;		// the link at the other end of the branch 

		MEBitset 			*navBits;		// the navigation bitset

							MELink();
							~MELink();
							
		void 				clone(MELink *inLink);
		
		void 				setIsStem(bool inIsStem) { isStem=inIsStem; };
		bool 				getIsStem() { return isStem; };
		bool 				getIsTip() { return tipNo>=0; };
		
		void 				setLinkNo(int inLinkNo) { linkNo=inLinkNo; };
		int 				getLinkNo() { return linkNo; };
		
		void 				setTipNo(int inTipNo) { tipNo=inTipNo; };
		int 				getTipNo() { return tipNo; };
		
		void 				setTaxonNo(int inTaxonNo) { taxonNo=inTaxonNo; };
		int 				getTaxonNo() { return taxonNo; };
		
		void 				setNodeNo(int inNodeNo) { nodeNo=inNodeNo; };
		int 				getNodeNo() { return nodeNo; };
		
		void 				setBranchNo(int inBranchNo) { branchNo=inBranchNo; };
		int 				getBranchNo() { return branchNo; };

		void 				setFlags(short inFlags) { flags=inFlags; }
		short				getFlags() { return flags; }
		
		void 				setFlag(short inFlag) { flags |= inFlag; }		// OR
		void 				toggleFlag(short inFlag) { flags ^= inFlag; }	// XOR
		void 				clearFlag(short inFlag) { flags &= ~inFlag; }	// NOT AND
		bool				getFlag(short inFlag) { return flags & inFlag; }
		
		void	 			setNext(MELink *inNext) { next=inNext; };
		MELink* 			getNext() { return next; };
		void	 			setBranch(MELink *inBranch) { branch=inBranch; };
		MELink* 			getBranch() { return branch; };

		MELink* 			getPrevious();

		int 				getNumBranches();
		
		void				insertBranch(MELink *inLink);
		void				removeBranch(MELink *inLink);
		MELink* 			getStemLink();
		bool				contains(MELink *inLink);
		
		bool				cladeContains(MELink *inLink);

		MEBitset*			getNavBits();
};

#endif /* _MELINK_H_ */
