/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MENBRFParser.cpp

				MENBRFParser Class
	
 ******************************************************************************/

// This class parses NBRF/Pir flat format sequence files. 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "MEGlobal.h"
#include "MEExceptions.h"

#include "CReader.h"
#include "MEParser.h"
#include "MENBRFParser.h"
#include "MEBasicSequence.h"
#include "MEBasicAlignment.h"


// ---------------------------------------------------------------------------------
//		 MENBRFParser
// ---------------------------------------------------------------------------------

MENBRFParser::MENBRFParser(CReader *inReader) : MEParser(inReader)
{
	reader->setCommentDelimiters('\0', '\0', '\0');
}

// ---------------------------------------------------------------------------------
//		 MENBRFParser::readAlignment
// ---------------------------------------------------------------------------------

void MENBRFParser::readAlignment(MEBasicAlignment *inAlignment)
{	
	char name[MAX_NAME_LEN+1];
	char desc[MAX_NAME_LEN+1];
	char ch, token[3];
	MEBasicSequence *seq;

	try {
		seqType=kUnknownSequence;
		ch=reader->readChar();
		do {
			reader->readToken(token, 2, ";", true);
			reader->readLine(name, getMaxNameLen(), false, false);
			reader->readLine(desc, getMaxNameLen(), false, true);
			
			if (token[0]=='D') {
				if (seqType==kUnknownSequence)
					seqType=kDNASequence;
				else if (seqType==kAminoAcidSequence)
					throw MEBadFormat("Unable to cope with both nucleotide and amino acids sequences");
			} else if (token[0]=='R' || token[0]=='N') {
				if (seqType==kUnknownSequence)
					seqType=kRNASequence;
				else if (seqType==kAminoAcidSequence)
					throw MEBadFormat("Unable to cope with both nucleotide and amino acids sequences");
			} else if (token[0]=='P' || token[0]=='F') {
				if (seqType==kUnknownSequence)
					seqType=kAminoAcidSequence;
				else if (seqType==kDNASequence || seqType==kRNASequence)
					throw MEBadFormat("Unable to cope with both nucleotide and amino acids sequences");
			} else
				throw MEBadFormat("First two characters of name should be standard sequence type code");
				
			seq=inAlignment->newSequence(seqType);
			seq->setName(name);
			if (strlen(desc)>0)
				seq->setDescription(desc);
			ch=readSequenceUntil(seq, "0123456789", "*>");
			inAlignment->addSequence(seq);
			if (ch=='*')
				ch=reader->skipUntil('>');
		} while (ch=='>');
	} catch (CEndOfFile) {
	}
}

