/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MENexusCreator.h

		Interface for the MENexusCreator Class
	
 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "File Creating"

#ifndef _MENEXUSCREATOR_H_
#define _MENEXUSCREATOR_H_

#include "MECreator.h"

class CWriter;
class MEBasicAlignment;
class METaxonList;
class METree;
class METreeList;
class MEMatrix;
class MEDistanceMatrix;

/******************************************************************************
	class MENexusCreator
 ******************************************************************************/
#pragma mark MENexusCreator class

class MENexusCreator : public MECreator {
	protected:
		bool hasHeader;
		
		bool useTaxaBlock;
		bool hasTaxaBlock;
		
		bool useTranslationList;
		bool hasTranslationList;
		
		bool treesBlockOpen;
		
		METaxonList*	translationList;
		
	public:
		MENexusCreator();
		MENexusCreator(CWriter *inWriter);
	
		virtual ~MENexusCreator();
		
		virtual void setHasHeader(bool inHasHeader) { hasHeader=inHasHeader; }
		virtual void setUseTaxaBlock(bool inUseTaxaBlock) { useTaxaBlock=inUseTaxaBlock; }
		virtual void setUseTranslationList(bool inUseTranslationList) { useTranslationList=inUseTranslationList; }

	protected:
		virtual void writeTaxaBlock(METaxonList *inTaxonList);

		virtual void writeDataBlockHeader(MEBasicAlignment *inAlignment, bool taxa, bool chars);
		virtual void writeCharactersBlock(MEBasicAlignment *inAlignment);
		virtual void writeDataBlock(MEBasicAlignment *inAlignment);
		
		virtual void writeTranslationList(METree *inTree);
		virtual void writeTreesBlock(METree *inTree);
		virtual void writeTreesBlock(METreeList *inTreeList);

		virtual void writeTreesBlock(MEPhylogeny *inPhylogeny);
		virtual void writeTreesBlock(MEPhylogenyList *inPhylogenyList);

		virtual void writeMatrixData(MEMatrix *inMatrix);
		virtual void writeDistancesBlock(MEDistanceMatrix *inMatrix);
		
		virtual void openHeader();
		virtual void closeTreesBlock();
		
	public:
		virtual void createAlignment(MEBasicAlignment *inAlignment);
		
		virtual void createTree(METree *inTree);
		virtual void createTreeList(METreeList *inTreeList);
		
		virtual void createPhylogeny(MEPhylogeny *inPhylogeny);
		virtual void createPhylogenyList(MEPhylogenyList *inPhylogenyList);

		virtual void createDistanceMatrix(MEDistanceMatrix *inDistanceMatrix);

		virtual void finishCreation();
};

#endif /* _MENEXUSCREATOR_H_ */
