/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MENexusParser.h

		Interface for the MENexusParser Class

 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "File Parsing"

#ifndef _MENEXUS_PARSER_H_
#define _MENEXUS_PARSER_H_

#include "MEParser.h"

enum NexusBlocks {
	kUnknownBlock,
	kTaxaBlock,
	kCharactersBlock,
	kDataBlock,
	kUnalignedBlock,
	kDistancesBlock,
	kSetsBlock,
	kCodonsBlock,
	kTreesBlock,
	kCustomBlock,
	numNexusBlocks
};

class CReader;
class MEBasicAlignment;
class METaxonMatrixList;
class MEDataMatrix;
class MECodonTable;
class METreeList;

// Class Declaration

class MENexusParser : public MEParser {			
	protected:
		NexusBlocks			nextBlock;
		
	public:
							MENexusParser(CReader *inReader);

	protected:
		virtual void		readDataBlockHeader(char *lookFor, int *nTaxa, int *nSites=NULL);
		virtual NexusBlocks	findNextBlock();
		virtual NexusBlocks	findBlockName(char *inBlockName);
		virtual void		findEndBlock();

		virtual void		readTaxaBlock(METaxonList* inTaxonList);

		virtual void		readSequenceData(MEBasicAlignment *inAlignment, 
											METaxonList* inTaxonList, int nTaxa, int nSites);
		virtual void		readContinuousData(MEDataMatrix *inDataMatrix, 
											METaxonList* inTaxonList, int nTaxa, int nChars);
											
		virtual bool		readCharactersBlock(MEBasicAlignment *inAlignment, 
											METaxonList* inTaxonList);
		virtual bool		readCharactersBlock(MEDataMatrix *inDataMatrix, 
											METaxonList* inTaxonList);
											
		virtual bool		readDataBlock(MEBasicAlignment *inAlignment);
		virtual bool		readDataBlock(MEDataMatrix *inDataMatrix);
		
		virtual bool		readUnalignedBlock(MEBasicAlignment *inAlignment, 
											METaxonList *inTaxonList);
		virtual bool		readDistancesBlock(METaxonMatrixList* inMatrixList, 
											METaxonList *inTaxonList);
		virtual bool		readSetsBlock();
		virtual bool		readCodonsBlock(MECodonTable* inCodonTable);
		virtual void		readTreesBlock(METree *inTree);
		virtual void		readTreesBlock(METreeList *inTreeList);
		
		virtual void		readAlignment(MEBasicAlignment *inAlignment);
		virtual void		readTree(METree *inTree);
		virtual void		readTreeList(METreeList *inTreeList);
		
	public:
		virtual NexusBlocks	getNextBlock();
		virtual	void		skipBlock();
		
		virtual bool		parseTaxaBlock(METaxonList* inTaxonList);
		virtual bool		parseCharactersBlock(MEBasicAlignment* inAlignment, METaxonList* inTaxonList);
		virtual bool		parseCharactersBlock(MEDataMatrix *inDataMatrix, METaxonList* inTaxonList);
		virtual bool		parseDataBlock(MEBasicAlignment *inAlignment);
		virtual bool		parseDataBlock(MEDataMatrix *inDataMatrix);
		virtual bool		parseUnalignedBlock(MEBasicAlignment* inAlignment, METaxonList* inTaxonList);
		virtual bool		parseDistancesBlock(METaxonMatrixList* inMatrixList, METaxonList* inTaxonList);
		virtual bool		parseSetsBlock();
		virtual bool		parseCodonsBlock(MECodonTable* inCodonTable);
		virtual bool		parseTreesBlock(METree *inTree);
		virtual bool		parseTreesBlock(METreeList *inTreeList);
};

#endif /* _MENEXUS_PARSER_H_ */
