/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEOptimiser.h

		Interface for the MEOptimiser Class
					
 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "Trees"

#ifndef _MEOPTIMISER_H_
#define _MEOPTIMISER_H_

#include "math.h"

#pragma mark // includes:
#include "MEObject.h"

#pragma mark // accessed classes:
class METree;
class MECriterion;
class CProgress;
class MEStream;

#pragma mark // class definition:

class MEOptimiser : public MEObject {
	protected:
		MECriterion		*mCriterion;
	
		int				kMaxBrentIterations;
		double			kBrentTolerance;
		double			kIterativeTolerance;
		
		int				numParameters;
		int				evaluateCount, lineCount;

		double			mOptimum;
		
		CProgress		*progress;

	public:
							MEOptimiser();
		virtual			~MEOptimiser();
	
		virtual void	setCriterion(MECriterion *inCriterion);
		virtual void	setProgress(CProgress *inProgress) { progress = inProgress; }
		
		virtual double	optimise();
		
	protected:
		virtual double	findOptimum();
		
		int 				estimateRemaining(double oldDiff, double newDiff);

		double 			sign(double a, double b) { return (b >= 0.0 ? fabs(a) : -fabs(a)); }
		void 				shift(double &a, double &b, double &c, double d) { a=b; b=c; c=d; }
		double 			brent(int inIndex, double inValue, double inMinValue, double inMaxValue);
		
		virtual double	evaluate(int inIndex, double inValue);

	public:
		virtual void	readObject(MEStream *stream);
		//: Reads the fields of this object from an MEStream
		
		virtual void	writeObject(MEStream *stream);
		//: Writes the fields of this object to an MEStream
};


#endif /* _MEOPTIMISER_H_ */
