/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEParser.h

		Interface for the MEParser Class
	
 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "File Parsing"

#ifndef _MEPARSER_H_
#define _MEPARSER_H_

#include "MEGlobal.h"

// Used to specify the parser format type.
enum FormatType {
	kFormatUnknown,	
	kFormatNEXUS,
	kFormatXML,
	kFormatPHYLIP,
	kFormatMOLPHY,
	kFormatMEGA,
	kFormatFLAT,
	kFormatNBRF,
	kFormatFASTA,
	kFormatCLUSTAL,
	kFormatGCG,
	kFormatGENBANK,
	kFormatPAG,
	kFormatCAIC,
	kFormatCOMPARE,
	kFormatTAB,
	kFormatTree,
	kFormatPAMLTree,
	kFormatPretty,
	numFormatTypes
};

// The names of the format types
extern const char* kFormatTypeNames[numFormatTypes]; 

// Whether the format is supported for input
extern const bool kInputFormatSupported[numFormatTypes]; 


class CReader;
class METaxonList;
class MEBasicSequence;
class MEBasicAlignment;
class MELink;
class METree;
class METreeList;
class MEDataMatrix;
class MEDataMatrixList;

/******************************************************************************
	class MEParser
 ******************************************************************************/
#pragma mark MEParser class

class MEParser {			// Class Declaration
	protected:
		CReader *reader;
		
		SequenceType seqType;
		bool isInterleaved;
		char gap[MAX_CHARS], missing[MAX_CHARS], match[MAX_CHARS];
		
		bool firstBranch;
		
	public:
									MEParser(CReader *inReader);
		virtual 					~MEParser();
		
		static FormatType 			detectFormat(CReader *inReader);
		static MEParser* 			createParserByType(FormatType inType, CReader *inReader);

		virtual MEBasicAlignment*	parseAlignment();
		virtual void 				parseAlignment(MEBasicAlignment *inAlignment);
												
		virtual METree*				parseTree();
		virtual void				parseTree(METree *inTree);

		virtual METreeList*			parseTreeList();
		virtual void				parseTreeList(METreeList *inTreeList);
		
		virtual MEDataMatrix*		parseDataMatrix();
		virtual void				parseDataMatrix(MEDataMatrix *inDataMatrix);
		
		virtual void				parseDataAndTree(MEDataMatrix *inDataMatrix, 
														METree *inTree);
		virtual void				parseDataAndTreeList(MEDataMatrixList *inDataMatrices, 
														METreeList *inTreeList);
		
	protected:
		virtual void				readAlignment(MEBasicAlignment *inAlignment);
		virtual void				readTree(METree *inTree);
		virtual void				readTreeList(METreeList *inTreeList);
		virtual void 				readDataMatrix(MEDataMatrix *inDataMatrix);
		virtual void 				readDataAndTree(MEDataMatrix *inDataMatrix, 
													METree *inTree);
		virtual void 				readDataAndTreeList(MEDataMatrixList *inDataMatrices, 
														METreeList *inTreeList);

		virtual int 				readSequence(char *seq, int maxLen, char *ignore);
		virtual char 				readSequenceUntil(char *seq, int maxBases, char *ignore, char *delimiters, int *nRead);
		virtual char 				readSequenceUntil(MEBasicSequence *seq, char *ignore, char *delimiters);

		virtual bool				readTipLabel(char *outName, long *outNumber);
		virtual MELink*				readTip(METree *inTree);
		virtual MELink*				readBranch(METree *inTree, MELink *inLink, MEDataMatrix *inData);
		virtual MELink*				readNode(METree *inTree, MEDataMatrix *inData);
		virtual void 				readNewickTree(METree *inTree, MEDataMatrix *inData=NULL);
};

#endif /* _MEPARSER_H_ */
