/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEParserUtils.cpp

				MEParserUtils Class
	
 ******************************************************************************/

// MEParserUtils provides a set of static functions for automatically
// parsing data from a reader.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "MEGlobal.h"
#include "MEExceptions.h"

#include "CReader.h"
#include "MEParser.h"
#include "MEPhylipParser.h"
#include "MENexusParser.h"

#include "MEBasicAlignment.h"
#include "METreeList.h"
#include "METree.h"

#include "MEParserUtils.h"

// ---------------------------------------------------------------------------------
//		 MEParserUtils::parse
// ---------------------------------------------------------------------------------

void MEParserUtils::parse(CReader *inReader, MEBasicAlignment *inAlignment)
{
	MEParser *parser;
	
	parser=createParser(inReader);
	parser->parseAlignment(inAlignment);
	
	delete parser;	
}


void MEParserUtils::parse(CReader *inReader, METree *inTree)
{
	MEParser *parser;
	
	parser=createParser(inReader);
	parser->parseTree(inTree);
	
	delete parser;
}


void MEParserUtils::parse(CReader *inReader, METreeList *inTreeList)
{
	MEParser *parser;
	
	parser=createParser(inReader);
	parser->parseTreeList(inTreeList);
	
	delete parser;
}


void MEParserUtils::parse(CReader *inReader, MEBasicAlignment *inAlignment, 
						METree *inTree)
{
	MEParser *parser;
	
	parser=createParser(inReader);
	parser->parseAlignment(inAlignment);
	inTree->setTaxonList(inAlignment, false);
	parser->parseTree(inTree);
	
	delete parser;
}


void MEParserUtils::parse(CReader *inReader, MEBasicAlignment *inAlignment, 
						METreeList *inTreeList)
{
	MEParser *parser;
	
	parser=createParser(inReader);
	parser->parseAlignment(inAlignment);
	inTreeList->setTaxonList(inAlignment, false);
	parser->parseTreeList(inTreeList);
	
	delete parser;
}


MEParser* MEParserUtils::createParser(CReader *inReader)
{
	FormatType format=MEParser::detectFormat(inReader);
	return MEParser::createParserByType(format, inReader);
}
