/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	MEParsimonyCriterion.cpp

		Interface for the MEParsimonyCriterion Class
				
 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = Analysis

#ifndef _MEPARSIMONY_CRITERION_H_
#define _MEPARSIMONY_CRITERION_H_

#include "METreeCriterion.h"

class MEPatternList;
class MELink;
class METree;


/******************************************************************************
	class MEParsimonyCriterion
 ******************************************************************************/
#pragma mark MEParsimonyCriterion class

//:	This class provides a simple parsimony criterion for a pattern list given a tree
class MEParsimonyCriterion : public METreeCriterion {
	protected:
		int						numStates;
		MEPatternList			*patterns;
				
	public:
								MEParsimonyCriterion();
								
		virtual void			setPatterns(MEPatternList *inPatterns);
								
		virtual int				getNumParameters() { return 0; }
		virtual double			getValue(int inIndex) { return 0.0; }
		virtual double			getMinValue(int inIndex) { return 0.0; }
		virtual double			getMaxValue(int inIndex) { return 0.0; }
		
		virtual void			setValue(int inIndex, double inValue) { }
		
	protected:
		virtual void			calcScore();
		virtual void			calcScoreForNode(MELink *inNode, int site, char *outSet);

		virtual bool			isEmptySet(char *inSet);
		virtual void			intersectSets(char *inSet1, char *inSet2, char *outSet);
		virtual void			unionSets(char *inSet1, char *inSet2, char *outSet);
};

#endif /* _MEPARSIMONY_CRITERION_H_ */

