/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEPatristicMatrix.h

				MEPatristicMatrix Class
					
 ******************************************************************************/

#pragma mark // accessed classes
class MEBitset;
class MEBitsetList;
class METree;


#pragma mark // includes
#include "MEDistanceMatrix.h"


#pragma mark // class definition
class MEPatristicMatrix : public MEDistanceMatrix {
public:
// methods:
	MEPatristicMatrix();
	virtual ~MEPatristicMatrix();
	
	virtual double		getDistancesFrom(MEBitset *A, MEDistanceMatrix *M = NULL);
	virtual double		getLeastSquaresFit(MEDistanceMatrix *M, 
							double inBranchLength = 0.0, int inBranch = -1);
	
	virtual void			minimizeLeastSquares(MEDistanceMatrix *M);
	
	virtual void			setTree(METree *T);

protected:
	virtual double		Brent(int inBranch, double inMin, double inMax, 
							double &inCurrent, MEDistanceMatrix *D);
	virtual void			calcLeastSquaresFit(MEDistanceMatrix *M, 
							double inBranchLength, int inBranch);

// variables:
	bool
		_fitKnown;
		
	double
		fit,
		r;
		
	MEBitsetList
		*pathsWithBranch,	// pathsWithBranch[i] = set of paths with branch i
		*branchesInPath;	// branchesInPath[i] = set of branches in path i
			// paths are indexed 0=(0,1), 1=(0,2), 2=(1,2), 3=(0,3), ...
		
	METree
		*tree;
};
