/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	MEPatternList.h

		Interface for the MEPatternList Class
					
	Stores and sorts patterns. These are compiled from an MEAlignment. The
	sequences in this should all have the same number of states.
    
 ******************************************************************************/

#ifndef _MEPATTERN_LIST_H_
#define _MEPATTERN_LIST_H_

#include "MEObject.h"
#include "METaxonList.h"

class MEBasicAlignment;
class METaxonList;

/******************************************************************************
	class MEPatternList
 ******************************************************************************/
#pragma mark MEPatternList class

class MEPatternList : public MEObject {
	public:
		virtual long		getID() const { return 'MPat'; }
		//: The ID of this object - 'MPat'.

		virtual char*		getClassName() const { return "Patterns"; }
		//: Returns the name of this class - "Patterns"
		
// variables:
	protected:
		char
			**patternStrings;

		double
			*patternWeights,
			*normWeights;
		
		int
			numPatterns,
			numTaxa,
			numStates,
			patternCapacity;
			
		bool
			normWeightsKnown;
		
		METaxonList
			*taxonList;
			
// methods:
	public:
							MEPatternList();
		virtual 			~MEPatternList();
	
		virtual int			getNumPatterns() { return numPatterns; }
		virtual int			getNumTaxa() { return numTaxa; }
		virtual int			getNumStates() { return numStates; }
		
		virtual void		setDimensions(int inNumTaxa, int inNumStates);

		virtual void		addPatterns(MEBasicAlignment *inAlignment);
		virtual int			addPattern(char *inPattern, double inWeight, bool makeCopy=false);
		
		virtual void		setPattern(int index, char *inPattern, double inWeight, bool makeCopy);
		virtual void		setPatternWeight(int index, double w);
		virtual void		setPatterns(char **inPatterns, double *inWeights, 
									int inNumPatterns, int inNumTaxa, int inNumStates);

		virtual char*		getPattern(int index);
		virtual void		getPattern(int index, char *inPattern);
		virtual void		getPattern(int index, int taxon, char *inPattern);

		virtual char**		getPatternStrings() { return patternStrings; }
		virtual char*		getSequence(int index, char *str = NULL);
		virtual char		getState(int n, int s);

		virtual double*		getEmpiricalFrequencies(double *inFrequencies = NULL);
		virtual double*		getEqualFrequencies(double *inFrequencies = NULL);

		virtual int			removePattern(int index, double inWeight);
		virtual int			removePattern(char *inPattern, double inWeight);
		virtual void		resetPatternStrings();
		
		virtual int			findPattern(char *searchPattern);
		virtual bool		isSamePatterns(char *pattern1, char *pattern2);

		virtual double		getNormPatternWeight(int index);
		virtual double		getPatternWeight(int index);
		virtual double*		getPatternWeights() { return patternWeights; }
		virtual double		getTotalPatternWeight();

		virtual METaxonList*	getTaxonList() { return taxonList; }
		
		virtual void		setTaxonList(METaxonList *inTaxonList) { taxonList = inTaxonList; }
		
		virtual void		sort(bool _ascending = false);

	protected:
		virtual void		checkCapacity(int requiredCapacity);

		virtual void		normaliseWeights();
};

#endif /* _MEPATTERN_LIST_H_ */
