/******************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEPermutation.h

 ******************************************************************/


#ifndef PERMUTATION_H
#define PERMUTATION_H

#include "MEUtilities.h"

//  defines:
#ifndef kMaxPermutationSize
 #define kMaxPermutationSize 2048
#endif
#define one2one		1
#define onto		2

// accessed classes:
class MEBitset;

//  class definition:
class permutation  {
public:
	virtual long					getID() const { return 'Perm'; }
	virtual char*					getClassName() const { return "Permutation"; }
	permutation(int d = 0);
	virtual ~permutation();

	virtual bool					_contains(MEBitset &bs);
	virtual bool					_contains(permutation &p);
	virtual bool					_contains(int x);
	virtual bool					_same(const permutation &p);

	virtual permutation&			choose(int n, permutation &p);

	virtual permutation&			clear();

	virtual permutation&			copyFrom(const permutation &p);
	virtual permutation&			copyFrom(const MEBitset &b);

	virtual permutation&			del(int pos, int x);
	virtual permutation&			delElement(int x);
	virtual permutation&			delIthElement(int i);
	virtual permutation&			delLast(int y);

	virtual permutation&			fillFrom(int x);
	virtual permutation&			fillRandom(const MEBitset &b);
	virtual permutation&			fillRandomMap(int min, int max, int type);
	virtual permutation&			fillRepeat(const MEBitset &b);
	virtual permutation&			fillStep(int maxVal);

	virtual int						getCard() { return dim; }
	virtual int						getIthElement(int i);
	virtual int						getSize() const { return size; }

	virtual permutation&			insert(int pos, int x);

	virtual bool					isFirstLex();
	virtual bool					isFirstLexSection(int start, int end);
	virtual bool					isLastLex();
	virtual bool					isLastLexSection(int start, int end);

	virtual int						operator[](int i) const;
	virtual permutation&			operator=(const permutation &pi);
	virtual bool					operator==(const permutation &pi) { return this->_same(pi); }
	virtual bool					operator!=(const permutation &pi) { return !this->_same(pi); }
	friend ostream&					operator<<(ostream &o, const permutation *p);

	virtual permutation&			prefix(int x);

	virtual permutation&			putNextJT(permutation &pi);
	virtual permutation&			putNextLex(permutation &pi);
	virtual permutation&			putPrevLex(permutation &pi);
	virtual permutation&			putSection(int start, int end, permutation &pi);

	virtual permutation&			randomize();

	virtual permutation&			renew(int d);
	virtual permutation&			resize(int d);

	virtual permutation&			setBnBNextLex(int pos);
	virtual permutation&			setFirstLex() { return sort(ascending); }
	virtual permutation&			setIthElement(int pos, int x);
	virtual permutation&			setLastLex() { return sort(descending); }
	virtual permutation&			setNextLex();
	virtual permutation&			setPrevLex();
	virtual permutation&			setSection(int start, int end, permutation &pi);
	virtual permutation&			setSectionFirstLex(int start, int end);
	virtual permutation&			setSectionLastLex(int start, int end);
	virtual permutation&			setSectionNextLex(int start, int end);

	virtual permutation&			sort(SortOrder direction = ascending, double *d = NULL);
	
	virtual void					strm(ostream &os) { os << write(); }
	virtual void					strmSelect(int start, int end, ostream &os);
	virtual void					strmSelect(int pos, ostream &os);
	
	virtual permutation&			suffix(int x);
	virtual permutation&			suffix(permutation *addp);
	virtual permutation&			suffix(const MEBitset &b);

	virtual permutation&			swap(int pos1, int pos2);
	virtual permutation&			swapRandomPair();

	virtual permutation&			truncate(int y);

	virtual permutation&			wipe();

	virtual char*					write(char *str = NULL);
	virtual char*					writeSelection(char *str, int begin, int end);

protected:
	// variables:
	int
		dim,	// number of elements in the permutation
		*perm,
		size;	// number of places in the permutation ( dim <= size )
};

#endif