/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEPhylipCreator.cpp

				MEPhylipCreator Class
	
 ******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "CWriter.h"
#include "MECreator.h"
#include "MEPhylipCreator.h"
#include "MEBasicSequence.h"
#include "MEBasicAlignment.h"
#include "METreeList.h"
#include "METree.h"
#include "MEPhylogenyList.h"
#include "MEPhylogeny.h"
#include "MEDistanceMatrix.h"


/******************************************************************************
 MEPhylipCreator
 
 ******************************************************************************/

MEPhylipCreator::MEPhylipCreator() : MECreator()
{
	hasSequences=false;
	useNumTaxa = useNumTrees = false;
	numTaxa = numTrees = 0;
	hasHeader = false;
}


MEPhylipCreator::MEPhylipCreator(CWriter *inWriter) : MECreator(inWriter)
{
	hasSequences=false;
	useNumTaxa = useNumTrees = false;
	numTaxa = numTrees = 0;
	hasHeader = false;
}

void MEPhylipCreator::createAlignment(MEBasicAlignment *inAlignment)
{
	writer->writeInteger(inAlignment->getNumSequences());
	writer->writeChar(' ');
	writer->writeInteger(inAlignment->getNumSites());
	writer->writeChar('\r');
		
	writeSequenceData(inAlignment, 0, 10, false, false, false, false, false, false, 0);
	hasSequences=true;
}


void MEPhylipCreator::createTree(METree *inTree)
{	
	if (!hasHeader) {
		if (useNumTaxa && useNumTrees) {
			writer->writeInteger(numTaxa);
			writer->writeChar('\t');
			writer->writeInteger(numTrees);
			writer->writeChar('\r');
		} else if (useNumTaxa) {
			writer->writeInteger(numTaxa);
			writer->writeChar('\r');
		} else if (useNumTrees) {
			writer->writeInteger(numTrees);
			writer->writeChar('\r');
		}
		hasHeader = true;
	}
		
	writeTree(inTree);
	writer->writeChar('\r');
}

void MEPhylipCreator::createTreeList(METreeList *inTreeList)
{	
	int i, n;
	
	if (!hasHeader) {
		if (useNumTaxa && useNumTrees) {
			writer->writeInteger(numTaxa);
			writer->writeChar('\t');
			writer->writeInteger(numTrees);
			writer->writeChar('\r');
		} else if (useNumTaxa) {
			writer->writeInteger(numTaxa);
			writer->writeChar('\r');
		} else if (useNumTrees) {
			writer->writeInteger(numTrees);
			writer->writeChar('\r');
		}
		hasHeader = true;
	}
	
	n=inTreeList->getNumTrees();	
	for (i=0; i<n; i++) {
		writeTree(inTreeList->getTree(i));
		writer->writeChar('\r');
	}
}


void MEPhylipCreator::createPhylogeny(MEPhylogeny *inPhylogeny)
{	
	if (!hasHeader) {
		if (useNumTaxa && useNumTrees) {
			writer->writeInteger(numTaxa);
			writer->writeChar('\t');
			writer->writeInteger(numTrees);
			writer->writeChar('\r');
		} else if (useNumTaxa) {
			writer->writeInteger(numTaxa);
			writer->writeChar('\r');
		} else if (useNumTrees) {
			writer->writeInteger(numTrees);
			writer->writeChar('\r');
		}
		hasHeader = true;
	}
		
	writePhylogeny(inPhylogeny);
	writer->writeChar('\r');
}

void MEPhylipCreator::createPhylogenyList(MEPhylogenyList *inPhylogenyList)
{	
	int i, n;
	
	if (!hasHeader) {
		if (useNumTaxa && useNumTrees) {
			writer->writeInteger(numTaxa);
			writer->writeChar('\t');
			writer->writeInteger(numTrees);
			writer->writeChar('\r');
		} else if (useNumTaxa) {
			writer->writeInteger(numTaxa);
			writer->writeChar('\r');
		} else if (useNumTrees) {
			writer->writeInteger(numTrees);
			writer->writeChar('\r');
		}
		hasHeader = true;
	}
	
	n=inPhylogenyList->getNumPhylogenies();	
	for (i=0; i<n; i++) {
		writePhylogeny(inPhylogenyList->getPhylogeny(i));
		writer->writeChar('\r');
	}
}

void MEPhylipCreator::writeMatrixData(MEMatrix *inMatrix)
{
	int i, j, n, nameLen, startCol, stopCol;
	char *name;
	
	n=inMatrix->getNumRows();
		
	for (i=0; i<n; i++) {
		name=inMatrix->getRowName(i);
		nameLen=strlen(name);
		if (nameLen<10) {
			writer->writeString(name);
			writer->writeSpaces(10-nameLen);
		} else
			writer->writeText(name, 10);

		if (inMatrix->getHasLowerTriangle())
			startCol = 0;
		else if (inMatrix->getHasDiagonal())
			startCol = i;
		else // must have upper triangle
			startCol = i + 1;
			
		if (inMatrix->getHasUpperTriangle())
			stopCol = n;
		else if (inMatrix->getHasDiagonal())
			stopCol = i + 1;
		else // must have lower triangle
			stopCol = i;
			
		for (j = startCol; j < stopCol; j++) {
			writer->writeDouble(inMatrix->getElement(i, j));
			if (j < stopCol - 1)
				writer->writeChar(' ');
		}
		writer->writeChar('\r');
	}
}

void MEPhylipCreator::createDistanceMatrix(MEDistanceMatrix *inDistanceMatrix)
{
	writer->writeInteger(inDistanceMatrix->getNumRows());
	writer->writeChar('\r');
		
	writeMatrixData(inDistanceMatrix);
}