/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEPhylipCreator.h

		Interface for the MEPhylipCreator Class
	
 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "File Creating"

#ifndef _MEPHYLIPCREATOR_H_
#define _MEPHYLIPCREATOR_H_

#include "MECreator.h"

class CWriter;
class MEBasicAlignment;
class METree;
class METreeList;
class MEPhylogeny;
class MEPhylogenyList;
class MEMatrix;
class MEDistanceMatrix;

/******************************************************************************
	class MEPhylipCreator
 ******************************************************************************/
#pragma mark MEPhylipCreator class

class MEPhylipCreator : public MECreator {
	protected:
		bool hasSequences;
		bool useNumTaxa;
		bool useNumTrees;
		
		int	numTaxa, numTrees;
		
		bool hasHeader;
				
	public:
		MEPhylipCreator();
		MEPhylipCreator(CWriter *inWriter);
	
		virtual void setUseNumTaxon(bool inUseNumTaxa, int inNumTaxa = 0) 
			{ useNumTaxa = inUseNumTaxa; numTaxa = inNumTaxa; }
		virtual void setUseNumTrees(bool inUseNumTrees, int inNumTrees = 0) 
			{ useNumTrees = inUseNumTrees; numTrees = inNumTrees; }
		virtual void setNumTaxon(bool inUseNumTaxa) { useNumTaxa = inUseNumTaxa; }

		virtual void createAlignment(MEBasicAlignment *inAlignment);
		virtual void createTree(METree *inTree);
		virtual void createTreeList(METreeList *inTreeList);
		
		virtual void createPhylogeny(MEPhylogeny *inPhylogeny);
		virtual void createPhylogenyList(MEPhylogenyList *inPhylogenyList);
		
		virtual void createDistanceMatrix(MEDistanceMatrix *inDistanceMatrix);

		virtual void finishCreation() { hasHeader = false; }

	protected:
		virtual void writeMatrixData(MEMatrix *inMatrix);
};

#endif /* _MEPHYLIPCREATOR_H_ */
