/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEPhylipParser.h

		Interface for the MEPhylipParser Class
	
 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "File Parsing"

#ifndef _MEPHYLIP_PARSER_H_
#define _MEPHYLIP_PARSER_H_

#include "MEParser.h"

class MEBasicAlignment;
class METreeList;
class CReader;
class MEDataMatrix;
class MEDataMatrixList;

class MEPhylipParser : public MEParser {			// Class Declaration
	protected:
		bool			interleavedKnown;
		bool			isStrict;
		
	public:
		MEPhylipParser(CReader *inReader);
	
		virtual void 	setIsInterleaved(bool inIsInterleaved);
		virtual bool	detectInterleaved(int nTaxa);

		virtual void 	setIsStrict(bool inIsStrict);
		
	protected:
		virtual int 	getMaxNameLen() { return 255; };
	
		virtual void 	readAlignment(MEBasicAlignment *inAlignment);
		virtual void	readTree(METree *inTree);
		virtual void	readTreeList(METreeList *inTreeList);

		virtual void	readDataMatrix(MEDataMatrix *inDataMatrix);
		
		virtual void 	readDataAndTree(MEDataMatrix *inDataMatrix, METree *inTree);
		virtual void 	readDataAndTreeList(MEDataMatrixList *inDataMatrices, METreeList *inTreeList);
};

#endif /* _MEPHYLIP_PARSER_H_ */
