/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEPhylogenyList.cpp

				MEPhylogenyList Class
				
 ******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "MEGlobal.h"
#include "MEExceptions.h"

#include "MEPhylogenyList.h"
#include "MEPhylogeny.h"
#include "MEStream.h"


#pragma mark === MEPhylogenyList ===

// ---------------------------------------------------------------------------------
//		 MEPhylogenyList
// ---------------------------------------------------------------------------------

MEPhylogenyList::MEPhylogenyList() : 
	MEList()
{
}

// ---------------------------------------------------------------------------------
//		 ~MEPhylogenyList
// ---------------------------------------------------------------------------------

MEPhylogenyList::~MEPhylogenyList()
{
}

// ---------------------------------------------------------------------------------
//		MEPhylogenyList::createObject
// ---------------------------------------------------------------------------------

MEObject* MEPhylogenyList::createObject()
{
	MEObject *obj;
	NewClass(obj, MEPhylogeny());
	return obj;
}

// ---------------------------------------------------------------------------------
//		 MEPhylogenyList::getPhylogeny
// ---------------------------------------------------------------------------------

MEPhylogeny* MEPhylogenyList::getPhylogeny(int index)
{
	return (MEPhylogeny *)getObject(index);
}

// ---------------------------------------------------------------------------------
//		 MEPhylogenyList::addPhylogeny
// ---------------------------------------------------------------------------------

int MEPhylogenyList::addPhylogeny(MEPhylogeny *inPhylogeny)
{
	return add(inPhylogeny, inPhylogeny->getOwner()==NULL);
}

// ---------------------------------------------------------------------------------
//		 MEPhylogenyList::insertPhylogeny
// ---------------------------------------------------------------------------------

void MEPhylogenyList::insertPhylogeny(int index, MEPhylogeny *inPhylogeny)
{
	insert(index, inPhylogeny, inPhylogeny->getOwner()==NULL);
}

// ---------------------------------------------------------------------------------
//		 MEPhylogenyList::removePhylogeny
// ---------------------------------------------------------------------------------

MEPhylogeny* MEPhylogenyList::removePhylogeny(int index)
{
	return (MEPhylogeny *)remove(index);
}


// ---------------------------------------------------------------------------------
//		 MEPhylogenyList::getMemoryUsage
// ---------------------------------------------------------------------------------

int MEPhylogenyList::getMemoryUsage()
{
	int sum = 0;
	for (int i = 0; i < numObjects; i++)
		sum += getPhylogeny(i)->getMemoryUsage();
	return sum;
}

#pragma mark === Object I/O ===

// ---------------------------------------------------------------------------------
//		 MEPhylogenyList::readObject
// ---------------------------------------------------------------------------------

void MEPhylogenyList::readObject(MEStream *stream)
{
	MEList::readObject(stream);
}

// ---------------------------------------------------------------------------------
//		 MEPhylogenyList::writeObject
// ---------------------------------------------------------------------------------

void MEPhylogenyList::writeObject(MEStream *stream)
{
	MEList::writeObject(stream);
}


