/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEPhylogenySimulator.h

		Interface for the MEPhylogenySimulator Class
					
 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "Trees"
 
#ifndef _MEPHYLOGENY_SIMULATOR_H_
#define _MEPHYLOGENY_SIMULATOR_H_

class METaxonList;
class METree;
class METreeList;

class MEPhylogeny;

class CIntArray;
class CDoubleArray;
class CProgress;


// =========================================================================
//	class MEGenotype
// =========================================================================

class MEGenotype {
	friend class MELineage;
	
	public:
		
							MEGenotype();
};

// =========================================================================
//	class MEPhylogenySimulator
// =========================================================================

class MEPhylogenySimulator {
	public:
		static int			mCapacityIncrement;

	protected:
		MEPhylogeny			*mPhylogeny;
		
		MEGenotype			**mGenotypes;
		long				mNumGenotypes;
		long				mGenotypeCapacity;
		
		long				*mAvailableGenotypes;
		long				mNumAvailableGenotypes;
		long				mAvailableGenotypeCapacity;
		
		bool				mPruneExtinctLineages;
		
		double				mCurrentTime;
			
		CIntArray			*mBirthOrder;
			
		CProgress			*mProgress;

	public:
		
							MEPhylogenySimulator();
		virtual				~MEPhylogenySimulator();
			
		virtual void 		setPruneExtinctLineages(bool inPruneExtinctLineages) 
								{ mPruneExtinctLineages = inPruneExtinctLineages; }

		virtual void		setProgress(CProgress *inProgress) 
								{ mProgress = inProgress; }
		
		virtual void		initialize();
		
		// This is an abstract class because of this method. This will typically be
		// a loop calling lineageBirth and lineageDeath until some stopping conditions.
		virtual void		simulate() = 0;

		virtual void 		setPhylogeny(MEPhylogeny* inPhylogeny);
		
		virtual MEPhylogeny *getPhylogeny() { return mPhylogeny; }

		virtual MEPhylogeny *getWholePhylogeny(bool inExtantOnly = true);
		virtual void		getWholePhylogeny(MEPhylogeny* ioPhylogeny, bool inExtantOnly = true);
		
		virtual MEPhylogeny *getSamplePhylogeny(long inNumLineages, bool inExtantOnly = true);
		virtual void		getSamplePhylogeny(MEPhylogeny* ioPhylogeny, long inNumLineages, bool inExtantOnly = true);
		
		virtual MEPhylogeny *getSamplePhylogeny(double inProbability, bool inExtantOnly = true);
		virtual void		getSamplePhylogeny(MEPhylogeny* ioPhylogeny, double inProbability, bool inExtantOnly = true);
		
		virtual void		setSampleFromFlags();
		
		virtual long		getNumLineages() { return mPhylogeny->getNumLineages(); }
		virtual long		getNumExtantLineages() { return mPhylogeny->getNumExtantLineages(); }
		
	protected:
		virtual bool		lineageBirth(long inExtantIndex, int inNumOffspring, bool inParentSurvives = false);
		virtual bool		lineageDeath(long inExtantIndex);
		
		virtual void		mutateLineage(long inParent, long inOffspring) = 0;
		virtual void		cloneLineage(long inParent, long inOffspring) = 0;

	
		virtual MEPhylogeny* newPhylogeny() = 0;
		
		virtual MEGenotype*	newGenotype() = 0;
		
		virtual long		createGenotype();
		virtual void		disposeGenotype(long inGenotype);
		
		virtual void		checkGenotypeCapacity(long inCapacity);
		virtual void		checkAvailableGenotypeCapacity(long inCapacity);
		
	public:
		virtual void		describe(CReportRecord *ioReport) = 0;

};

#endif /* _MEPHYLOGENY_SIMULATOR_H_ */
