/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEResolutionGenerator.h

		Interface for the MEResolutionGenerator Class
					
 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "Trees"

#pragma mark // includes:
// ANSI / C++
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

// Mol. Evol.
#include "MEExceptions.h"
#include "MEGlobal.h"
#include "MEStream.h"
#include "MEMaths.h"

#include "METree.h"
#include "METaxonList.h"

#include "MEResolutionGenerator.h"


#pragma mark // methods:


// ---------------------------------------------------------------------------------
//		 MEResolutionGenerator::MEResolutionGenerator
// ---------------------------------------------------------------------------------

MEResolutionGenerator::MEResolutionGenerator() : METreeGenerator()
{
	mPolytomies = NULL;
	mNumPolytomies = 0;
	mCurrentResolution = 0;
}

// ---------------------------------------------------------------------------------
//		 MEResolutionGenerator::~MEResolutionGenerator
// ---------------------------------------------------------------------------------

MEResolutionGenerator::~MEResolutionGenerator()
{
	if (mPolytomies != NULL)
		delete [] mPolytomies;
}

// ---------------------------------------------------------------------------------
//		 MEResolutionGenerator::setSourceTree
// ---------------------------------------------------------------------------------

void MEResolutionGenerator::setSourceTree(METree *inTree)
{
	METreeGenerator::setSourceTree(inTree);
}

// ---------------------------------------------------------------------------------
//		 MEResolutionGenerator::getNumTrees
// ---------------------------------------------------------------------------------

long MEResolutionGenerator::getNumTrees()
{
	ThrowIfNull(mSourceTree);
	
	return 0;
}

// ---------------------------------------------------------------------------------
//		 MEResolutionGenerator::getApproxNumTrees
// ---------------------------------------------------------------------------------

// Returns the approximate number of trees as a floating point number.
double MEResolutionGenerator::getApproxNumTrees()
{
	ThrowIfNull(mSourceTree);
	
	return 0;
}

// ---------------------------------------------------------------------------------
//		 MEResolutionGenerator::generate
// ---------------------------------------------------------------------------------

void MEResolutionGenerator::generate(int index, METree *inTree)
{
	ThrowIfNull(mSourceTree);
	TestBounds(index, mSourceTree->getNumBranches());
	
	inTree->cloneTree(mSourceTree);
	METaxonList *taxonList = new METaxonList();
	taxonList->cloneList(mSourceTree->getTaxonList());
	inTree->setTaxonList(taxonList, true);
	
	throw MEException("Unimplemented");
}

// ---------------------------------------------------------------------------------
//		 MEResolutionGenerator::generateRandom
// ---------------------------------------------------------------------------------

void MEResolutionGenerator::generateRandom(METree *inTree)
{
	ThrowIfNull(mSourceTree);
	
	inTree->cloneTree(mSourceTree);
	METaxonList *taxonList = new METaxonList();
	taxonList->cloneList(mSourceTree->getTaxonList());
	inTree->setTaxonList(taxonList, true);
	
	inTree->resolveTree();
}

// ---------------------------------------------------------------------------------
//		 MEResolutionGenerator::generateFirst
// ---------------------------------------------------------------------------------

void MEResolutionGenerator::generateFirst(METree *inTree)
{
	mCurrentResolution = 0;
	generate(mCurrentResolution, inTree);
}

// ---------------------------------------------------------------------------------
//		 MEResolutionGenerator::generateNext
// ---------------------------------------------------------------------------------

void MEResolutionGenerator::generateNext(METree *inTree)
{
	mCurrentResolution++;
	generate(mCurrentResolution, inTree);
}

