/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MERootingGenerator.h

		Interface for the MERootingGenerator Class
					
 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "Trees"

#pragma mark // includes:
// ANSI / C++
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

// Mol. Evol.
#include "MEExceptions.h"
#include "MEGlobal.h"
#include "MEStream.h"
#include "MEMaths.h"

#include "METree.h"
#include "METaxonList.h"

#include "MERootingGenerator.h"


#pragma mark // methods:


// ---------------------------------------------------------------------------------
//		 MERootingGenerator::MERootingGenerator
// ---------------------------------------------------------------------------------

MERootingGenerator::MERootingGenerator() : METreeGenerator()
{
	mCurrentRoot = 0;
}

// ---------------------------------------------------------------------------------
//		 MERootingGenerator::~MERootingGenerator
// ---------------------------------------------------------------------------------

MERootingGenerator::~MERootingGenerator()
{
}

// ---------------------------------------------------------------------------------
//		 MERootingGenerator::getNumTrees
// ---------------------------------------------------------------------------------

long MERootingGenerator::getNumTrees()
{
	ThrowIfNull(mSourceTree);
	
	return mSourceTree->getNumBranches();
}

// ---------------------------------------------------------------------------------
//		 MERootingGenerator::getApproxNumTrees
// ---------------------------------------------------------------------------------

// Returns the approximate number of trees as a floating point number.
double MERootingGenerator::getApproxNumTrees()
{
	ThrowIfNull(mSourceTree);
	
	return mSourceTree->getNumBranches();
}

// ---------------------------------------------------------------------------------
//		 MERootingGenerator::generate
// ---------------------------------------------------------------------------------

void MERootingGenerator::generate(int index, METree *inTree)
{
	ThrowIfNull(mSourceTree);
	TestBounds(index, mSourceTree->getNumBranches());
	
	inTree->cloneTree(mSourceTree);
	METaxonList *taxonList = new METaxonList();
	taxonList->cloneList(mSourceTree->getTaxonList());
	inTree->setTaxonList(taxonList, true);
	
	MELink *link = inTree->getBranch(index);
	inTree->outgroupTree(link);
}

// ---------------------------------------------------------------------------------
//		 MERootingGenerator::generateRandom
// ---------------------------------------------------------------------------------

void MERootingGenerator::generateRandom(METree *inTree)
{
	ThrowIfNull(mSourceTree);
	
	inTree->cloneTree(mSourceTree);
	METaxonList *taxonList = new METaxonList();
	taxonList->cloneList(mSourceTree->getTaxonList());
	inTree->setTaxonList(taxonList, true);
	
	int i = MERandom::iRan(inTree->getNumBranches());
	MELink *link = inTree->getBranch(i);
	inTree->outgroupTree(link);
}

// ---------------------------------------------------------------------------------
//		 MERootingGenerator::generateFirst
// ---------------------------------------------------------------------------------

void MERootingGenerator::generateFirst(METree *inTree)
{
	mCurrentRoot = 0;
	generate(mCurrentRoot, inTree);
}

// ---------------------------------------------------------------------------------
//		 MERootingGenerator::generateNext
// ---------------------------------------------------------------------------------

void MERootingGenerator::generateNext(METree *inTree)
{
	mCurrentRoot++;
	generate(mCurrentRoot, inTree);
}

