/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	MESandersonTreeCriterion.cpp

		Interface for the MESandersonTreeCriterion Class

 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = Analysis

#ifndef _MESANDERSON_TREE_CRITERION_H_
#define _MESANDERSON_TREE_CRITERION_H_

#include "METreeCriterion.h"

enum SandersonModel {
	kRateDifferenceAtEachNode,
	kDifferenceWithMeanAtRoot,
	kDifferenceWithMeanAtEachNode
};

class MELink;
class METree;


/******************************************************************************
	class MESandersonTreeCriterion
 ******************************************************************************/
#pragma mark MESandersonTreeCriterion class

//:	This class provides a simple parsimony criterion for a pattern list given a tree
class MESandersonTreeCriterion : public METreeCriterion {
	protected:
		int						numParameters;
		double*					parameters;
			
		double					sumRates;
		int 					numRates;
								
		SandersonModel			mSandersonModel;
		
		double					mRate;
		
	public:
								MESandersonTreeCriterion();
		virtual					~MESandersonTreeCriterion();
								
		virtual void 			setTree(METree *inTree);
		virtual void 			setSandersonModel(SandersonModel inSandersonModel);

		virtual int				getNumParameters();
		virtual double			getValue(int inIndex);
		virtual double			getMinValue(int inIndex);
		virtual double			getMaxValue(int inIndex);
		virtual void			setValue(int inIndex, double inValue);
		
		virtual double			getRate();
		virtual double			getMinRate();
		virtual double			getMaxRate();
		virtual void			setRate(double inRate);

	protected:
		virtual void			calcScore();
		virtual void			calcSumRateForNode(MELink *inNode, double inRate, double inHeight);
		virtual void			calcScoreForNode(MELink *inNode, double inRate, double inHeight);
		virtual void			calcScoreForNode2(MELink *inNode, double inMeanRate, double inHeight);

		virtual void			nodeHeightsToValues(MELink *inNode, double inHeight);
		virtual void			heightsToValues();

		virtual void			nodeValuesToHeightsBelow(MELink *inNode, double inHeight);
		virtual void			valuesToHeights();

};

#endif /* _MESANDERSON_TREE_CRITERION_H_ */

