/******************************************************************************
 MESpatialSimulator.h

		Interface for the MESpatialSimulator Class
					
	Copyright  2002 Andrew Rambaut, University of Oxford. All rights reserved.

 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "Trees"
 
#ifndef _MESPACIAL_SIMULATOR_H_
#define _MESPACIAL_SIMULATOR_H_

#include "CExceptions.h"
#include "MEPhylogeny.h"
#include "MEBirthDeathSimulator.h"

class CReportRecord;
class CFigureElement;
class CComplexGraphicElement;

// =========================================================================
//	class MESpatialLineage
// =========================================================================

class MESpatialLineage : public MELineage {
	friend class MESpatialPhylogeny;
	friend class MESpatialSimulator;

	protected:
		long				mX, mY;
		long				mLinks[8];
		int					mOffspringDirection;
		
		long				mOrder;
		double				mBalance;
		
		int					mCount;
		
	public:
		
							MESpatialLineage();
};

// =========================================================================
//	class MESpatialPhylogeny
// =========================================================================

class MESpatialPhylogeny : public MEPhylogeny {

	public:
		
							MESpatialPhylogeny() : MEPhylogeny() { }
							
	protected:	
		virtual MELineage*	newLineage() { return new MESpatialLineage(); }

		virtual void		initializeLineage(long inLineageIndex);
};


// =========================================================================
//	class MESpatialSimulator
// =========================================================================

class MESpatialSimulator : public MEBirthDeathSimulator {
	protected:
		int					mNumAdjacencies;
		int 				mMode;
					
		long				mOrder;
		long				mNeighbourCount;
		
		double				mU30, mV30, mU60, mV60;

	public:
		
							MESpatialSimulator();
		virtual				~MESpatialSimulator();
			
		virtual void		setNumAdjacencies(int inNumAdjacencies);
		
		virtual void		initialize();

	protected:
		virtual bool		event();

		virtual void		mutateLineage(long inParent, long inOffspring);
		virtual void		cloneLineage(long inParent, long inOffspring);

		virtual double		timeToNextEvent();
		virtual bool		eventIsBirth();

		virtual MEPhylogeny*	newPhylogeny();

		virtual void		offsetCoordinates(int inDirection, long inX, long inY, long &outX, long &outY);
		virtual int			getOppositeDirection(int inDirection);
		virtual long		findLineageAtPosition(long inX, long inY);

	protected:
		virtual void		calcBalance();
		
	public:
		virtual void		drawSpatial(CReportRecord* ioReport);

	protected:
		virtual void		drawLineage(long inLineage, double inGridSize, 
											CComplexGraphicElement *ioGridGraphic,
											CComplexGraphicElement *ioBranchGraphic,
											CComplexGraphicElement *ioBlobGraphic,
											double &outX, double &outY);
		
		virtual void		getGridCentreCoordinates(long inX, long inY, double inGridSize, 
														double &outX, double &outY);
		virtual void		getTriangleVertexCoordinates(int inVertex, double inGridSize, 
															bool inUpsideDown,
															double &outX, double &outY);
		virtual void		getHexagonVertexCoordinates(int inVertex, double inGridSize, 
														double &outX, double &outY);
		virtual void		getSquareVertexCoordinates(int inVertex, double inGridSize, 
														double &outX, double &outY);
		
	public:
		virtual void		describe(CReportRecord *ioReport);
};

#endif /* _MESPACIAL_SIMULATOR_H_ */
