/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEStdTextStream.cpp

				MEStdTextStream Class
				
 ******************************************************************************/

// MEStdTextStream overrides METextStream to write a text format to a standard C++ stream.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "MEGlobal.h"
#include "MEExceptions.h"
#include "MEStdTextStream.h"

#pragma mark === MEStdTextStream ===

// ---------------------------------------------------------------------------------
//		 MEStdTextStream
// ---------------------------------------------------------------------------------

MEStdTextStream::MEStdTextStream() : METextStream()
{
	iStream=NULL;
	oStream=NULL;
}

MEStdTextStream::MEStdTextStream(istream *inStream) : METextStream()
{
	iStream=NULL;
	oStream=NULL;
	setStream(inStream);
}


MEStdTextStream::MEStdTextStream(ostream *inStream) : METextStream()
{
	iStream=NULL;
	oStream=NULL;
	setStream(inStream);
}


// ---------------------------------------------------------------------------------
//		 ~MEStdTextStream
// ---------------------------------------------------------------------------------

MEStdTextStream::~MEStdTextStream()
{
}


// ---------------------------------------------------------------------------------
//		 MEStdTextStream::setStream
// ---------------------------------------------------------------------------------

void MEStdTextStream::setStream(istream *inStream)
{
	iStream=inStream;
}

// ---------------------------------------------------------------------------------
//		 MEStdTextStream::setStream
// ---------------------------------------------------------------------------------

void MEStdTextStream::setStream(ostream *inStream)
{
	oStream=inStream;
}

#pragma mark === Low Level Reading ===

// ---------------------------------------------------------------------------------
//		 MEStdTextStream::readChar
// ---------------------------------------------------------------------------------

// Returns the next character from the file. 

char MEStdTextStream::readChar()
{
	char ch;
		
	iStream->get(ch);
	if (iStream->fail())
		throw CEndOfFile();

	return ch;
}


// ---------------------------------------------------------------------------------
//		 MEStdTextStream::nextChar
// ---------------------------------------------------------------------------------

// Returns the next character from the file without moving the file pointer on. 

char MEStdTextStream::nextChar()
{	
	return iStream->peek();
}


#pragma mark === Low Level Writing ===

// ---------------------------------------------------------------------------------
//		 MEStdTextStream::writeChar
// ---------------------------------------------------------------------------------

void MEStdTextStream::writeChar(char ch)
{
	*oStream << ch;
}


// ---------------------------------------------------------------------------------
//		 MEStdTextStream::writeText
// ---------------------------------------------------------------------------------

void MEStdTextStream::writeText(const char *st, int len)
{
	oStream->write(st, len);
}


// ---------------------------------------------------------------------------------
//		 MEStdTextStream::writeInteger
// ---------------------------------------------------------------------------------

void MEStdTextStream::writeInteger(int d)
{
	*oStream << d;
}


// ---------------------------------------------------------------------------------
//		 MEStdTextStream::writeDouble
// ---------------------------------------------------------------------------------


void MEStdTextStream::writeDouble(double f)
{
	*oStream << f;
}


