/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEStream.h

		Interface for the MEStream Class
					
 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "Read/Write"

#ifndef _MESTREAM_H_
#define _MESTREAM_H_

class MEObject;
class MEUndead;
class MEList;
class MEIntArray;
class MEDoubleArray;
class METree;
class MELink;
class MEMatrix;
union ObjectID;

/******************************************************************************
	class MEStream
 ******************************************************************************/
#pragma mark MEStream class

class MEStream
{
	protected:
		MEList					*cache;
		MEUndead				*undead;
		char					lastLabel[256];
		
	public:
	 							MEStream();						
	 														
	protected:
		void 					initStream();
		
	public:
	 	virtual					~MEStream();

		virtual void 			setUndead(MEUndead *inUndead);
		virtual void 			clearCache();
		
		virtual bool			get(char *label, int *i) = 0;
		virtual bool			get(char *label, double *d) = 0;
		virtual bool			get(char *label, bool *b) = 0;
		virtual bool			get(char *label, char *ch) = 0;
		virtual bool			get(char *label, char *str, int maxLen) = 0;
		virtual bool			get(char *label, ObjectID *id) = 0;
		virtual bool			get(char *label, int *d, const char **stringList, const int numStrings) = 0;
		virtual MEObject*		get(char *label) = 0;

		virtual void 			getList(MEList *inList) = 0;
		virtual void 			getIntArray(MEIntArray *inList) = 0;
		virtual void 			getDoubleArray(MEDoubleArray *inList) = 0;
		virtual void			getTree(METree *inTree) = 0;
		virtual void			getMatrix(MEMatrix *inMatrix) = 0;
		
	protected:
		virtual MEObject*		getObject();
		virtual MEObject*		getObjectRef(int ref);

	public:
		virtual void			put(char *label, int i) = 0;
		virtual void			put(char *label, double d) = 0;
		virtual void			put(char *label, bool b) = 0;
		virtual void			put(char *label, char ch) = 0;
		virtual void			put(char *label, const char *str) = 0;
		virtual void			put(char *label, const char *str, int length) = 0;
		virtual void			put(char *label, const ObjectID id) = 0;
		virtual void			put(char *label, MEObject *inObject) = 0;
	
		virtual void			putList(MEList *inList) = 0;
		virtual void			putIntArray(MEIntArray *inList) = 0;
		virtual void			putDoubleArray(MEDoubleArray *inList) = 0;
		virtual void			putTree(METree *inTree) = 0;
		virtual void			putMatrix(MEMatrix *inMatrix) = 0;
		
	protected:
		virtual void			putObject(MEObject *inObject);
};

#endif /* _MESTREAM_H_ */
