/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 METabCreator.cpp

				METabCreator Class
	
 ******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "MEExceptions.h"
#include "CWriter.h"
#include "MECreator.h"
#include "METabCreator.h"
#include "MEBasicSequence.h"
#include "MEBasicAlignment.h"
#include "METreeList.h"
#include "METree.h"
#include "MEDistanceMatrix.h"


/******************************************************************************
 METabCreator
 
 ******************************************************************************/

METabCreator::METabCreator() : MECreator()
{
	writeColumns=false;
}


METabCreator::METabCreator(CWriter *inWriter) : MECreator(inWriter)
{
	writeColumns=false;
}


void METabCreator::writeMatrixData(MEMatrix *inMatrix)
{
	int i, j, n, startCol, stopCol;
	char *name;
	
	n=inMatrix->getNumRows();
		
	if (writeColumns) {
		name = inMatrix->getName();
		if (name != NULL)
			writer->writeLabel(name);
		writer->writeChar('\r');
			
		for (i = 0; i < n-1; i++) {
			for (j = i+1; j < n; j++) {
				name=inMatrix->getRowName(i);
				writer->writeLabel(name);
				writer->writeChar('\t');
				name=inMatrix->getRowName(j);
				writer->writeLabel(name);
				writer->writeChar('\t');

				writer->writeDouble(inMatrix->getElement(i, j));
				writer->writeChar('\r');
			}
		}
	} else {
		name = inMatrix->getName();
		if (name != NULL)
			writer->writeLabel(name);
		writer->writeChar('\t');
			
		for (i=0; i<n; i++) {
			name=inMatrix->getRowName(i);
			writer->writeLabel(name);
			if (i < n - 1)
				writer->writeChar('\t');
		}
		writer->writeChar('\r');

		for (i=0; i<n; i++) {
			name=inMatrix->getRowName(i);
			writer->writeLabel(name);
			writer->writeChar('\t');

			if (inMatrix->getHasLowerTriangle())
				startCol = 0;
			else if (inMatrix->getHasDiagonal())
				startCol = i;
			else // must have upper triangle
				startCol = i + 1;
				
			if (inMatrix->getHasUpperTriangle())
				stopCol = n;
			else if (inMatrix->getHasDiagonal())
				stopCol = i + 1;
			else // must have lower triangle
				stopCol = i;
				
			for (j = 0; j < n; j++) {
				if (j >= startCol && j < stopCol)
					writer->writeDouble(inMatrix->getElement(i, j));
				if (j < n-1)
					writer->writeChar('\t');
			}
			writer->writeChar('\r');
		}
	}
}

void METabCreator::createDistanceMatrix(MEDistanceMatrix *inDistanceMatrix)
{
	writeMatrixData(inDistanceMatrix);
}

