/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 METabParser.cpp

				METabParser Class
	
 ******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "MEExceptions.h"

#include "CReader.h"
#include "MEParser.h"
#include "METabParser.h"
#include "MEBasicSequence.h"
#include "MEBasicAlignment.h"
#include "METreeList.h"
#include "METree.h"
#include "MEDataMatrix.h"


// ---------------------------------------------------------------------------------
//		 METabParser
// ---------------------------------------------------------------------------------

METabParser::METabParser(CReader *inReader) : MEParser(inReader)
{
	reader->setCommentDelimiters('\0', '\0', '\0');
	
	strcpy(delims, "\t\r\n");
	multiDelims=false;
	hasColLabels=true;
	taxonLabelCols=1;
}

// ---------------------------------------------------------------------------------
//		 METabParser::setColumnDelimiters
// ---------------------------------------------------------------------------------

void METabParser::setColumnDelimiters(char *inDelims, bool inMultiDelims)
{
	strncpy(delims, inDelims, 16);
	delims[16]='\0';
	strcat(delims, "\r\n"); // end of line also delimits a column
	multiDelims=inMultiDelims;
}

// ---------------------------------------------------------------------------------
//		 METabParser::setHasColumnLabels
// ---------------------------------------------------------------------------------

void METabParser::setHasColumnLabels(bool inHasColLabels)
{
	hasColLabels=inHasColLabels;
}

// ---------------------------------------------------------------------------------
//		 METabParser::setTaxonLabelColumns
// ---------------------------------------------------------------------------------

void METabParser::setTaxonLabelColumns(int inNumTaxonLabelCols)
{
	TestBounds(inNumTaxonLabelCols, 3);
	taxonLabelCols=inNumTaxonLabelCols;
}

// ---------------------------------------------------------------------------------
//		 METabParser::readDataMatrix
// ---------------------------------------------------------------------------------

void METabParser::readDataMatrix(MEDataMatrix *inDataMatrix)
{
	char str[256], str2[256], ch;
	int numCols=-1;
	int row, col;
	double f;
	
	reader->skipSpace();

	// Read the column names if we have them
	if (hasColLabels) {
		if (taxonLabelCols) {
			reader->readString(str, 127, delims);
			if (taxonLabelCols==2)
				reader->readString(str2, 127, delims);
		}
		
		ch=reader->readString(str, 255, delims);

		inDataMatrix->addColumn(str);
		col=1;
		while (ch!='\r' && ch!='\n') {
			ch=reader->readString(str, 255, delims);

			inDataMatrix->addColumn(str);
			col++;
		}
		reader->skipWhile("\r\n");
		numCols=col;
	}
	
	bool done=false;
	row=0;
	while (!done) {
		try {
			if (taxonLabelCols) {
				reader->readString(str, 127, delims);
				if (taxonLabelCols==2) {
					ch=reader->readString(str2, 127, delims);
						
					strcat(str, " ");
					strcat(str, str2);
				}
				inDataMatrix->addRow(str);
			} else
				inDataMatrix->addRow();
			
			ch=reader->readToken(str, 255, delims);
				
			if (sscanf(str, "%lf", &f)==1)
				inDataMatrix->setElement(row, 0, f);
			else
				inDataMatrix->setMissingElement(row, 0);

			col=1;
			while (ch!='\r' && ch!='\n' && !done) {
				ch=reader->readToken(str, 255, delims);
				
				if (sscanf(str, "%lf", &f)==1)
					inDataMatrix->setElement(row, col, f);
				else
					inDataMatrix->setMissingElement(row, col);
				col++;
			}
			reader->skipWhile("\r\n");
		} catch (CEndOfFile e) {
			done=true;
		}
		if (numCols<0)
			numCols=col;
		else if (numCols!=col)
			throw MEBadFormat("Rows of matrix are different lengths");
			
		row++;
	}				
}
