/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 METaxonList.cpp

				METaxonList Class
				
 ******************************************************************************/

// METaxonList is a simple list of taxon. This class does not add anything
// to MEList but makes the code more obvious. Additional field could be added
// at a later date. A set of getTaxon/addTaxon/insertTaxon/removeTaxon just
// map to equivalent getObject/add/insert/remove functions in the base MEList
// class. 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "MEGlobal.h"
#include "MEExceptions.h"

#include "MEStream.h"
#include "METaxon.h"
#include "MEList.h"
#include "METaxonList.h"

#pragma mark === METaxonList ===

// ---------------------------------------------------------------------------------
//		 METaxonList
// ---------------------------------------------------------------------------------

METaxonList::METaxonList() : MEList() 
{
	taxaDefined=false;
	numHiddenTaxa=0;
}

// ---------------------------------------------------------------------------------
//		 ~METaxonList
// ---------------------------------------------------------------------------------

METaxonList::~METaxonList()
{
}

// ---------------------------------------------------------------------------------
//		 METaxonList::clone
// ---------------------------------------------------------------------------------

void METaxonList::clone(MEObject *src)
{
	MEList::clone(src);
}

// ---------------------------------------------------------------------------------
//		 METaxonList::getNumTaxa
// ---------------------------------------------------------------------------------

int	METaxonList::getNumTaxa()
{
	return numObjects;
}

// ---------------------------------------------------------------------------------
//		METaxonList::createObject
// ---------------------------------------------------------------------------------

MEObject* METaxonList::createObject()
{
	MEObject *obj;
	NewClass(obj, METaxon());
	return obj;
}

// ---------------------------------------------------------------------------------
//		 METaxonList::getTaxon
// ---------------------------------------------------------------------------------

METaxon* METaxonList::getTaxon(int index)
{
	return (METaxon *)getObject(index);
}

// ---------------------------------------------------------------------------------
//		 METaxonList::addTaxon
// ---------------------------------------------------------------------------------

int METaxonList::addTaxon(METaxon *inTaxon)
{
	return add(inTaxon, inTaxon->getOwner()==NULL);
}

// ---------------------------------------------------------------------------------
//		 METaxonList::insertTaxon
// ---------------------------------------------------------------------------------

void METaxonList::insertTaxon(int index, METaxon *inTaxon)
{
	insert(index, inTaxon, inTaxon->getOwner()==NULL);
}

// ---------------------------------------------------------------------------------
//		 METaxonList::removeTaxon
// ---------------------------------------------------------------------------------

METaxon* METaxonList::removeTaxon(int index)
{
	return (METaxon *)remove(index);
}

// ---------------------------------------------------------------------------------
//		 METaxonList::readObject
// ---------------------------------------------------------------------------------

// Just call the MEList base function - there are no additional fields at present.
void METaxonList::readObject(MEStream *stream)
{
	MEList::readObject(stream);
}

// ---------------------------------------------------------------------------------
//		 METaxonList::writeObject
// ---------------------------------------------------------------------------------

// Just call the MEList base function - there are no additional fields at present.
void METaxonList::writeObject(MEStream *stream)
{
	MEList::writeObject(stream);
}


