/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 METaxonList.h

		Interface for the METaxonList Class
					
 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "Basic Sequences"
 
#ifndef _METAXONLIST_H_
#define _METAXONLIST_H_

#include "MEList.h"

class MEStream;
class METaxon;

/******************************************************************************
	class METaxonList
 ******************************************************************************/
#pragma mark METaxonList class

class METaxonList : public MEList {
	public:
		virtual long		getID() const { return 'MTxL'; }
		//: The ID of this object - 'MTxL'.

		virtual char*		getClassName() const { return "Taxon List"; }
		//: Returns the name of this class - "Taxon List"
		
	protected:
		bool				taxaDefined;
		int					numHiddenTaxa;
		
	public:
	 						METaxonList();						
	 	virtual				~METaxonList();
	 	
		virtual void 		clone(MEObject *src);
		
		virtual void		setTaxaDefined(bool inTaxaDefined) { taxaDefined=inTaxaDefined; }
		virtual bool		getTaxaDefined() { return taxaDefined; }
		
		virtual int			getNumTaxa();
		
		virtual MEObject*	createObject();

		virtual METaxon*	getTaxon(int index);
		virtual int			addTaxon(METaxon *inTaxon);
		virtual void		insertTaxon(int index, METaxon *inTaxon);
		virtual METaxon*	removeTaxon(int index);

		virtual char*		getTaxonName(int index) { return getObjectName(index); };
		
		virtual void		readObject(MEStream *stream);
		virtual void		writeObject(MEStream *stream);
};

#endif /* _METAXONLIST_H_ */
