/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 METaxonMatrix.h

		Interface for the METaxonMatrix Class
					
 ******************************************************************************/
//! author = "Andrew Rambaut & Mike Charleston"
//! group = "Matrices"
 
#ifndef _METAXON_MATRIX_H_
#define _METAXON_MATRIX_H_

#include "MEMatrix.h"

class METaxonList;
class METaxon;
class MEIntArray;

class MEStream;

class METaxonMatrix : public MEMatrix {
	public:
		virtual long		getID() const { return 'MTxM'; }
		//: The ID of this object - 'MTxM'.
	
		virtual char*		getClassName() const { return "Taxon Matrix"; }
		//: Returns the name of this class - "Taxon Matrix"
		
	protected:
		METaxonList*		taxonList;
		MEIntArray*			taxonIndices;
		
	public:
	 						METaxonMatrix();						
	 						METaxonMatrix(int inDimension);						
	 						METaxonMatrix(int inNumRows, int inNumCols);	
	 										
	 	virtual				~METaxonMatrix();

		virtual void		setTaxonList(METaxonList *inTaxonList, bool isOwner, bool merge=false);
		virtual METaxonList *getTaxonList() { return taxonList; };
		char*				getTaxonName(int row);
		METaxon*			getTaxon(int row);
		virtual void		setTaxonIndex(int row, int index);
		
		virtual void		createLocalTaxonList();

		virtual int			findRow(char *inName);
		virtual int			findRow(int inTaxonNo);

		virtual MEList		*getRowList();
		virtual MEObject	*getRowObject(int row);

		virtual void		readObject(MEStream *stream);
		virtual void		writeObject(MEStream *stream);
};

#endif /* _METAXON_MATRIX_H_ */
