/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 METreeGenerator.h

		Interface for the METreeGenerator Class
					
 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "Trees"

#ifndef _METREE_GENERATOR_H_
#define _METREE_GENERATOR_H_

#pragma mark // includes:
#include "MEObject.h"

#pragma mark // accessed classes:
class METree;
class MEStream;

#pragma mark // class definition:

class METreeGenerator : public MEObject {
	protected:
		METree			*mSourceTree;
	
	public:
							METreeGenerator();
		virtual			~METreeGenerator();
	
		virtual void	setSourceTree(METree *inTree);
		
		virtual long	getNumTrees();
		virtual double	getApproxNumTrees();
		
	protected:
		long 				countResolutions(int degree);
		double			countApproxResolutions(int degree);

	public:
		virtual void	generate(int index, METree *inTree);
		
		virtual void	generateRandom(METree *inTree);

		virtual void	generateFirst(METree *inTree);
		virtual void	generateNext(METree *inTree);
	
		virtual void	readObject(MEStream *stream);
		//: Reads the fields of this object from an MEStream
		
		virtual void	writeObject(MEStream *stream);
		//: Writes the fields of this object to an MEStream
};


#endif /* _METREE_GENERATOR_H_ */
