/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEUndead.cpp

				MEUndead Class
				
 ******************************************************************************/

// MEUndead provides a static function which will allocated a new instance
// of an MolEvol class based upon an ID code. This is used to resurrect classes
// stored on a stream. Called by MEObject. Derive a class to resurrect more
// objects.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "MEExceptions.h"

#include "MEUndead.h"
#include "MEObject.h"

#include "MEList.h"
#include "MEIntArray.h"
#include "MEDoubleArray.h"
#include "MECodeTable.h"
#include "MECodeTableList.h"
#include "METaxon.h"
#include "METaxonList.h"
#include "MEBasicSequence.h"
#include "MEBasicAlignment.h"
#include "MEBasicAlignmentList.h"
#include "MESequence.h"
#include "MEFeature.h"
#include "MEFeatureList.h"
#include "MEAlignment.h"
#include "MEAlignmentList.h"
#include "MEPatternList.h"
#include "METree.h"
#include "METreeList.h"
#include "MEMatrix.h"
#include "METaxonMatrix.h"
#include "MEDistanceMatrix.h"
#include "MEDataColumn.h"
#include "MEDataMatrix.h"
#include "MEMatrixList.h"
#include "METaxonMatrixList.h"
#include "MEDataMatrixList.h"

#pragma mark === MEUndead ===

// ---------------------------------------------------------------------------------
//		 MEUndead::defaultNewByID
// ---------------------------------------------------------------------------------

// Create a new object of type id. This is static so that it can be called
// without having a parent object. 
MEObject* MEUndead::defaultNewByID(long id)
{
	MEObject *obj;
	
	switch (id) { 
		case 'MObj': NewClass(obj, MEObject()); break;
		case 'MLst': NewClass(obj, MEList()); break;
		case 'MInA': NewClass(obj, MEIntArray()); break;
		case 'MDoA': NewClass(obj, MEDoubleArray()); break;
		case 'MCod': NewClass(obj, MECodeTable()); break;
		case 'MCoL': NewClass(obj, MECodeTableList()); break;
		case 'MTax': NewClass(obj, METaxon()); break;
		case 'MTxL': NewClass(obj, METaxonList()); break;
		case 'MBSq': NewClass(obj, MEBasicSequence()); break;
		case 'MBAl': NewClass(obj, MEBasicAlignment()); break;
		case 'MBAL': NewClass(obj, MEBasicAlignmentList()); break;
		case 'MSeq': NewClass(obj, MESequence()); break;
		case 'MFea': NewClass(obj, MEFeature()); break;
		case 'MFeL': NewClass(obj, MEFeatureList()); break;
		case 'MAli': NewClass(obj, MEAlignment()); break;
		case 'MAlL': NewClass(obj, MEAlignmentList()); break;
		case 'MPat': NewClass(obj, MEPatternList()); break;
		case 'MTre': NewClass(obj, METree()); break;
		case 'MTrL': NewClass(obj, METreeList()); break;
		case 'MMat': NewClass(obj, MEMatrix()); break;
		case 'MTxM': NewClass(obj, METaxonMatrix()); break;
		case 'MDis': NewClass(obj, MEDistanceMatrix()); break;
		case 'MCol': NewClass(obj, MEDataColumn()); break;
		case 'MDat': NewClass(obj, MEDataMatrix()); break;
		case 'MMaL': NewClass(obj, MEMatrixList()); break;
		case 'MTML': NewClass(obj, METaxonMatrixList()); break;
		case 'MDML': NewClass(obj, MEDataMatrixList()); break;
		default:
			throw MEUnknownClass();
	}
	
	return obj;
}

// ---------------------------------------------------------------------------------
//		 MEUndead::canAcceptID
// ---------------------------------------------------------------------------------

bool MEUndead::canAcceptID(long id)
{
	switch (id) { 
		case 'MObj':
		case 'MLst':
		case 'MCod':
		case 'MCoL':
		case 'MTax':
		case 'MTxL':
		case 'MBSq':
		case 'MBAl':
		case 'MBAL':
		case 'MSeq':
		case 'MFea':
		case 'MFeL':
		case 'MAli':
		case 'MAlL':
		case 'MPat':
		case 'MTre':
		case 'MTrL':
		case 'MMat':
		case 'MTxM':
		case 'MDis':
		case 'MCol':
		case 'MDat':
		case 'MMaL':
		case 'MTML':
		case 'MDML':
			return true;
	}
	
	return false;
}

