/******************************************************************************

Molecular Evolution C++ Class Library (MolEvolLib) v1.0
Copyright (C) 1997-2002 Andrew Rambaut and Michael Charleston, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of MolEvolLib C++ Class Library.

    MolEvolLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolEvolLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 MEXMLCreator.h

		Interface for the MEXMLCreator Class
	
 ******************************************************************************/
//! author = "Andrew Rambaut"
//! group = "File Creating"

#ifndef _MEXMLCREATOR_H_
#define _MEXMLCREATOR_H_

#include "MECreator.h"

class CWriter;
class MEObject;
class MEBasicAlignment;
class METaxon;
class METaxonList;
class METree;
class METreeList;
class MEMatrix;
class MEDistanceMatrix;

/******************************************************************************
	class MEXMLCreator
 ******************************************************************************/
#pragma mark MEXMLCreator class

class MEXMLCreator : public MECreator {
		
	protected:
		bool hasHeader;
		
		bool useTaxaElement;
		bool hasTaxaElement;
		
		int indent;
		
	public:
		MEXMLCreator();
		MEXMLCreator(CWriter *inWriter);
	
		virtual ~MEXMLCreator();
		
		virtual void setUseTaxaElement(bool inUseTaxaElement) { useTaxaElement=inUseTaxaElement; }

	protected:
		virtual void writeIndent();
		virtual void writeOpenElement(MEObject *inObject, char *inTag, char *inObjectType);
		virtual void writeCloseElement(char *inTag);
		
		virtual void writeTaxonElement(METaxon *inTaxon);
		virtual void writeTaxaElement(METaxonList *inTaxonList);

		virtual void writeSequenceElement(MEBasicSequence *inSequence, char *inSeqString, int length);
		virtual void writeAlignmentElement(MEBasicAlignment *inAlignment);
		
		virtual void writeNodeElement(METree *inTree, MELink *inNode, double inHeight, double inLength);
		virtual void writeTreeElement(METree *inTree);
		virtual void writeTreesElement(METreeList *inTreeList);

		virtual void writeLineageElement(MEPhylogeny *inPhylogeny, METaxonList *inTaxonList, 
											long inLineage, long &ioTip);
		virtual void writeTreeElement(MEPhylogeny *inPhylogeny, METaxonList *inTaxonList);
		virtual void writeTreesElement(MEPhylogenyList *inPhylogenyList, METaxonList *inTaxonList);
		
		virtual void openHeader();
		
	public:
		virtual void createAlignment(MEBasicAlignment *inAlignment);
		
		virtual void createTree(METree *inTree);
		virtual void createTreeList(METreeList *inTreeList);
		
		virtual void finishCreation();
};

#endif /* _MEXMLCREATOR_H_ */
