/******************************************************************************

Phylogeny Generator (PhyloGen) v1.0
Copyright (C) 2002 Andrew Rambaut, 
Department of Zoology, University of Oxford, UK.

http://evolve.zoo.ox.ac.uk/
mailto:andrew.rambaut@zoo.ox.ac.uk

This file is part of the PhyloGen program.

    PhyloGen is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    PhyloGen is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolEvolLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	phylo-gen.cpp

 ******************************************************************************/

#include <fstream>
#include <iostream>

#include <time.h>
#include <string.h>

#ifdef __MWERKS__ 
#ifdef macintosh 
	#include <SIOUX.h>
	#include <console.h>
#endif
#endif

#include "MEMaths.h"

#include "CRandom.h"
#include "CTextWriter.h"
#include "CTextReporter.h"
#include "CTextProgress.h"

#include "PhyloGenApp.h"

int main(int argc, char **argv)
{
	#ifdef __MWERKS__ 
	#ifdef macintosh
		SIOUXSettings.autocloseonquit = 1;
		SIOUXSettings.columns = 80;
		SIOUXSettings.rows = 50;
	#endif
	#endif

	PhyloGenApp *app = new PhyloGenApp();
	
	CTextWriter *writer = new CTextWriter(&std::cout);
	CTextReporter *reporter = new CTextReporter(writer);
	
	app->setResultsReporter(reporter);
	app->setLogReporter(NULL);
	app->setErrorReporter(reporter);
	app->setProgress(new CTextProgress(&std::cerr, 50));
	
	app->writeTitle();

	MERandom::setSeed(time(NULL));
	CRandom::setSeed(time(NULL));
	
	std::cout << "seed = " << CRandom::getSeed() << std::endl;

	if (argc > 1) {
		char cmd[256] = "RUN ";
		strcat(cmd, argv[1]);
		strcat(cmd, ";");
		app->processCommand(cmd, true);
	} 
	 
	app->run();
	
	delete app;
}
		
